Attribute VB_Name = "GridFunctions"
' GridFunctions:
' Jenness Enterprises
' http://www.jennessent.com
' jeffj@jennessent.com
'--------------------------------------
' AddFieldToVAT - GIVEN A RASTER, FIELDNAME, FIELD TYPE AND FIELD LENGTH, ADDS THE FIELD TO THE VAT.  IF NOT POSSIBLE
'                 (MAYBE USING FLOATING POINT GRID) THEN RETURNS NOTHING. OTHERWISE RETURNS TABLE.
' CalcGridLine - GIVEN pStartPolygon, pEndPolygon, pCorPolygon (FOR BOUNDARY REGION),
'                  pCorRaster As IRaster (FOR COST-DISTANCE), pEnv (IRasterAnalysisEnvironment),
'                  AND OPTIONAL ShouldClean, RETURNS AN IPolyline REPRESENTING LINE FROM START POLYGON TO END POLYGON
' CellValue - GIVEN A POINT AND RASTER, RETURNS CELL VALUE AS VARIANT.  MIGHT NEED TO MODIFY LATER FOR VERY LARGE GRIDS!
' CellValues - GIVEN A POINT COLLECTION AND RASTER, RETURNS CELL VALUES AS VARIANT ARRAY.  MIGHT NEED TO MODIFY LATER FOR VERY LARGE GRIDS!
' ClipRasterToPolygon - GIVEN pRaster As IRaster, pPolygon As IPolygon, SaveInside As Boolean,
'                  OPTIONAL pClipEnvelope, OPTIONAL CellSize, OPTIONAL pEnv (IRasterAnalysisEnvironment), RETURNS IRaster
' CompareSpatialReferences - GIVEN TWO SPATIAL REFERENCES, RETURNS BOOLEAN
' DistributePointsAlongShape - GIVEN ICurve AND SEPARATION DISTANCE, RETURNS A MULTIPOINT OF POINTS ALONG THAT SHAPE
' EllipticArcToPolygon - GIVEN AN ISegmentCollection AND NumVertices, RETURNS Polygon4 BASED ON CONVEX HULL AROUND VERTICES
' EllipticArcToPolygon2 - GIVEN AN ISegmentCollection AND NumVertices, RETURNS IMultipoint
' OpenRasterWorkspace - GIVEN sPath, RETURNS A RASTER WORKSPACE
' ReturnCellCount - GIVEN A RASTER, RETURNS COUNT OF NON-NULL CELLS
' ReturnCellSize - GIVEN A RASTER, RETURNS CELL SIZE BASED ON Y-DIMENSION
' ReturnPixelHeight - GIVEN A RASTER, RETURNS PIXEL HEIGHT (Y-DIMENSION)
' ReturnPixelWidth - GIVEN A RASTER, RETURNS PIXEL WIDTH (X-DIMENSION)
' ReturnPointsByCellSize - GIVEN A RASTER AND LINE, RETURNS IPointCollection WITH POINTS DISTRIBUTED APPROXIMATELY EQUAL TO
'                  CELL SIZE
' SetSpatialAnalysisSettings - GIVEN TargetEnv AND SourceEnv, ASSIGNS PROPERTIES OF SourceEnv TO TargetEnv
' SaveRasterAs -  GIVEN A RASTER BAND COLLECTION, FOLDER PATH, RASTER FILENAME, AND RASTER TYPE, SAVES RASTER AS PERMANENT FILE

Public Enum enumRasterType
   enum_Grid_Type
   enum_Imagine_Type
   enum_TIFF_Type
   enum_JPEG_Type
   enum_JP2000_Type
   enum_BMP_Type
   enum_PNG_Type
   enum_GIF_Type
   enum_PCI_Raster_Type
   enum_X11_Pixmap_Type
   enum_PCRaster_Type
   enum_Memory_Raster_Type
   enum_HDF4_Type
   enum_BIL_Type
   enum_BIP_Type
   enum_BSQ_Type
   enum_IDRISI_Type
   enum_Geodatabase_Type
End Enum

Option Explicit
Const c_sModuleFileName As String = "D:\arcGIS_stuff\general_tools\Registration\GridFunctions.bas"


Public Function OpenRasterWorkspace(sPath As String) As IRasterWorkspace
  On Error GoTo ErrorHandler

  Dim pWKSF As IWorkspaceFactory
58:   Set pWKSF = New RasterWorkspaceFactory

  Dim pRasterWs As IRasterWorkspace
61:   Set pRasterWs = pWKSF.OpenFromFile(sPath, 0)
62:   Set OpenRasterWorkspace = pRasterWs

  Exit Function
ErrorHandler:
  HandleError True, "OpenRasterWorkspace " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Sub SaveRasterAs(pRasterBandCol As IRasterBandCollection, strPath As String, strName As String, aRasterType As enumRasterType)
  On Error GoTo ErrorHandler
'  MsgBox "GridFunctions - Before: " & strPath
'  strPath = Linkages.aml_func_mod.BasicTrimAvenue(strPath, "", "/\")
'  MsgBox "GridFunctions - After: " & strPath
  
  Dim pSaveAs As IRasterBandCollection
75:   Set pSaveAs = pRasterBandCol
  Dim pRasterWs As IRasterWorkspace
77:   Set pRasterWs = OpenRasterWorkspace(strPath)
78:   If aRasterType = enum_Grid_Type Then
79:     pSaveAs.SaveAs strName, pRasterWs, "GRID"
80:   ElseIf aRasterType = enum_Imagine_Type Then
81:     pSaveAs.SaveAs strName, pRasterWs, "IMAGINE Image"
82:   ElseIf aRasterType = enum_TIFF_Type Then
83:     pSaveAs.SaveAs strName, pRasterWs, "TIFF"
84:   ElseIf aRasterType = enum_JPEG_Type Then
85:     pSaveAs.SaveAs strName, pRasterWs, "JPG"
86:   ElseIf aRasterType = enum_JP2000_Type Then
87:     pSaveAs.SaveAs strName, pRasterWs, "JP2"
88:   ElseIf aRasterType = enum_BMP_Type Then
89:     pSaveAs.SaveAs strName, pRasterWs, "BMP"
90:   ElseIf aRasterType = enum_PNG_Type Then
91:     pSaveAs.SaveAs strName, pRasterWs, "PNG"
92:   ElseIf aRasterType = enum_GIF_Type Then
93:     pSaveAs.SaveAs strName, pRasterWs, "GIF"
94:   ElseIf aRasterType = enum_PCI_Raster_Type Then
95:     pSaveAs.SaveAs strName, pRasterWs, "PIX"
96:   ElseIf aRasterType = enum_X11_Pixmap_Type Then
97:     pSaveAs.SaveAs strName, pRasterWs, "XPM"
98:   ElseIf aRasterType = enum_PCRaster_Type Then
99:     pSaveAs.SaveAs strName, pRasterWs, "MAP"
100:   ElseIf aRasterType = enum_Memory_Raster_Type Then
101:     pSaveAs.SaveAs strName, pRasterWs, "MEM"
102:   ElseIf aRasterType = enum_HDF4_Type Then
103:     pSaveAs.SaveAs strName, pRasterWs, "HDF4"
104:   ElseIf aRasterType = enum_BIL_Type Then
105:     pSaveAs.SaveAs strName, pRasterWs, "BIL"
106:   ElseIf aRasterType = enum_BIP_Type Then
107:     pSaveAs.SaveAs strName, pRasterWs, "BIP"
108:   ElseIf aRasterType = enum_BSQ_Type Then
109:     pSaveAs.SaveAs strName, pRasterWs, "BSQ"
110:   ElseIf aRasterType = enum_IDRISI_Type Then
111:     pSaveAs.SaveAs strName, pRasterWs, "GDB"
112:   ElseIf aRasterType = enum_Geodatabase_Type Then
113:     pSaveAs.SaveAs strName, pRasterWs, "GDB"
114:   End If
  

  Exit Sub
ErrorHandler:
  HandleError True, "SaveRasterAs " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function ReturnCellSize(pRaster As IRaster) As Double
  On Error GoTo ERH
  Dim pRasLayer As IRasterLayer
124:   Set pRasLayer = New RasterLayer
125:   pRasLayer.CreateFromRaster pRaster
  
  Dim pRasterProps As IRasterProps
128:   Set pRasterProps = pRaster
  
  Dim lngNumRows As Long
131:   lngNumRows = pRasLayer.RowCount
  
  Dim pEnvelope As IEnvelope
134:   Set pEnvelope = pRasterProps.Extent
  
136:   ReturnCellSize = pEnvelope.Height / lngNumRows
   
'  Debug.Print "By Rows = " & ReturnCellSize & ",   By Columns = " & pEnvelope.Width / pRasLayer.ColumnCount
  
  Exit Function
ERH:
142:     MsgBox "Failed in ReturnCellSize: " & err.Description

End Function
Public Function ReturnPixelHeight(pRaster As IRaster) As Double
  On Error GoTo ERH
  Dim pRasLayer As IRasterLayer
148:   Set pRasLayer = New RasterLayer
149:   pRasLayer.CreateFromRaster pRaster
  
  Dim pRasterProps As IRasterProps
152:   Set pRasterProps = pRaster
  
  Dim lngNumRows As Long
155:   lngNumRows = pRasLayer.RowCount
  
  Dim pEnvelope As IEnvelope
158:   Set pEnvelope = pRasterProps.Extent
  
160:   ReturnPixelHeight = pEnvelope.Height / lngNumRows
   
'  Debug.Print "By Rows = " & ReturnCellSize & ",   By Columns = " & pEnvelope.Width / pRasLayer.ColumnCount
  
  Exit Function
ERH:
  HandleError True, "ReturnPixelHeight " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4


End Function

Public Function ReturnCellCount(pRaster As IRaster) As Long
  On Error GoTo ErrorHandler

  




  Exit Function
ErrorHandler:
  HandleError True, "ReturnCellCount " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcContinuousGridStats(pRaster As IRaster, pRasterStats As IRasterStatistics, _
      lngNumBins As Long) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

  
  Dim pRasterAnalysisProps As IRasterAnalysisProps
  Dim pRasterProps As IRasterProps
191:   Set pRasterAnalysisProps = pRaster
192:   Set pRasterProps = pRaster
  
  Dim dblMaximum As Double
  Dim dblMinimum As Double
  Dim dblMean As Double
  Dim dblMedian As Double
  Dim dblMode As Double
  Dim dblStDev As Double
  
  




  Exit Function
ErrorHandler:
  HandleError True, "CalcContinuousGridStats " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnPixelWidth(pRaster As IRaster) As Double
  On Error GoTo ERH
  Dim pRasLayer As IRasterLayer
214:   Set pRasLayer = New RasterLayer
215:   pRasLayer.CreateFromRaster pRaster
  
  Dim pRasterProps As IRasterProps
218:   Set pRasterProps = pRaster
  
  Dim lngNumCols As Long
221:   lngNumCols = pRasLayer.ColumnCount
  
  Dim pEnvelope As IEnvelope
224:   Set pEnvelope = pRasterProps.Extent
  
226:   ReturnPixelWidth = pEnvelope.Width / lngNumCols
   
'  Debug.Print "By Rows = " & ReturnCellSize & ",   By Columns = " & pEnvelope.Width / pRasLayer.ColumnCount
  
  Exit Function
ERH:
  HandleError True, "ReturnPixelWidth " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4

End Function

Public Function ReturnPointsByCellSize(pRaster As IRaster, ByVal pLine As IGeometry) As IPointCollection
  On Error GoTo ERH
  
  Dim pCurve As ICurve
  Dim dblLength As Double
  
242:   If TypeOf pLine Is ICurve Then
243:     Set pCurve = pLine
244:     dblLength = pCurve.length
245:   Else
246:     MsgBox "Invalid geometry type!  Must implement 'ICurve'..."
247:     Set ReturnPointsByCellSize = Nothing
    Exit Function
249:   End If
  Dim pPointCollection As IPointCollection
  
  ' CHECK SPATIAL REFERENCES; MIGHT NEED TO PROJECT POLYLINE
  Dim pSrcSpRef As ISpatialReference
  Dim pRasProps As IRasterProps
255:   Set pRasProps = pRaster
256:   Set pSrcSpRef = pRasProps.SpatialReference
  
  Dim pTrgSpRef As ISpatialReference
259:   Set pTrgSpRef = pLine.SpatialReference
  
261:   If Not GridFunctions.CompareSpatialReferences(pSrcSpRef, pTrgSpRef) Then
262:     pLine.Project pSrcSpRef
263:   End If
  
  ' GET GRID CELL SIZE
  Dim dblCellSize As Double
267:   dblCellSize = GridFunctions.ReturnCellSize(pRaster)
  Dim NumPoints As Long
269:   NumPoints = Int(dblLength / dblCellSize) + 1
  
  Dim pMpt As IMultipoint
  Dim pSegCol As ISegmentCollection
273:   Set pSegCol = pLine
274:   Set pMpt = GridFunctions.EllipticArcToPolygon2(pSegCol, NumPoints)
275:   Set pPointCollection = pMpt
  
277:   Set ReturnPointsByCellSize = pPointCollection
    
  Exit Function
ERH:
281:   If (Erl <> 0) Then
282:     MsgBox "Failed in ReturnPointsByCellSize: " & err.Description & vbCrLf & "Error at line " & CStr(Erl)
283:   Else
284:     MsgBox "Failed in ReturnPointsByCellSize: " & err.Description & vbCrLf & "No Line Number Available..."
285:   End If

End Function
Public Function CompareSpatialReferences(ByVal pSourceSR As ISpatialReference, ByVal pTargetSR As ISpatialReference) As Boolean
  
  On Error GoTo ERH
  
292:   If pSourceSR Is Nothing And pTargetSR Is Nothing Then
293:     CompareSpatialReferences = True
    Exit Function
295:   ElseIf pSourceSR Is Nothing Or pTargetSR Is Nothing Then
296:     CompareSpatialReferences = False
    Exit Function
298:   End If
  
  Dim pSourceClone As IClone
  Dim pTargetClone As IClone
  Dim bSREqual As Boolean
  
304:   Set pSourceClone = pSourceSR
305:   Set pTargetClone = pTargetSR
  
  'Compare the coordinate system component of the spatial reference
308:   bSREqual = pSourceClone.IsEqual(pTargetClone)
  
  'If the comparison failed, return false and exit
311:   If Not bSREqual Then
312:     CompareSpatialReferences = False
    Exit Function
314:   End If
  
  'We can also compare the XY precision to ensure the spatial references are equal
  Dim pSourceSR2 As ISpatialReference2
  Dim bXYIsEqual As Boolean
  
320:   Set pSourceSR2 = pSourceSR
321:   bXYIsEqual = pSourceSR2.IsXYPrecisionEqual(pTargetSR)
  
  'If the comparison failed, return false and exit
324:   If Not bXYIsEqual Then
325:     CompareSpatialReferences = False
    Exit Function
327:   End If
  
329:   CompareSpatialReferences = True
  Exit Function
ERH:
332:     MsgBox "Failed in CompareSpatialReferences: " & err.Description
End Function
Sub SetSpatialAnalysisSettings(TargetEnv As IRasterAnalysisEnvironment, _
                               SourceEnv As IRasterAnalysisEnvironment)
    On Error GoTo ERH
337:     If Not SourceEnv Is Nothing Then
338:         Set TargetEnv.OutWorkspace = SourceEnv.OutWorkspace
339:         If Not SourceEnv.OutSpatialReference Is Nothing Then
340:             Set TargetEnv.OutSpatialReference = SourceEnv.OutSpatialReference
341:         End If
342:         TargetEnv.DefaultOutputRasterPrefix = SourceEnv.DefaultOutputRasterPrefix
343:         TargetEnv.DefaultOutputVectorPrefix = SourceEnv.DefaultOutputVectorPrefix
344:         If Not SourceEnv.Mask Is Nothing Then
345:             Set TargetEnv.Mask = SourceEnv.Mask
346:         End If
        Dim nCellSize As Double
348:         SourceEnv.GetCellSize 3, nCellSize
349:         If nCellSize <> 0 Then
350:             TargetEnv.SetCellSize 3, nCellSize
351:         End If
        Dim pExtent As IEnvelope
353:         SourceEnv.GetExtent 3, pExtent
354:         If Not pExtent Is Nothing Then
355:             TargetEnv.SetExtent 3, pExtent
356:         End If
357:         TargetEnv.VerifyType = SourceEnv.VerifyType
358:     End If
    Exit Sub
ERH:
361:     MsgBox "Failed in SetSpatialAnalysisSettings: " & err.Description
End Sub

Public Function ClipRasterToPolygon(pRaster As IRaster, ByVal pPolygon As IPolygon, SaveInside As Boolean, _
      Optional ByVal pClipEnvelope As IEnvelope, Optional CellSize As Double, _
      Optional ByVal pEnv As IRasterAnalysisEnvironment, Optional booShowProgress As Boolean, _
      Optional pApp As IApplication) As IRaster
      
  On Error GoTo ErrorHandler

  
'  If pPolygon.IsEmpty Then
'    Dim xx As Long
'    xx = 1
'  End If
  
    ' PROGRESS BAR STUFF
378:   If booShowProgress Then
    Dim psbar As IStatusBar
380:     Set psbar = pApp.StatusBar
381:     psbar.ProgressBar.position = 1
    Dim pPro As IStepProgressor
383:     Set pPro = psbar.ProgressBar
384:   End If
  
'  MsgBox "Hello 1"
  
  ' FORCE OBJECT TO BE POLYGON IF THE FIRST SEGMENT IS A CURVE
  Dim booWorkingWithCurves As Boolean
  Dim pSegmentCollectionCurves As ISegmentCollection
'  Dim pSegTopoOp As ITopologicalOperator
'  Set pSegTopoOp = pPolygon
'  Dim pCheckSegLine As IPolyline
'  Set pCheckSegLine = pSegTopoOp.Boundary
395:   Set pSegmentCollectionCurves = pPolygon
  Dim pSegmentCurve As ISegment
397:   Set pSegmentCurve = pSegmentCollectionCurves.Segment(0)
  Dim pGeometryTypeA As esriGeometryType
399:   pGeometryTypeA = pSegmentCurve.GeometryType
  
401:   booWorkingWithCurves = (pGeometryTypeA = esriGeometryBezier3Curve) Or _
    (pGeometryTypeA = esriGeometryCircularArc) Or _
    (pGeometryTypeA = esriGeometryEllipticArc)
  
405:   If booWorkingWithCurves Then
    Dim pNewPoints As IPointCollection
407:     Set pNewPoints = GridFunctions.EllipticArcToPolygon2(pSegmentCollectionCurves, 100)
    Dim pNewPolygon As IPointCollection
409:     Set pNewPolygon = New Polygon
410:     pNewPolygon.SetPointCollection pNewPoints
411:     Set pPolygon = pNewPolygon
412:   End If
    
  
'  MsgBox "Hello 2"
  
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
  ' MAKE EXTRACTION OPERATOR
  Dim pExtractionOp As IExtractionOp
422:   Set pExtractionOp = New RasterExtractionOp
  
  Dim pRastAnalysisEnv As IRasterAnalysisEnvironment
  Dim pSpatialReference As ISpatialReference
  Dim pPolySpatRef As ISpatialReference
427:   Set pPolySpatRef = pPolygon.SpatialReference
  Dim SpatRefSame As Boolean
  Dim pEnvelope As IEnvelope
  
431:   If Not pEnv Is Nothing Then
432:     Set pRastAnalysisEnv = pEnv
433:     Set pSpatialReference = pRastAnalysisEnv.OutSpatialReference
    
    ' CHECK SPATIAL REFERENCE OF INCOMING POLYGON
436:     SpatRefSame = CompareSpatialReferences(pSpatialReference, pPolySpatRef)
437:     If Not SpatRefSame Then pPolygon.Project pSpatialReference
438:     pEnv.GetExtent esriRasterEnvValue, pEnvelope
    
440:   Else
    
    ' MAKE ANALYSIS ENVIRONMENT
443:     Set pRastAnalysisEnv = pExtractionOp
444:     pRastAnalysisEnv.RestoreToPreviousDefaultEnvironment
    
    ' ASSIGN CURRENT ANALYSIS ENVIRONMENT SETTINGS
    Dim theEnvType As esriRasterEnvSettingEnum
    Dim theExtEnvType As esriRasterEnvSettingEnum
    Dim pTempEnv As IEnvelope
    Dim theCellSize As Double
451:     pRastAnalysisEnv.GetCellSize theEnvType, theCellSize
452:     pRastAnalysisEnv.GetExtent theExtEnvType, pTempEnv
    Dim pRasterAnalysisProps As IRasterAnalysisProps
    Dim pRasterProps As IRasterProps
455:         Set pRasterAnalysisProps = pRaster
456:         Set pRasterProps = pRaster
    
    ' DETERMINE OUTPUT SPATIAL REFERENCE BASED ON INPUT RASTER
    
460:     Set pSpatialReference = pRasterProps.SpatialReference
    
    ' CHECK SPATIAL REFERENCE OF INCOMING POLYGON
463:     SpatRefSame = CompareSpatialReferences(pSpatialReference, pPolySpatRef)
464:     If Not SpatRefSame Then pPolygon.Project pSpatialReference
    
    ' DETERMINE ANALYSIS ENVIRONMENT TYPE AND CELL SIZE
467:     If CellSize <= 0 Then
468:       theCellSize = pRasterAnalysisProps.PixelHeight
469:       theEnvType = pRastAnalysisEnv.VerifyType
470:     Else
471:       theCellSize = CellSize
472:       theEnvType = esriRasterEnvValue
473:     End If
    
    Dim pTopoOp As ITopologicalOperator2
476:     If pClipEnvelope Is Nothing Then
      Dim pPolyEnvelope As IEnvelope
478:           Set pPolyEnvelope = pPolygon.Envelope
      Dim pRastEnvelope As IEnvelope
480:           Set pRastEnvelope = pRasterProps.Extent
481:           Set pTopoOp = pPolyEnvelope
482:           pTopoOp.IsKnownSimple = False
483:           pTopoOp.Simplify
      
485:       Set pEnvelope = pTopoOp.Intersect(pPolyEnvelope, esriGeometry2Dimension)
486:     Else
      ' CHECK SPATIAL REFERENCE OF ENVELOPE
488:       SpatRefSame = CompareSpatialReferences(pSpatialReference, pClipEnvelope.SpatialReference)
489:       If Not SpatRefSame Then
490:         pClipEnvelope.Project pSpatialReference
491:       End If
492:       Set pEnvelope = pClipEnvelope.Envelope
493:     End If
    
    ' SET ANALYSIS ENVIRONMENT PROPERTIES
496:     pRastAnalysisEnv.SetCellSize theEnvType, theCellSize
497:     pRastAnalysisEnv.SetExtent esriRasterEnvValue, pEnvelope
498:     Set pRastAnalysisEnv.OutSpatialReference = pSpatialReference
499:   End If
  
  
'  MsgBox "Hello 3"
503:   DoEvents
  
  ' CLIP INCOMING POLYGON TO ANALYSIS AREA; MIGHT HELP AVOID PROBLEMS WITH MULTIPART POLYGONS, HOLES, ETC.
  Dim pIntPolygon As IPolygon4
507:   Set pTopoOp = pPolygon
508:   pTopoOp.IsKnownSimple = False
509:   pTopoOp.Simplify
510:   Set pIntPolygon = pTopoOp.Intersect(pEnvelope, esriGeometry2Dimension)
511:   Set pIntPolygon = pPolygon
512:   Set pIntPolygon.SpatialReference = pPolygon.SpatialReference
  
'  ThisDocument.Graphic_MakeFromGeometry pMxDoc, pIntPolygon, "TestClipPolyGraphics"
  
  
517:   Set pTopoOp = pIntPolygon
518:   pTopoOp.IsKnownSimple = False
519:   pTopoOp.Simplify
  
  Dim pGeometryCollection As IGeometryCollection
  Dim pExtRing As IGeometryCollection
  Dim pIntRingBag As IGeometryCollection
524:   Set pGeometryCollection = pIntPolygon.ConnectedComponentBag
  Dim pIntGeoCol As IGeometryCollection
  Dim pIntPoly As IPolygon4
  Dim pOutGeoCol As IGeometryCollection
  Dim pOutPoly As IPolygon4
  
  Dim pSubPoly As IPolygon4
  Dim pSubRing As IRing
  Dim anIndex As Long
  Dim anIndex2 As Long
  
  Dim pClipRaster As IRaster
  Dim pOuterRaster As IRaster
  Dim pInnerRaster As IRaster
  
  Dim pNegativeGeometryCollection As IGeometryCollection
  Dim anIndex3 As Long
  Dim pNegPoly As IPolygon4
  
  ' MAKE A BUNCH OF NEW RASTER OPERATORS AND ASSIGN THE CURRENT ANALYSIS ENVIRONMENT
  Dim pRasMakerOp As IRasterMakerOp
545:   Set pRasMakerOp = New RasterMakerOp
546:   SetSpatialAnalysisSettings pRasMakerOp, pRastAnalysisEnv
  Dim pCondOp As IConditionalOp
548:   Set pCondOp = New RasterConditionalOp
549:   SetSpatialAnalysisSettings pCondOp, pRastAnalysisEnv
  Dim pLogicOp As ILogicalOp
551:   Set pLogicOp = New RasterMathOps
552:   SetSpatialAnalysisSettings pLogicOp, pRastAnalysisEnv
  Dim pMathOp As IMathOp
554:   Set pMathOp = New RasterMathOps
555:   SetSpatialAnalysisSettings pMathOp, pRastAnalysisEnv
  
  Dim pRasLayer As IRasterLayer
  Dim pTestGeometry As IGeometry
  Dim pTestGeoColl As IGeometryCollection
  Dim pSegmentCollection1 As ISegmentCollection
  Dim pSegment1 As ISegment
  Dim pGeometryType As esriGeometryType
  Dim pEllArcPolygon As IPolygon4
  
  Dim pFinalGrid As IRaster
566:   Set pFinalGrid = pRasMakerOp.MakeConstant(0, True)
567:   Set pClipRaster = pRasMakerOp.MakeConstant(1, True)
  
569:   If booShowProgress Then
570:     pPro.MaxRange = pGeometryCollection.GeometryCount + 2
571:     pPro.StepValue = 1
572:     pPro.Show
573:   End If
  
575:   DoEvents
'  MsgBox "Hello 4"
  
578:   For anIndex = 0 To pGeometryCollection.GeometryCount - 1

580:     Set pSubPoly = pGeometryCollection.Geometry(anIndex)
581:     Set pExtRing = pSubPoly.ExteriorRingBag
582:     Set pSubRing = pExtRing.Geometry(0)
583:     Set pOutGeoCol = New Polygon
584:     pOutGeoCol.AddGeometry pSubRing
585:     Set pOutPoly = pOutGeoCol
586:     Set pTopoOp = pOutPoly
587:     pTopoOp.IsKnownSimple = False
588:     pTopoOp.Simplify
    
'    Debug.Print anIndex & ":  Outer ring of Polygon is Circle?  " & CStr(TypeOf pOutPoly Is ICircularArc)

592:     Set pSegmentCollection1 = pOutPoly
593:     Set pSegment1 = pSegmentCollection1.Segment(0)
594:     pGeometryType = pSegment1.GeometryType
    
596:     If pGeometryType = esriGeometryCircularArc Then
597:       Set pOuterRaster = pExtractionOp.Circle(pClipRaster, pSegment1, Not SaveInside)
598:     ElseIf pGeometryType = esriGeometryEllipticArc Then
599:       Set pEllArcPolygon = EllipticArcToPolygon(pSegmentCollection1, 75)
600:       Set pOuterRaster = pExtractionOp.Polygon(pClipRaster, pEllArcPolygon, Not SaveInside)
601:     ElseIf pGeometryType = esriGeometryEnvelope Then
602:       Set pOuterRaster = pExtractionOp.Rectangle(pClipRaster, pOutPoly, Not SaveInside)
603:     Else
604:       Set pOuterRaster = pExtractionOp.Polygon(pClipRaster, pOutPoly, Not SaveInside)
605:     End If
606:     Set pOuterRaster = pLogicOp.IsNull(pOuterRaster)

608:     If pSubPoly.InteriorRingCount(pSubRing) > 0 Then
609:       Set pIntRingBag = pSubPoly.InteriorRingBag(pSubRing)
   
611:       For anIndex2 = 0 To pIntRingBag.GeometryCount - 1
612:         Set pIntGeoCol = New Polygon
613:         pIntGeoCol.AddGeometry pIntRingBag.Geometry(anIndex2)
614:         Set pIntPoly = pIntGeoCol
615:         Set pTopoOp = pIntPoly
616:         pTopoOp.IsKnownSimple = False
617:         pTopoOp.Simplify

619:         Set pSegmentCollection1 = pIntPoly
620:         Set pSegment1 = pSegmentCollection1.Segment(0)
621:         pGeometryType = pSegment1.GeometryType
                   
       ' CHECK FOR UNUSUAL SHAPES
624:         If pGeometryType = esriGeometryCircularArc Then
625:           Set pInnerRaster = pExtractionOp.Circle(pClipRaster, pSegment1, SaveInside)
626:           Set pInnerRaster = pLogicOp.IsNull(pInnerRaster)
627:           Set pOuterRaster = pMathOp.Times(pInnerRaster, pOuterRaster)
628:         ElseIf pGeometryType = esriGeometryEllipticArc Then
629:           Set pEllArcPolygon = EllipticArcToPolygon(pSegmentCollection1, 75)
630:           Set pInnerRaster = pExtractionOp.Polygon(pClipRaster, pEllArcPolygon, SaveInside)
631:           Set pInnerRaster = pLogicOp.IsNull(pInnerRaster)
632:           Set pOuterRaster = pMathOp.Times(pInnerRaster, pOuterRaster)
633:         ElseIf pGeometryType = esriGeometryEnvelope Then
634:           Set pInnerRaster = pExtractionOp.Rectangle(pClipRaster, pIntPoly, SaveInside)
635:           Set pInnerRaster = pLogicOp.IsNull(pInnerRaster)
636:           Set pOuterRaster = pMathOp.Times(pInnerRaster, pOuterRaster)
637:         Else
638:           Set pNegativeGeometryCollection = pIntPoly.ConnectedComponentBag
639:           For anIndex3 = 0 To pNegativeGeometryCollection.GeometryCount - 1
640:             Set pNegPoly = pNegativeGeometryCollection.Geometry(anIndex3)
641:             Set pTopoOp = pNegPoly
642:             pTopoOp.IsKnownSimple = False
643:             pTopoOp.Simplify
644:             Set pInnerRaster = pExtractionOp.Polygon(pClipRaster, pNegPoly, SaveInside)
645:             Set pInnerRaster = pLogicOp.IsNull(pInnerRaster)
646:             Set pOuterRaster = pMathOp.Times(pInnerRaster, pOuterRaster)
647:           Next anIndex3
648:         End If
        
'        ' CHECK FOR UNUSUAL SHAPES
'        If pGeometryType = esriGeometryCircularArc Then
'          Set pInnerRaster = pExtractionOp.Circle(pClipRaster, pSegment1, SaveInside)
'        ElseIf pGeometryType = esriGeometryEllipticArc Then
'          Set pEllArcPolygon = EllipticArcToPolygon(pSegmentCollection1, 75)
'          Set pInnerRaster = pExtractionOp.Polygon(pClipRaster, pEllArcPolygon, SaveInside)
'        ElseIf pGeometryType = esriGeometryEnvelope Then
'          Set pInnerRaster = pExtractionOp.Rectangle(pClipRaster, pIntPoly, SaveInside)
'        Else
'          Set pInnerRaster = pExtractionOp.Polygon(pClipRaster, pIntPoly, SaveInside)
'        End If
'        Set pInnerRaster = pLogicOp.IsNull(pInnerRaster)
'        Set pOuterRaster = pMathOp.Times(pInnerRaster, pOuterRaster)
        
        
665:         DoEvents
666:       Next anIndex2
    
668:     End If
    
670:     Set pFinalGrid = pMathOp.Plus(pFinalGrid, pOuterRaster)
671:     If booShowProgress Then
672:       pPro.Step
673:     End If
674:     DoEvents
675:   Next anIndex
  
  ' FOR DEBUGGING
  
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
'  Dim pMap As IMap
'  Set pMap = pMxDoc.FocusMap
'  Set pRasLayer = New RasterLayer
'  pRasLayer.CreateFromRaster pFinalGrid
'  pMap.AddLayer pRasLayer
 
687:   Set pFinalGrid = pCondOp.SetNull(pLogicOp.EqualTo(pFinalGrid, pRasMakerOp.MakeConstant(0, True)), pFinalGrid)
688:   If booShowProgress Then
689:     pPro.Step
690:   End If
691:   Set ClipRasterToPolygon = pMathOp.Times(pFinalGrid, pRaster)
692:   If booShowProgress Then
693:     pPro.Step
694:   End If
  
696:   DoEvents
  
  ' RESET ANALYSIS ENVIRONMENT TO PREVIOUS STATE
699:   pRastAnalysisEnv.RestoreToPreviousDefaultEnvironment
  
701:   If booShowProgress Then
702:     pPro.Hide
703:   End If

  Exit Function
ErrorHandler:
  HandleError True, "ClipRasterToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function EllipticArcToPolygon(SegCollection As ISegmentCollection, NumVertices As Long) As IPolygon4
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
      
On Error GoTo ERH

  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
724:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
726:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
730:   For anIndex = 0 To lngSegCount
731:     theTestLength = SegCollection.Segment(anIndex).length
732:     theLength = theLength + theTestLength
733:     lngLengths(anIndex) = theTestLength
734:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
740:   For anIndex = 0 To lngSegCount
741:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
742:     If lngNumVertices < 8 Then lngNumVertices = 8
743:     lngVertices(anIndex) = lngNumVertices
744:   Next anIndex
  
  Dim pMpt As IPointCollection
747:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
749:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
755:   For anIndex = 0 To lngSegCount
756:     lngNumVertices = lngVertices(anIndex)
757:     pRatio = 1 / lngNumVertices
758:     Set pCurve = SegCollection.Segment(anIndex)
    
760:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
762:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
763:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
767:       pMpt.AddPoint pClone.Clone
768:     Next anIndex2
769:   Next anIndex
  
  Dim pPoly4 As IPolygon4
  Dim pTopoOp2 As ITopologicalOperator2
  Dim pTopoOp3 As ITopologicalOperator3
774:   Set pTopoOp2 = pMpt
775:   Set pPoly4 = pTopoOp2.ConvexHull
776:   Set pTopoOp3 = pPoly4
777:   pTopoOp3.IsKnownSimple = False
778:   pTopoOp3.Simplify
  
780:   Set EllipticArcToPolygon = pPoly4
    Exit Function
  
ERH:
784:     MsgBox "Failed in EllipticArcToPolygon: " & err.Description
End Function

Public Function EllipticArcToPolygon2(SegCollection As ISegmentCollection, NumVertices As Long) As IMultipoint
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
      
On Error GoTo ERH

  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
800:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
802:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
806:   For anIndex = 0 To lngSegCount
807:     theTestLength = SegCollection.Segment(anIndex).length
808:     theLength = theLength + theTestLength
809:     lngLengths(anIndex) = theTestLength
810:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
816:   For anIndex = 0 To lngSegCount
817:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
818:     If lngNumVertices < 8 Then lngNumVertices = 8
819:     lngVertices(anIndex) = lngNumVertices
820:   Next anIndex
  
  Dim pMpt As IPointCollection
823:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
825:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
831:   For anIndex = 0 To lngSegCount
832:     lngNumVertices = lngVertices(anIndex)
833:     pRatio = 1 / lngNumVertices
834:     Set pCurve = SegCollection.Segment(anIndex)
    
836:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
838:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
839:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
843:       pMpt.AddPoint pClone.Clone
844:     Next anIndex2
845:   Next anIndex
  
847:   Set EllipticArcToPolygon2 = pMpt
    Exit Function
  
ERH:
851:     MsgBox "Failed in EllipticArcToPolygon2: " & err.Description
End Function

Public Function DistributePointsAlongShape(pCurve As ICurve, SeparationDistance As Double) As IPointCollection
      
On Error GoTo ERH
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
'  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim theLength As Double
863:   theLength = pCurve.length
  
  Dim pMpt As IPointCollection
866:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
868:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim dblRatio As Double
872:   dblRatio = SeparationDistance / theLength
  
  Dim theCurrentDist As Double
875:   theCurrentDist = 0
  
877:   Do While theCurrentDist < 1
878:     pCurve.QueryPoint esriNoExtension, theCurrentDist, True, pPoint
879:     Set pClone = pPoint
880:     pMpt.AddPoint pClone.Clone
'    ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPoint, "TestClipPolyGraphics"
882:     theCurrentDist = theCurrentDist + dblRatio
883:   Loop
884:   pCurve.QueryPoint esriNoExtension, 1, True, pPoint
885:   Set pClone = pPoint
886:   pMpt.AddPoint pClone.Clone
  
  Dim pGeometry As IGeometry
889:   Set pGeometry = pMpt
890:   Set pGeometry.SpatialReference = pCurve.SpatialReference
  
892:   Set DistributePointsAlongShape = pMpt
  
    Exit Function
  
ERH:
897:     MsgBox "Failed in DistributePointsAlongShape: " & err.Description
End Function

Public Function CalcGridLine(ByVal pStartPolygon As IPolygon, ByVal pEndPolygon As IPolygon, _
      ByVal pCorPolygon As IPolygon, pCorRaster As IRaster, pEnv As IRasterAnalysisEnvironment, _
      Optional ShouldClean As Boolean) As IPolyline
  On Error GoTo ErrorHandler

  
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  Dim pClone As IClone
  
  Dim pEnvelope As IEnvelope
911:   pEnv.GetExtent esriRasterEnvValue, pEnvelope
'  Set pEnvelope = pEnv.GetExtent

  'Create a RasterMakerOp operator
  Dim pRasMakerOp As IRasterMakerOp
916:   Set pRasMakerOp = New RasterMakerOp
917:   GridFunctions.SetSpatialAnalysisSettings pRasMakerOp, pEnv

  Dim pSpRef As ISpatialReference
920:   Set pSpRef = pEnv.OutSpatialReference
  
  ' SET POLYGON SPATIAL REFERENCES; PROJECT THEM IF NECESSARY
923:   If Not (GridFunctions.CompareSpatialReferences(pStartPolygon.SpatialReference, pSpRef)) Then
924:     pStartPolygon.Project pSpRef
925:   End If
926:   If Not (GridFunctions.CompareSpatialReferences(pEndPolygon.SpatialReference, pSpRef)) Then
927:     pEndPolygon.Project pSpRef
928:   End If
  
  ' GENERATE A SERIES OF POINTS AROUND THE BOUNDARY OF THE END POLYGON
  Dim pPoints As IPointCollection
932:   Set pPoints = GridFunctions.ReturnPointsByCellSize(pCorRaster, pEndPolygon)
  Dim anIndex As Long
'  For anIndex = 0 To pPoints.PointCount - 1
'    ThisDocument.Graphic_MakeFromGeometry pMxDoc, pPoints.Point(anIndex), "DeleteMatrix"
'  Next anIndex
  
  ' Create the RasterDistanceOp object
  Dim pDistanceOp As IDistanceOp
940:   Set pDistanceOp = New RasterDistanceOp
  
  ' SET ANALYSIS ENVIRONMENT
943:   SetSpatialAnalysisSettings pDistanceOp, pEnv
  
  ' Declare the input source raster object
  Dim pBaseRaster As IRaster
947:   Set pBaseRaster = pRasMakerOp.MakeConstant(1, True)
  Dim pSourceDataset As IRaster
949:   Set pSourceDataset = ClipRasterToPolygon(pBaseRaster, pStartPolygon, True, , , pEnv)
  
  ' Declare the input cost raster object
  Dim pCostDataset As IGeoDataset
  
  ' Declare the output raster object
  Dim pOutputRaster As IGeoDataset
  
  ' Calls the method
958:   Set pOutputRaster = pDistanceOp.CostDistanceFull(pSourceDataset, pCorRaster, True, True, False)
  
  ' To access the backlink raster from the output generated from the above code:
  Dim pRasterBandCollection As IRasterBandCollection
962:   Set pRasterBandCollection = pOutputRaster
  Dim pDistBand As IRasterBand        ' DISTANCE BAND
964:   Set pDistBand = pRasterBandCollection.Item(0)
  Dim pDistRaster As IRasterBandCollection
966:   Set pDistRaster = New Raster
967:   pDistRaster.Add pDistBand, 0
  
  Dim pDistAsRaster As IRaster
970:   Set pDistAsRaster = pDistRaster
  
  Dim pBacklinkBand As IRasterBand    ' BACKLINK BAND
973:   Set pBacklinkBand = pRasterBandCollection.Item(1)
  Dim pBacklinkRaster As IRasterBandCollection
975:   Set pBacklinkRaster = New Raster
976:   pBacklinkRaster.Add pBacklinkBand, 0
  
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
'  Dim pMap As IMap
'  Set pMap = pMxDoc.FocusMap
  
'  ThisDocument.Graphic_MakeFromGeometry pMxDoc, pCleanLine, "TestClipPolyGraphics"
  
'  Dim pRasLayer As IRasterLayer
'  Set pRasLayer = New RasterLayer
'  pRasLayer.CreateFromRaster pCorRaster
'  pMap.AddLayer pRasLayer
'  Dim pRasLayer2 As IRasterLayer
'  Set pRasLayer2 = New RasterLayer
'  pRasLayer2.CreateFromRaster pDistAsRaster
'  pMap.AddLayer pRasLayer2
'  Dim pRasLayer3 As IRasterLayer
'  Set pRasLayer3 = New RasterLayer
'  pRasLayer3.CreateFromRaster pBacklinkRaster
'  pMap.AddLayer pRasLayer3
  
'  Dim pEnumVertices As IEnumVertex
'  Set pEnumVertices = pPoints.EnumVertices
'  Dim pVertex As IPoint
'  Set pVertex = New Point
'  Dim lngPlaceholder As Long
'  Dim lngPlaceholder2 As Long
  
  ' ArcGIS APPEARS TO OCCASIONALLY SCREW UP THE SINGLE BACK PATH FUNCTION.  THEREFORE WE'LL
  ' CREATE THEM ALL
  
  Dim pPathCollection As IGeometryCollection
1009:   Set pPathCollection = pDistanceOp.CostPathAsPolyline(pPoints, pDistAsRaster, pBacklinkRaster)
  ' IF THIS PATH COLLECTION IS EMPTY, THEN LIKELY THAT ONE OF THE POLYGONS DOES NOT INTERSECT ANY GRID CELLS.
  ' THEREFORE CANNOT FIND PATH.
1012:   If pPathCollection.GeometryCount = 0 Then
    Dim pEmptyLine As IPolyline
1014:     Set pEmptyLine = New Polyline
1015:     pEmptyLine.SetEmpty
1016:     Set CalcGridLine = pEmptyLine
    Exit Function
1018:   End If
  
'  Dim pixelWidth As Double, pixelHeight As Double
'  pixelWidth = GridFunctions.ReturnPixelWidth(pDistAsRaster)
'  pixelHeight = GridFunctions.ReturnPixelHeight(pDistAsRaster)
'
'  Dim lngEnvYMax As Long, lngEnvXMin As Long
'  Dim pDistEnv As IEnvelope, pDistProps As IRasterProps
'  Set pDistProps = pDistAsRaster
'  Set pDistEnv = pDistProps.Extent
'  lngEnvYMax = pDistEnv.YMax
'  lngEnvXMin = pDistEnv.XMin
'
'  pEnumVertices.Reset
'  pEnumVertices.QueryNext pVertex, lngPlaceholder, lngPlaceholder2
  
  ' determine which row & col were clicked on
'  Dim lRow As Long, lCol As Long
'  Dim dblMinDist As Double
'  dblMinDist = pDistEnv.Width * pDistEnv.Height
'  Dim pMinPoint As IPoint
'  Set pClone = pVertex
'  Set pMinPoint = pClone.Clone
  
'  Dim pSizePoint As IPnt
'  Dim pPixBlock As IPixelBlock
'  Set pSizePoint = New DblPnt
'  pSizePoint.SetCoords 1, 1
'  Dim pTLCPoint As IPnt
'  Dim pSafeArray As Variant
'  Dim dblPixVal As Double
  
'  Do While Not pVertex.IsEmpty
'    lCol = Round(Abs(pVertex.x - lngEnvXMin - (0.5 * pixelWidth)) / pixelWidth)
'    lRow = Round(Abs(pVertex.Y - lngEnvYMax + (0.5 * pixelHeight)) / pixelHeight)
'    Debug.Print "row: " & lRow & ", col " & lCol
    
    ' create a pixel block (that has only one pixel)
    
'    Set pPixBlock = pDistAsRaster.CreatePixelBlock(pSizePoint)
'    Set pTLCPoint = New DblPnt
'    pTLCPoint.SetCoords lCol, lRow
'    pDistAsRaster.Read pTLCPoint, pPixBlock
'    pSafeArray = pPixBlock.SafeArray(0)
'    dblPixVal = pSafeArray(0, 0)
'    If dblPixVal < dblMinDist Then
'      dblMinDist = dblPixVal
'      Set pClone = pVertex
'      Set pMinPoint = pClone.Clone
'    End If
 '   Debug.Print "row: " & lRow & ", col " & lCol & ", val = " & dblPixVal & ".   Current min = " & dblMinDist
'    pEnumVertices.QueryNext pVertex, lngPlaceholder, lngPlaceholder2
'  Loop
  
'  Dim pPathPointCol As IPointCollection
'  Set pPathPointCol = New Multipoint
'  pPathPointCol.AddPoint pMinPoint
  
'  Dim pPathCollection As IGeometryCollection
'  Set pPathCollection = pDistanceOp.CostPathAsPolyline(pPathPointCol, pDistAsRaster, pBacklinkRaster)
   
  Dim pPath As IPolyline
  Dim pMinPath As IPolyline
  Dim dblShortDist As Double
1082:   Set pPath = pPathCollection.Geometry(0)
1083:   Set pMinPath = pPath
1084:   dblShortDist = pPath.length
1085:   If pPathCollection.GeometryCount > 1 Then
1086:     For anIndex = 1 To pPathCollection.GeometryCount - 1
1087:       Set pPath = pPathCollection.Geometry(anIndex)
1088:       If pPath.length < dblShortDist Then
1089:         Set pMinPath = pPath
1090:         dblShortDist = pPath.length
1091:       End If
1092:     Next anIndex
1093:   End If
  
  ' CLEAN POLYLINE
  Dim pBoundary As IPolyline
  Dim pTopoOp As ITopologicalOperator
1098:   Set pTopoOp = pCorPolygon
1099:   Set pBoundary = pTopoOp.Boundary
  
  Dim pCleanLine As IPolyline
1102:   If (ShouldClean) Then
1103:     Set pCleanLine = CleanPolyline(pMinPath, pBoundary)
1104:   Else
1105:     Set pCleanLine = pMinPath
1106:   End If
  
  
'    Dim pixelWidth As Double, pixelHeight As Double
'    pixelWidth = pREnv.Width / pRLayer.ColumnCount
'    pixelHeight = pREnv.Height / pRLayer.RowCount
    
    ' determine which row & col were clicked on
'    Dim lRow As Long, lCol As Long
'    lCol = Round(Abs(pPoint.x - pREnv.XMin - (0.5 * pixelWidth)) / pixelWidth)
'    lRow = Round(Abs(pPoint.Y - pREnv.YMax + (0.5 * pixelHeight)) / pixelHeight)
  
  
  
  
  
  
  
  'pExtractOp.Polygon(pOutRaster, pCorPolygon, True)
  
  'Create a raster layer from the output and add it to ArcMap

'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
'  Dim pMap As IMap
'  Set pMap = pMxDoc.FocusMap
  
'  ThisDocument.Graphic_MakeFromGeometry pMxDoc, pCleanLine, "TestClipPolyGraphics"
  
'  Dim pRasLayer As IRasterLayer
'  Set pRasLayer = New RasterLayer
'  pRasLayer.CreateFromRaster pCorRaster
'  pMap.AddLayer pRasLayer
'  Dim pRasLayer2 As IRasterLayer
'  Set pRasLayer2 = New RasterLayer
'  pRasLayer2.CreateFromRaster pDistAsRaster
'  pMap.AddLayer pRasLayer2
'  Dim pRasLayer3 As IRasterLayer
'  Set pRasLayer3 = New RasterLayer
'  pRasLayer3.CreateFromRaster pBacklinkRaster
'  pMap.AddLayer pRasLayer3
  
'  Set pCorRaster = Nothing
1149:   Set pDistAsRaster = Nothing
1150:   Set pBacklinkRaster = Nothing
1151:   pCleanLine.ReverseOrientation
1152:   Set pCleanLine.SpatialReference = pEnv.OutSpatialReference
1153:   Set CalcGridLine = pCleanLine


  Exit Function
ErrorHandler:
  HandleError True, "CalcGridLine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function CellValues(pPoints As IPointCollection, pRaster As IRaster) As esriSystem.IVariantArray
  On Error GoTo ErrorHandler

    Dim pRP As IRasterProps
1166:     Set pRP = pRaster
    
    Dim dblCellSize As Double
1169:     dblCellSize = ReturnCellSize(pRaster)
    
    'Get extent
    Dim pExtent As IEnvelope
1173:     Set pExtent = pRP.Extent
    Dim X1 As Double, Y1 As Double, X2 As Double, Y2 As Double
1175:     pExtent.QueryCoords X1, Y1, X2, Y2
    
    'Get a PixelBloc with cellvalues
    Dim pPB As IPixelBlock3
    Dim dWidth As Double, dHeight As Double
1180:     dWidth = pRP.Width
1181:     dHeight = pRP.Height
  
    'Create a DblPnt to hold the PixelBlock size
    Dim pPnt As IPnt
1185:     Set pPnt = New DblPnt
1186:     pPnt.SetCoords dWidth, dHeight
     
    'Create a point to set origin
    Dim pOrigin As IPnt
1190:     Set pOrigin = New DblPnt
1191:     pOrigin.SetCoords 0, 0
     
    'Create and read entire grid into PixelBlock
1194:     Set pPB = pRaster.CreatePixelBlock(pPnt)
1195:     pRaster.Read pOrigin, pPB
    
    Dim lngIndex As Long
    Dim dblCellValue As Double
    Dim pPoint As IPoint
    Dim dX As Double, dY As Double
    Dim nX As Double, nY As Double
    Dim iX As Long, iY As Long
    
    Dim pOutArray As esriSystem.IVariantArray
1205:     Set pOutArray = New esriSystem.VarArray
    
    Dim vCellValue As Variant
    
1209:     For lngIndex = 0 To pPoints.PointCount - 1
1210:       Set pPoint = pPoints.Point(lngIndex)
      ' RETURN NULL IF OUTSIDE EXTENT
1212:       If pPoint.X < X1 Or pPoint.X > X2 Or pPoint.Y < Y1 Or pPoint.Y > Y2 Then
1213:         pOutArray.Add Null
1214:       Else
    
1216:         dX = pPoint.X - X1
1217:         dY = Y2 - pPoint.Y
        
        'Find ncells from left-top
1220:         nX = dX / dblCellSize
1221:         nY = dY / dblCellSize
        
        'Determine cell index by taking the int. It will also take care of the fact that the index is 0 based
1224:         iX = Int(nX)
1225:         iY = Int(nY)
        
1227:         If (iX < 0) Then iX = 0
1228:         If (iY < 0) Then iY = 0
1229:         If (iX > pRP.Width - 1) Then
1230:           iX = pRP.Width - 1
1231:         End If
1232:         If (iY > pRP.Height - 1) Then
1233:           iY = pRP.Height - 1
1234:         End If
        
        'Read cell values
1237:         vCellValue = pPB.GetVal(0, iX, iY)
1238:         Debug.Print "From CellValues function..." & vCellValue
1239:         If IsEmpty(vCellValue) Then
1240:           pOutArray.Add Null
1241:         Else
1242:           pOutArray.Add CDbl(vCellValue)
1243:         End If
1244:       End If
1245:     Next lngIndex
1246:     Set CellValues = pOutArray
    

  Exit Function
ErrorHandler:
  HandleError True, "CellValues " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CellValue(pPoint As IPoint, pRaster As IRaster) As Variant
  On Error GoTo ErrorHandler

    
'    Dim pRawPixels As IRawPixels
'    Dim pRasterBandCol As IRasterBandCollection
'    Set pRasterBandCol = pRaster
'    Set pRawPixels = pRasterBandCol.Item(0)
'create a point which determines the size for IPixelBlock
'    Dim aoiPNT As IPnt
'    Set aoiPNT = New DblPnt
'        aoiPNT.SetCoords 1, 1
'create the IPixelBlock
'    Dim aoiPixelBlock As IPixelBlock
'    Set aoiPixelBlock = pRawPixels.CreatePixelBlock(aoiPNT)
    
    
    
    Dim pRP As IRasterProps
1273:     Set pRP = pRaster
    
    Dim dblCellSize As Double
1276:     dblCellSize = ReturnCellSize(pRaster)
    
    'Get extent
    Dim pExtent As IEnvelope
1280:     Set pExtent = pRP.Extent
    Dim X1 As Double, Y1 As Double, X2 As Double, Y2 As Double
1282:     pExtent.QueryCoords X1, Y1, X2, Y2
    
    ' RETURN NULL IF OUTSIDE EXTENT
1285:     If pPoint.X < X1 Or pPoint.X > X2 Or pPoint.Y < Y1 Or pPoint.Y > Y2 Then
1286:       CellValue = Null
      Exit Function
1288:     End If
    
    ' GET 1-CELL PIXELBLOCK
    'Get cell index from map coordinate
    Dim pCellPoint As IPoint  'Get dx dy from left-top
    Dim dX As Double, dY As Double
1294:     dX = pPoint.X - X1
1295:     dY = Y2 - pPoint.Y
    
    'Find ncells from left-top
    Dim nX As Double, nY As Double
1299:     nX = dX / dblCellSize
1300:     nY = dY / dblCellSize
    
    'Determine cell index by taking the int. It will also take care of the fact that the index is 0 based
    Dim iX As Long, iY As Long
1304:     iX = Int(nX)
1305:     iY = Int(nY)
    
    'If the index is < 0, set it to 0 - do I need this adjustment?
1308:     If (iX < 0) Then iX = 0
1309:     If (iY < 0) Then iY = 0
1310:     If (iX > pRP.Width - 1) Then
1311:       iX = pRP.Width - 1
1312:     End If
1313:     If (iY > pRP.Height - 1) Then
1314:       iY = pRP.Height - 1
1315:     End If
      
    'Create a point from cell index and return
1318:     Set pCellPoint = New Point
1319:     pCellPoint.PutCoords CDbl(iX), CDbl(iY)
    
    'Get a PixelBloc with cellvalues
    Dim pPB As IPixelBlock3
    Dim dWidth As Double, dHeight As Double
1324:     dWidth = pRP.Width
1325:     dHeight = pRP.Height
  
    'Create a DblPnt to hold the PixelBlock size
    Dim pPnt As IPnt
1329:     Set pPnt = New DblPnt
1330:     pPnt.SetCoords dWidth, dHeight
     
    'Create a point to set origin
    Dim pOrigin As IPnt
1334:     Set pOrigin = New DblPnt
1335:     pOrigin.SetCoords 0, 0
     
    'Create and read the PixelBlock
1338:     Set pPB = pRaster.CreatePixelBlock(pPnt)
    
    'Read cell values
1341:     pRaster.Read pOrigin, pPB
    Dim vCellValue As Variant
1343:     vCellValue = pPB.GetVal(0, iX, iY)
1344:     Debug.Print "From function..." & vCellValue
1345:     If IsEmpty(vCellValue) Then
1346:       CellValue = Null
1347:     Else
1348:       CellValue = CDbl(vCellValue)
1349:     End If
    

  Exit Function
ErrorHandler:
  HandleError True, "CellValue " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CleanPolyline(ByVal pPolyline As IPolyline, pBoundary As IPolyline) As IPolyline
  On Error GoTo ErrorHandler


  Dim pPointCol As IPointCollection
1362:   Set pPointCol = pPolyline
  
  Dim pNewPointCol As IPointCollection
1365:   Set pNewPointCol = New Polyline
  
  Dim pSegment As IPointCollection
  Dim pStartPoint As IPoint
  Dim pEndPoint As IPoint
  Dim pClone As IClone
  
  Dim pCount As Long
1373:   pCount = pPointCol.PointCount
  
1375:   Set pStartPoint = pPointCol.Point(0)
1376:   Set pClone = pStartPoint
1377:   pNewPointCol.AddPoint pClone.Clone
  
  Dim anIndex1 As Long
  Dim anIndex2 As Long
  Dim anIndexStep As Long
  
  Dim pRelOp As IRelationalOperator
  
1385:   anIndex1 = 1
1386:   anIndexStep = 0
  
1388:   Do Until anIndex1 = pPointCol.PointCount - 1
1389:     Set pEndPoint = pPointCol.Point(anIndex1)
1390:     Set pSegment = New Polyline
1391:     pSegment.AddPoint pStartPoint
1392:     pSegment.AddPoint pEndPoint
1393:     Set pRelOp = pSegment
1394:     If pRelOp.Disjoint(pBoundary) Then
1395:       anIndex1 = anIndex1 + 1
1396:     Else
1397:       anIndex2 = anIndex1 - 1
1398:       Do Until anIndex2 <= anIndexStep
1399:         Set pStartPoint = pPointCol.Point(anIndex2)
1400:         Set pSegment = New Polyline
1401:         pSegment.AddPoint pStartPoint
1402:         pSegment.AddPoint pEndPoint
1403:         Set pRelOp = pSegment
1404:         If pRelOp.Disjoint(pBoundary) Then
1405:           anIndex2 = anIndex2 - 1
1406:         Else
1407:           Set pClone = pStartPoint
1408:           pNewPointCol.AddPoint pClone.Clone
1409:           anIndexStep = anIndex2 + 1
1410:           Set pStartPoint = pPointCol.Point(anIndexStep)
1411:           Set pClone = pStartPoint
1412:           pNewPointCol.AddPoint pClone.Clone
1413:           anIndex1 = anIndexStep + 1
1414:           Exit Do
1415:         End If
1416:       Loop
1417:     End If
1418:   Loop
1419:   Set pClone = pPointCol.Point(pPointCol.PointCount - 1)
1420:   pNewPointCol.AddPoint pClone.Clone
1421:   Set CleanPolyline = pNewPointCol
1422:   Set CleanPolyline.SpatialReference = pPolyline.SpatialReference


  Exit Function
ErrorHandler:
  HandleError True, "CleanPolyline " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


