Attribute VB_Name = "aml_func_mod"
'             Environmental Systems Research Institute, Inc.
'Module Name: aml_func.bas
'Description: Used to perform AML-like functions for string manipipulation.
'             Also parses out tokens in a string to elements in an array.
'
'   Requires: ParseString and ParseStringR require that you Dim the named array arg in
'             the calling propgram: Must be a zero dimensioned string array:
'             Dim yourarray() as string

'    Methods: After   - Returns the substring to the right of the leftmost occurrence of
'                       searchStr
'             Before  - Returns the substring to the left of the leftmost occurrence of
'                       searchStr.
'        ExistFileDir - Returns True/False if file or Directory exists.
'             Extract - Returns an element from a list of elements
'             Index   - Returns the position of the leftmost occurrence of a specified
'                       string in a target string.
'             Keyword - Returns the position of a keyword within a list of keywords.
'             Search  - Returns the position of the first character of a search string
'                       in a target string.
'             Sort    - Returns a sorted a list of elements.
'             Subst   - Returns a string that has had one string subsituted for another
'             Substr  - Returns a substring that starts at a specified character position.
'             Token   - Allows tokens in a list to be manipulated.
'
'                       Count - the number of tokens in a list
'                       Find  - the position of a token in a list
'                       Move  - tokens in a list
'                       Insert- a new token into a list
'                       Delete- a token in a list
'                       Replace-one token in a list for another
'                       Switch -one token in a list for another
'
'        ParseString  - Populates string array with tokens in a string.
'        ParseStringR - Same as ParseString except blanks and commas are
'                       treated as delimiters.
'
'    History: DMA     - 03/04/97 - Original coding
'       Glenn Meister - 12/19/97 - Added ExistFileDir function
'
'   MODIFICATIONS:  JEFF JENNESS
'   GetFullFileString - ' CONVERTS 8.3 FILESTRING TO FULL TEXT
'           ReturnDir - GIVEN A STRING, RETURNS TEXT PRECEDING LAST "\" CHARACTER
'      ReturnFilename - GIVEN A STRING, RETURNS TEXT FOLLOWING LAST "\" CHARACTER
'         ReturnFiles - GIVEN A STRING PATH, RETURNS COLLECTION CONTAINING THE NUMBER OF FILES THAT MET CRITERIA
'                       AND ARRAY OF STRING FILENAMES.  OPTIONS FOR INCLUDE READ-ONLY, SYSTEM, HIDDEN FILES.
'       ReturnFolders - GIVENA STRING PATH, RETURNS COLLECTION CONTAINING THE NUMBER OF FOLDERS THAT MET CRITERIA
'                       AND ARRAY OF FOLDER FILENAMES.
'       ClipExtension - GIVEN A STRING, RETURNS TEXT PRECEDING LAST "." CHARACTER
'        SetExtension - GIVEN A STRING AND EXTENSION, RETURNS TEXT PRECEDING LAST "." CHARACTER, PLUS "." AND NEW EXTENSION
'    GetExtensionText - GIVEN A STRING, RETURNS TEXT FOLLOWING LAST "." CHARACTER
'      FieldIsNumeric - GIVEN A FIELD, RETURNS BOOLEAN
'       FieldIsString - GIVEN A FIELD, RETURNS BOOLEAN
'         FieldIsDate - GIVEN A FIELD, RETURNS BOOLEAN
'        FieldIsShape - GIVEN A FIELD, RETURNS BOOLEAN
'        InsertCommas - GIVEN A NUMBER OR STRING, INSERTS COMMAS AND RETURNS STRING
'     BasicTrimAvenue - GIVEN A STRING, RETURNS A STRING WITH VALUES TRIMMED OFF EACH SIDE
'  MakeUniqueFilename - GIVEN A STRING REPRESENTING A FILEPATH, RETURNS A UNIQUE FILENAME WHICH MAY HAVE NUMBERS APPENDED TO IT
'      GetTheUserName - JUST RETURNS THE USER NAME; CAN BE USED TO FIND THE "MY DOCUMENTS" FOLDER
'    TempPathLocation - RETURNS PATHNAME TO TEMP DIRECTORY
'     CreateShapefile - GIVEN PATHNAME AND SHAPE TYPE, RETURNS A pFeatureClass.  ESRI CODE, MODIFIED BY JENNESS
'    CreateShapefile2 - GIVEN PATHNAME, SHAPE TYPE AND ARRAY OF FIELDS, RETURNS A pFeatureClass.
'    CreatedBASETable - GIVEN PATHNAME AND OPTIONAL FIELDS, RETURNS ITABLE
'        GetMxDocPath - GIVEN APPLICATION OBJECT, RETURNS PATHNAME OF MAP DOCUMENT
'         QuoteString - GIVEN A STRING, RETURNS A QUOTED VERSION OF THAT STRING
'    SubstituteString - GIVEN A STRING, A SEARCH STRING AND A SUBSTITUTE STRING, REPLACES ALL INSTANCES OF SEARCH TEXT WITH
'                       SUBSTITUTE TEXT IN THE ORIGINAL STRING
'ReturnArcGISInstallDir - RETURNS A STRING CONTAINING THE ARCGIS INSTALL LOCATION
'ReturnArcGISGeneralDir - RETURNS DIRECTORY STRING FOR INSTALL, LAST LOCATION, LAST BROWSED, LAST EXPORT AND LAST SAVE TO
' ReturnArcGISVersion - RETURNS ARCGIS VERSION NUMBER
'ReturnArcGISVersionAlt - MORE DIRECT METHOD OF GETTING ARCGIS VERSION; RETURNS A LONG WHERE 0 = 8.3, 1 = 9.0, 2 = 9.2, 3 = 9.3
'          FileExists - GIVEN A STRING PATH, RETURNS TRUE OR FALSE
' ReturnShapeTypeName - GIVEN A SHAPE TYPE ENUMERATION, RETURNS SHAPE NAME
' QueryForNewFilename - GIVEN A SAMPLE FILENAME, OPENS DIALOG AND QUERIES USER TO SPECIFY NEW FILENAME

Option Explicit

' FILENAME 8.3 FORMAT CONVERSION BELOW COPIED FROM http://forums.esri.com/Thread.asp?c=93&f=993&t=123512&mc=1#msgid400360
'   POSTED BY  Brett N.Meroney
'              GIS Programmer / Analyst
'              Integrated Laboratory Systems, Inc.
'              contractor to US-EPA Region VIII
'              Golden , CO
' NECESSARY TO CONVERT 8.3 FILENAMES TO USEABLE FILENAMES

Private Const strMAXPATH = 260
Private Const m_Quotation = """"

'
' Win32 Registry functions
'
Private Declare Function RegOpenKeyEx Lib "advapi32.dll" Alias "RegOpenKeyExA" _
    (ByVal hKey As Long, _
     ByVal lpSubKey As String, _
     ByVal ulOptions As Long, _
     ByVal samDesired As Long, _
     phkResult As Long) _
     As Long
Private Declare Function RegCreateKeyEx Lib "advapi32.dll" Alias "RegCreateKeyExA" _
    (ByVal hKey As Long, _
     ByVal lpSubKey As String, _
     ByVal Reserved As Long, _
     ByVal lpClass As String, _
     ByVal dwOptions As Long, _
     ByVal samDesired As Long, _
     lpSecurityAttributes As Any, _
     phkResult As Long, _
     lpdwDisposition As Long) _
     As Long
Private Declare Function RegQueryValueEx Lib "advapi32.dll" Alias "RegQueryValueExA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String, _
     ByVal lpReserved As Long, _
     lpType As Long, _
     lpData As Any, _
     lpcbData As Long) _
     As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare Function RegQueryInfoKey Lib "advapi32.dll" Alias "RegQueryInfoKeyA" _
     (ByVal hKey As Long, _
     ByVal lpClass As String, _
     lpcbClass As Long, _
     lpReserved As Long, _
     lpcSubKeys As Long, _
     lpcbMaxSubKeyLen As Long, _
     lpcbMaxClassLen As Long, _
     lpcValues As Long, _
     lpcbMaxValueNameLen As Long, _
     lpcbMaxValueLen As Long, _
     lpcbSecurityDescriptor As Long, _
     lpftLastWriteTime As Any) _
     As Long
Private Declare Function RegEnumKeyEx Lib "advapi32.dll" Alias "RegEnumKeyExA" _
     (ByVal hKey As Long, _
     ByVal dwIndex As Long, _
     ByVal lpName As String, _
     lpcbName As Long, _
     lpReserved As Long, _
     ByVal lpClass As String, _
     lpcbClass As Long, _
     lpftLastWriteTime As Any) _
     As Long
Private Declare Function RegEnumValue Lib "advapi32.dll" Alias "RegEnumValueA" _
     (ByVal hKey As Long, _
      ByVal dwIndex As Long, _
     ByVal lpValueName As String, _
     lpcbValueName As Long, _
     lpReserved As Long, _
     lpType As Long, _
     lpData As Any, _
     lpcbData As Long) _
     As Long
Private Declare Function RegSetValueEx Lib "advapi32.dll" Alias "RegSetValueExA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String, _
     ByVal Reserved As Long, _
     ByVal dwType As Long, _
     lpData As Any, _
     ByVal cbData As Long) _
     As Long         ' Note that if you declare the lpData parameter as String, you must pass it By Value.
Private Declare Function RegDeleteKey Lib "advapi32.dll" Alias "RegDeleteKeyA" _
     (ByVal hKey As Long, _
      ByVal lpSubKey As String) _
     As Long
Private Declare Function RegDeleteValue Lib "advapi32.dll" Alias "RegDeleteValueA" _
     (ByVal hKey As Long, _
     ByVal lpValueName As String) _
     As Long
Private Declare Function RegCloseKey Lib "advapi32.dll" _
     (ByVal hKey As Long) _
     As Long
'
' Constants for Windows 32-bit Registry API
'
Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_DYN_DATA = &H80000006
'
' Reg result codes
'
Private Const REG_CREATED_NEW_KEY = &H1                      ' New Registry Key created
Private Const REG_OPENED_EXISTING_KEY = &H2                      ' Existing Key opened
'
' Reg Create Type Values...
'
Private Const REG_OPTION_RESERVED = 0           ' Parameter is reserved
Private Const REG_OPTION_NON_VOLATILE = 0       ' Key is preserved when system is rebooted
Private Const REG_OPTION_VOLATILE = 1           ' Key is not preserved when system is rebooted
Private Const REG_OPTION_CREATE_LINK = 2        ' Created key is a symbolic link
Private Const REG_OPTION_BACKUP_RESTORE = 4     ' open for backup or restore
'
' Reg Key Security Options
'
Private Const DELETE = &H10000
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const SYNCHRONIZE = &H100000
Private Const STANDARD_RIGHTS_READ = (READ_CONTROL)
Private Const STANDARD_RIGHTS_WRITE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_EXECUTE = (READ_CONTROL)
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_ALL = &H1F0000
Private Const SPECIFIC_RIGHTS_ALL = &HFFFF
Private Const KEY_QUERY_VALUE = &H1
Private Const KEY_SET_VALUE = &H2
Private Const KEY_CREATE_SUB_KEY = &H4
Private Const KEY_ENUMERATE_SUB_KEYS = &H8
Private Const KEY_NOTIFY = &H10
Private Const KEY_CREATE_LINK = &H20
Private Const KEY_READ = ((STANDARD_RIGHTS_READ Or KEY_QUERY_VALUE Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY) And (Not SYNCHRONIZE))
Private Const KEY_WRITE = ((STANDARD_RIGHTS_WRITE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY) And (Not SYNCHRONIZE))
Private Const KEY_ALL_ACCESS = ((STANDARD_RIGHTS_ALL Or KEY_QUERY_VALUE Or KEY_SET_VALUE Or KEY_CREATE_SUB_KEY Or KEY_ENUMERATE_SUB_KEYS Or KEY_NOTIFY Or KEY_CREATE_LINK) And (Not SYNCHRONIZE))
Private Const KEY_EXECUTE = ((KEY_READ) And (Not SYNCHRONIZE))

Private Const ERROR_SUCCESS = 0&
Private Const ERROR_MORE_DATA = 234
Private Const ERROR_NO_MORE_ITEMS = 259

Private Const REG_SZ = 1                         ' Unicode nul terminated string
'

Private Declare Function GetLongPathName Lib "Kernel32" Alias _
    "GetLongPathNameA" (ByVal lpszShortPath As String, _
    ByVal lpszLongPath As String, ByVal cchBuffer As Long) _
    As Long

Public Enum enumArcGISFolderTypes
   enumLastBrowsedLocation
   enumLastExportToLocation
   enumLastLocation
   enumLastSaveToLocation
   enumArcGISInstallLocation
End Enum

Private Declare Function GetUserName Lib "advapi32.dll" Alias "GetUserNameA" (ByVal lpBuffer As String, nSize As Long) As Long
Private Declare Function GetTempPath Lib "Kernel32" Alias "GetTempPathA" (ByVal nBufferLength As Long, ByVal lpBuffer As String) As Long
Const Quotation = """"
' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\arcGIS_stuff\general_tools\Registration\aml_func_mod.bas"

Public Function ReturnArcGISVersionAlt(pMxDoc As IMxDocument) As Long

247:    ReturnArcGISVersionAlt = "Unable to determine ArcGIS Version!"
   
   Dim pDocVersion As IDocumentVersion
250:    Set pDocVersion = pMxDoc
   
252:    ReturnArcGISVersionAlt = pDocVersion.DocumentVersion


End Function

Public Function QueryForNewFilename(strSampleName As String) As String
  On Error GoTo ErrorHandler



  Exit Function
ErrorHandler:
  HandleError True, "QueryForNewFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnShapeName(pEnum As esriGeometryType) As String
  On Error GoTo ErrorHandler


  Select Case pEnum
  Case 0
274:     ReturnShapeName = "Unknown Geometry"
  Case 1
276:     ReturnShapeName = "Point"
  Case 2
278:     ReturnShapeName = "Multipoint"
  Case 3
280:     ReturnShapeName = "Polyline"
  Case 4
282:     ReturnShapeName = "Polygon"
  Case 5
284:     ReturnShapeName = "Envelope"
  Case 6
286:     ReturnShapeName = "Path"
  Case 7
288:     ReturnShapeName = "Unknown Geometry"
  Case 9
290:     ReturnShapeName = "Multipatch"
  Case 11
292:     ReturnShapeName = "Ring"
  Case 13
294:     ReturnShapeName = "Line"
  Case 14
296:     ReturnShapeName = "Circular Arc"
  Case 15
298:     ReturnShapeName = "Bezier Curve"
  Case 16
300:     ReturnShapeName = "Elliptic Arc"
  Case 17
302:     ReturnShapeName = "Geometry Bag"
  Case 18
304:     ReturnShapeName = "Triangle Strip"
  Case 19
306:     ReturnShapeName = "Triangle Fan"
  Case 20
308:     ReturnShapeName = "Ray"
  Case 21
310:     ReturnShapeName = "Sphere"
  Case 22
312:     ReturnShapeName = "Triangles"
  Case Else
314:     ReturnShapeName = "Unknown Geometry"
315:   End Select


  Exit Function
ErrorHandler:
  HandleError True, "ReturnShapeName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FileExists(strFileName As String) As Boolean
  On Error GoTo ErrorHandler


328:   FileExists = Dir(strFileName) <> ""


  Exit Function
ErrorHandler:
  HandleError True, "FileExists " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISVersion() As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
349:    ReturnArcGISVersion = "Unable to determine ArcGIS Version!"
   
   Dim nRet As Long
   Dim hKey As Long
   Dim nType As Long
   Dim nBytes As Long
   Dim Buffer As String
   
   Dim strSection As String
358:    strSection = ""
   Dim strKey As String
360:    strKey = "RealVersion"
   Dim strDefault As String
362:    strDefault = ""
   Dim strDir As String
   
   ' Assume failure and set return to Default
'   GetSetting = Default

   ' Open key
369:    nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SubKey("ESRI", "ArcGIS", strSection), 0&, KEY_READ, hKey)
370:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
372:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
375:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
376:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
378:          If nBytes > 0 Then
379:             Buffer = Space(nBytes)
380:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
381:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
383:                strDir = Left(Buffer, nBytes - 1)
384:                ReturnArcGISVersion = strDir
385:             End If
386:          End If
387:       End If
388:       Call RegCloseKey(hKey)
389:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISVersion " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISInstallDir() As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
410:    ReturnArcGISInstallDir = "Unable to determine ArcGIS Install location!"
   
   Dim nRet As Long
   Dim hKey As Long
   Dim nType As Long
   Dim nBytes As Long
   Dim Buffer As String
   
   Dim strSection As String
419:    strSection = ""
   Dim strKey As String
421:    strKey = "InstallDir"
   Dim strDefault As String
423:    strDefault = ""
   Dim strDir As String
   
   
   ' Assume failure and set return to Default
'   GetSetting = Default

   ' Open key
431:    nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, SubKey("ESRI", "ArcGIS", strSection), 0&, KEY_READ, hKey)
432:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
434:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
437:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
438:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
440:          If nBytes > 0 Then
441:             Buffer = Space(nBytes)
442:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
443:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
445:                strDir = Left(Buffer, nBytes - 1)
446:                If Right(strDir, 1) = "\" Then strDir = Left(strDir, Len(strDir) - 1)
447:                ReturnArcGISInstallDir = strDir
448:             End If
449:          End If
450:       End If
451:       Call RegCloseKey(hKey)
452:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISInstallDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnArcGISGeneralDir(pArcGISFolderType As enumArcGISFolderTypes) As String
  On Error GoTo ErrorHandler


' ORIGINAL FUNCTION:
' Public Function GetSetting(ByVal Section As String, ByVal Key As String, Optional ByVal Default As String = "") As String
   ' Section   Required. String expression containing the name of the section where the key setting is found.
   '           If omitted, key setting is assumed to be in default subkey.
   ' Key       Required. String expression containing the name of the key setting to return.
   ' Default   Optional. Expression containing the value to return if no value is set in the key setting.
   '           If omitted, default is assumed to be a zero-length string ("").
   ' ADAPTED BY JENNESS FROM Ask the VB Pro column, [Getting Started with VB", Spring 1998.]
   ' SET DEFAULT VALUE
   
   
  Dim nRet As Long
  Dim hKey As Long
  Dim nType As Long
  Dim nBytes As Long
  Dim Buffer As String
  Dim strKey As String
  Dim strDir As String
  
  Select Case pArcGISFolderType
    Case enumArcGISInstallLocation
485:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Install location!"
486:       nRet = RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Software\ESRI\ArcGIS", 0&, KEY_READ, hKey)
487:       strKey = "InstallDir"
    Case enumLastSaveToLocation
489:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Saved To location!"
490:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
491:       strKey = "LastSaveToLocation"
    Case enumLastBrowsedLocation
493:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Browsed To location!"
494:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
495:       strKey = "LastBrowseLocation"
    Case enumLastExportToLocation
497:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last Exported To location!"
498:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
499:       strKey = "LastExportToLocation"
    Case enumLastLocation
501:       ReturnArcGISGeneralDir = "Unable to determine ArcGIS Last location!"
502:       nRet = RegOpenKeyEx(HKEY_CURRENT_USER, "Software\ESRI\ArcCatalog\Settings", 0&, KEY_READ, hKey)
503:       strKey = "LastLocation"
504:   End Select
   
  ' Assume failure and set return to Default
  '   GetSetting = Default

   ' Open key
510:    If nRet = ERROR_SUCCESS Then
      ' Set appropriate value for default query
512:       If strKey = "*" Then strKey = vbNullString
      
      ' Determine how large the buffer needs to be
515:       nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, nBytes)
516:       If nRet = ERROR_SUCCESS Then
         ' Build buffer and get data
518:          If nBytes > 0 Then
519:             Buffer = Space(nBytes)
520:             nRet = RegQueryValueEx(hKey, strKey, 0&, nType, ByVal Buffer, Len(Buffer))
521:             If nRet = ERROR_SUCCESS Then
               ' Trim NULL and return successful query!
523:                strDir = Left(Buffer, nBytes - 1)
524:                If Right(strDir, 1) = "\" Then strDir = Left(strDir, Len(strDir) - 1)
525:                ReturnArcGISGeneralDir = strDir
526:             End If
527:          End If
528:       End If
529:       Call RegCloseKey(hKey)
530:    End If


  Exit Function
ErrorHandler:
  HandleError True, "ReturnArcGISGeneralDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function SubKey(ByVal strCompany As String, ByVal strAppName As String, Optional ByVal Section As String = "") As String
  On Error GoTo ErrorHandler


   ' Build SubKey from known values
543:    SubKey = "Software\" & strCompany & "\" & strAppName
544:    If Len(Section) Then
545:       SubKey = SubKey & "\" & Section
546:    End If


  Exit Function
ErrorHandler:
  HandleError False, "SubKey " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ReturnFolders(ByVal DirPath As String, Optional ByVal FolderName As String) As Collection
  On Error GoTo ErrorHandler


  ' RETURNS A COLLECTION CONTAINING:
  '    A) Count of files that met criteria
  '    B) String Array of full folder filenames
  
  Dim returnCollection As Collection
562:   Set returnCollection = New Collection
  
  Dim strFolders() As String
  
  Dim strSearchString As String
567:   If FolderName = "" Then
568:     strSearchString = "*"
569:   Else
570:     strSearchString = FolderName
571:   End If
  
  Dim strSearchPath As String
  
575:   DirPath = Trim(DirPath)
576:   If Not Right(DirPath, 1) = "\" Then DirPath = DirPath & "\"
577:   strSearchPath = DirPath & strSearchString
  
  Dim strFoundFolder As String
  Dim anIndex As Integer
581:   anIndex = -1
  
583:   strFoundFolder = Dir(strSearchPath, 16)
  
585:   Do While Not strFoundFolder = ""
586:     If (Not strFoundFolder = ".") And (Not strFoundFolder = "..") Then
587:       If GetAttr(DirPath & strFoundFolder) = vbDirectory Then
588:         anIndex = anIndex + 1
        ReDim Preserve strFolders(anIndex)
590:         strFolders(anIndex) = DirPath & strFoundFolder
591:       End If
592:     End If
593:     strFoundFolder = Dir
594:   Loop
  
596:   returnCollection.Add (UBound(strFolders) - LBound(strFolders) + 1)
597:   returnCollection.Add (strFolders)
  
599:   Set ReturnFolders = returnCollection


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFolders " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnFiles(ByVal DirPath As String, Optional ByVal FileName As String, Optional ByVal IncludeReadOnlyFiles As Boolean, _
                            Optional ByVal IncludeHiddenFiles As Boolean, Optional ByVal IncludeSystemFiles As Boolean) As Collection
  On Error GoTo ErrorHandler


  ' RETURNS A COLLECTION CONTAINING:
  '    A) Count of files that met criteria
  '    B) String Array of full filenames
  
  Dim returnCollection As Collection
617:   Set returnCollection = New Collection
  
  Dim strFiles() As String
  Dim intOption As Integer
621:   intOption = 0
622:   If IncludeReadOnlyFiles Then intOption = intOption + 1
623:   If IncludeHiddenFiles Then intOption = intOption + 2
624:   If IncludeSystemFiles Then intOption = intOption + 4
  
  Dim strSearchString As String
627:   If FileName = "" Then
628:     strSearchString = "*"
629:   Else
630:     strSearchString = FileName
631:   End If
  
  Dim strSearchPath As String
  
635:   DirPath = Trim(DirPath)
636:   If Not Right(DirPath, 1) = "\" Then DirPath = DirPath & "\"
637:   strSearchPath = DirPath & strSearchString
  
  Dim strFoundFile As String
  Dim anIndex As Integer
  
642:   strFoundFile = Dir(strSearchPath, intOption)
  
644:   Do While Not strFoundFile = ""
    ReDim Preserve strFiles(anIndex)
646:     strFiles(anIndex) = DirPath & strFoundFile
647:     strFoundFile = Dir
648:     anIndex = anIndex + 1
649:   Loop
  
651:   returnCollection.Add anIndex
'  If anIndex = 0 Then
'    returnCollection.Add 0
'  Else
'    returnCollection.Add (UBound(strFiles) - LBound(strFiles) + 1)
'  End If
657:   returnCollection.Add (strFiles)
  
659:   Set ReturnFiles = returnCollection


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFiles " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function QuoteString(strInput As String) As String
  On Error GoTo ErrorHandler


  Dim strQuoted As String
673:   strQuoted = m_Quotation & SubstituteString(strInput, Chr(34), m_Quotation & m_Quotation) & m_Quotation
    
675:   QuoteString = strQuoted


  Exit Function
ErrorHandler:
  HandleError True, "QuoteString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ContainsString(strInText As String, strSearchText As String) As Boolean
  On Error GoTo ErrorHandler

  
687:   If strInText = "" Or strSearchText = "" Then
688:     ContainsString = False
689:   Else
690:     ContainsString = InStr(1, strInText, strSearchText, vbTextCompare) > 0
691:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ContainsString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SubstituteString(strFullText As String, strSearchText As String, strSubstituteText As String)
  On Error GoTo ErrorHandler


  Dim lngIndex As Long
  Dim lngStartPos As Long
  Dim lngSearchLength As Long
  Dim lngFullLength As Long
  
708:   lngStartPos = 1
709:   lngSearchLength = Len(strSearchText)
710:   lngFullLength = Len(strFullText)
  
712:   lngIndex = InStr(lngStartPos, strFullText, strSearchText, vbTextCompare)
  
  Dim strNewString As String
715:   If lngIndex = 0 Then
716:     strNewString = strFullText
717:   Else
718:     strNewString = Left(strFullText, lngIndex - 1) & strSubstituteText
719:     lngStartPos = lngIndex + lngSearchLength
720:   End If
  
722:   Do While lngIndex <> 0
723:     lngIndex = InStr(lngStartPos, strFullText, strSearchText, vbTextCompare)
724:     If lngIndex = 0 Then
725:       strNewString = strNewString & Right(strFullText, lngFullLength - lngStartPos + 1)
726:     Else
727:       strNewString = strNewString & Mid(strFullText, lngStartPos, lngIndex - lngStartPos) & strSubstituteText
728:       lngStartPos = lngIndex + lngSearchLength
729:     End If
  
731:   Loop
732:   SubstituteString = strNewString


  Exit Function
ErrorHandler:
  HandleError True, "SubstituteString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetMxDocPath(pApp As IApplication) As String
  On Error GoTo ErrorHandler


  Dim pTemplates As ITemplates
  Dim lTempCount As Long
  
747:   Set pTemplates = pApp.Templates
748:   lTempCount = pTemplates.Count
  
  ' The document is always the last item
751:   GetMxDocPath = pTemplates.Item(lTempCount - 1)


  Exit Function
ErrorHandler:
  HandleError True, "GetMxDocPath " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CreatedBASETable(strFullName As String, Optional pFields As IFields) As ITable
  On Error GoTo ErrorHandler


' createDBF: simple function to create a DBASE file.
' note: the name of the DBASE file should not contain the .dbf extension
' ESRI Sample; modified by Jenness August 20 2007
  
  Dim strName As String
  Dim strFolder As String
  
770:   strFolder = aml_func_mod.ReturnDir(strFullName)
771:   strName = aml_func_mod.ReturnFilename(strFullName)
772:   If Right(strName, 4) = ".dbf" Then strName = Left(strName, Len(strName) - 4)
  
  ' Open the Workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
  Dim fs As Object
  Dim pFieldsEdit As IFieldsEdit
  Dim pFieldEdit As IFieldEdit
  Dim pField As IField

782:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
783:   If Not aml_func_mod.ExistFileDir(strFolder) Then
784:     MsgBox "Folder does not exist: " & vbCr & strFolder
    Exit Function
786:   End If

788:   Set pFWS = pWorkspaceFactory.OpenFromFile(strFolder, 0)
  
  ' if a fields collection is not passed in then create one
791:   If pFields Is Nothing Then
    ' create the fields used by our object
793:     Set pFields = New Fields
794:     Set pFieldsEdit = pFields
795:     pFieldsEdit.FieldCount = 1
    
    'Create text Field
798:     Set pField = New Field
799:     Set pFieldEdit = pField
800:     With pFieldEdit
801:         .Precision = 8
802:         .Name = "Unique_ID"
803:         .Type = esriFieldTypeInteger
804:     End With
805:     Set pFieldsEdit.Field(0) = pField
806:   End If
  
  Dim strString As String
  Dim lngIndex As Long
  Dim pFieldInfo As IFieldInfo
  
812:   Set pField = pFields.Field(0)


'  MsgBox pField.Name
'  MsgBox pField.Scale
'  MsgBox pField.Precision
'  MsgBox pField.Type

'  For lngIndex = 0 To pFields.FieldCount - 1
'    strString = strString + "-----------------" & "  Field Name = " & pFields.Field(lngIndex).Name & vbCrLf & _
'        "  Field Scale = " & pFields.Field(lngIndex).Scale & vbCrLf & "  Precision = " & pFields.Field(lngIndex).Precision & _
'        vbCrLf & "  Field Type = " & CStr(pFields.Field(lngIndex).Type) & vbCrLf
'  Next lngIndex
'  MsgBox "Problem with workspace? " & CStr(pFWS Is Nothing) & vbCrLf & "Filename = " & strFullName & vbCrLf & _
'        "Folder = " & strFolder & vbCrLf & "strName = " & strName & vbCrLf & pFields.FieldCount & " fields..." & vbCrLf & _
'        "File already exists? " & CBool(FileExists(strFullName)) & vbCrLf & strString
  
829:   Set CreatedBASETable = pFWS.CreateTable(strName, pFields, Nothing, Nothing, "")


  Exit Function
ErrorHandler:
  HandleError True, "CreatedBASETable " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function CreateShapefile(sPath As String, sName As String, pSpatialReference As ISpatialReference, strShapeType As String) As IFeatureClass   ' Don't include filename!
  On Error GoTo ErrorHandler

  
842:   If Right(sPath, 4) = ".shp" Then sPath = ReturnDir(sPath)
843:   If Right(sName, 4) = ".shp" Then sName = Left(sName, Len(sName) - 4)
  
  ' SET GEOMETRY TYPE, AND EXIT IF NOT ONE OF STANDARD OPTIONS
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
848:   Set pGeomDef = New GeometryDef
849:   Set pGeomDefEdit = pGeomDef
850:   With pGeomDefEdit
    Select Case strShapeType
      Case "Polygon", "polygon"
853:         .GeometryType = esriGeometryPolygon
      Case "Polyline", "polyline"
855:         .GeometryType = esriGeometryPolyline
      Case "Point", "point"
857:         .GeometryType = esriGeometryPoint
      Case "Multipoint", "multipoint", "MultiPoint"
859:         .GeometryType = esriGeometryMultipoint
      Case "Multipatch", "multipatch", "MultiPatch"
861:         .GeometryType = esriGeometryMultiPatch
      Case Else
863:         MsgBox "Invalid Shape Type [" & strShapeType & "]!  This function is only written to generate " & _
            "Point, Polyline, Polygon, Multipoint or Multipatch shapefiles...", vbCritical, "Invalid Shape Type:"
865:     End Select
'    Set .SpatialReference = New UnknownCoordinateSystem
867:     Set .SpatialReference = pSpatialReference
868:   End With
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
873:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  
'   If Not pWorkspaceFactory.IsWorkspace(sPath) Then
876:   If Not ExistFileDir(sPath) Then
877:     MsgBox "Unable to create Feature Class:" & vbCrLf & _
           sPath & " is not a valid workspace...", , "Failed to Create Feature Class:"
879:     Set CreateShapefile = Nothing
    Exit Function
881:   End If
  
883:   Set pFWS = pWorkspaceFactory.OpenFromFile(sPath, 0)
  
  ' Set up a simple fields collection
  Dim pFields As IFields
  Dim pFieldsEdit As IFieldsEdit
888:   Set pFields = New Fields
889:   Set pFieldsEdit = pFields
  
  Dim pField As IField
  Dim pFieldEdit As IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
896:   Set pField = New Field
897:   Set pFieldEdit = pField
898:   pFieldEdit.Name = "Shape"
899:   pFieldEdit.Type = esriFieldTypeGeometry
  
901:   Set pFieldEdit.GeometryDef = pGeomDef
902:   pFieldsEdit.AddField pField

  ' Add an ID field
905:   Set pField = New Field
906:   Set pFieldEdit = pField
907:   With pFieldEdit
908:       .length = 8
909:       .Name = "Unique_ID"
910:       .Type = esriFieldTypeInteger
911:       .Precision = 0
912:   End With
913:   pFieldsEdit.AddField pField
  
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim booFileExists As Boolean
  Dim strCheckString As String
919:   If Right(sPath, 1) = "\" Then
920:     strCheckString = sPath & sName & ".shp"
'    MsgBox sPath & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & sName & ".shp") <> "")
922:   Else
923:     strCheckString = sPath & "\" & sName & ".shp"
'    MsgBox sPath & "\" & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & "\" & sName & ".shp") <> "")
925:   End If
  
927:   booFileExists = (Dir(strCheckString) <> "")
  
929:   If booFileExists Then
930:     MsgBox "The following file already exists:" & vbCrLf & vbCrLf & strCheckString & vbCrLf & vbCrLf & _
           "Please select a new filename...", , "Duplicate Filename:"
932:     Set CreateShapefile = Nothing
    Exit Function
934:   End If
  
  Dim pFeatClass As IFeatureClass
937:   Set pFeatClass = pFWS.CreateFeatureClass(sName, pFields, Nothing, _
                                           Nothing, esriFTSimple, "Shape", "")
                                           
940:   Set CreateShapefile = pFeatClass


  Exit Function
ErrorHandler:
  HandleError True, "CreateShapefile " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CreateShapefile2(sPath As String, sName As String, pSpatialReference As ISpatialReference, _
    strShapeType As String, pAddFields As esriSystem.IVariantArray) As IFeatureClass     ' Don't include filename!
  On Error GoTo ErrorHandler

  
953:   If Right(sPath, 4) = ".shp" Then sPath = ReturnDir(sPath)
954:   If Right(sName, 4) = ".shp" Then sName = Left(sName, Len(sName) - 4)
  
  ' SET GEOMETRY TYPE, AND EXIT IF NOT ONE OF STANDARD OPTIONS
  Dim pGeomDef As IGeometryDef
  Dim pGeomDefEdit As IGeometryDefEdit
959:   Set pGeomDef = New GeometryDef
960:   Set pGeomDefEdit = pGeomDef
961:   With pGeomDefEdit
    Select Case strShapeType
      Case "Polygon", "polygon"
964:         .GeometryType = esriGeometryPolygon
      Case "Polyline", "polyline"
966:         .GeometryType = esriGeometryPolyline
      Case "Point", "point"
968:         .GeometryType = esriGeometryPoint
      Case "Multipoint", "multipoint", "MultiPoint"
970:         .GeometryType = esriGeometryMultipoint
      Case "Multipatch", "multipatch", "MultiPatch"
972:         .GeometryType = esriGeometryMultiPatch
      Case Else
974:         MsgBox "Invalid Shape Type [" & strShapeType & "]!  This function is only written to generate " & _
            "Point, Polyline, Polygon, Multipoint or Multipatch shapefiles...", vbCritical, "Invalid Shape Type:"
976:     End Select
'    Set .SpatialReference = New UnknownCoordinateSystem
978:     Set .SpatialReference = pSpatialReference
979:   End With
  
  ' Open the folder to contain the shapefile as a workspace
  Dim pFWS As IFeatureWorkspace
  Dim pWorkspaceFactory As IWorkspaceFactory
984:   Set pWorkspaceFactory = New ShapefileWorkspaceFactory
  
'   If Not pWorkspaceFactory.IsWorkspace(sPath) Then
987:   If Not ExistFileDir(sPath) Then
988:     MsgBox "Unable to create Feature Class:" & vbCrLf & _
           sPath & " is not a valid workspace...", , "Failed to Create Feature Class:"
990:     Set CreateShapefile2 = Nothing
    Exit Function
992:   End If
  
994:   Set pFWS = pWorkspaceFactory.OpenFromFile(sPath, 0)
  
'   Set up a simple fields collection
  Dim pFields As IFields
  Dim pFieldsEdit As IFieldsEdit
999:   Set pFields = New Fields
1000:   Set pFieldsEdit = pFields

  Dim pField As IField
  Dim pFieldEdit As IFieldEdit
  
  ' Make the shape field
  ' it will need a geometry definition, with a spatial reference
1007:   Set pField = New Field
1008:   Set pFieldEdit = pField
1009:   pFieldEdit.Name = "Shape"
1010:   pFieldEdit.Type = esriFieldTypeGeometry

1012:   Set pFieldEdit.GeometryDef = pGeomDef
1013:   pFieldsEdit.AddField pField
  
  ' ADD FIELDS
  Dim lngIndex As Long
1017:   For lngIndex = 0 To pAddFields.Count - 1
1018:     pFieldsEdit.AddField pAddFields.Element(lngIndex)
1019:   Next lngIndex
    
  ' Create the shapefile
  ' (some parameters apply to geodatabase options and can be defaulted as Nothing)
  Dim booFileExists As Boolean
  Dim strCheckString As String
1025:   If Right(sPath, 1) = "\" Then
1026:     strCheckString = sPath & sName & ".shp"
'    MsgBox sPath & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & sName & ".shp") <> "")
1028:   Else
1029:     strCheckString = sPath & "\" & sName & ".shp"
'    MsgBox sPath & "\" & sName & ".shp" & vbCrLf & "File Exists? " & CStr(Dir(sPath & "\" & sName & ".shp") <> "")
1031:   End If
  
1033:   booFileExists = (Dir(strCheckString) <> "")
  
1035:   If booFileExists Then
1036:     MsgBox "The following file already exists:" & vbCrLf & vbCrLf & strCheckString & vbCrLf & vbCrLf & _
           "Please select a new filename...", , "Duplicate Filename:"
1038:     Set CreateShapefile2 = Nothing
    Exit Function
1040:   End If
  
  Dim pFeatClass As IFeatureClass
1043:   Set pFeatClass = pFWS.CreateFeatureClass(sName, pFields, Nothing, _
                                           Nothing, esriFTSimple, "Shape", "")
                                           
1046:   Set CreateShapefile2 = pFeatClass


  Exit Function
ErrorHandler:
  HandleError True, "CreateShapefile2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function TempPathLocation() As String
  On Error GoTo ErrorHandler


  Dim sBuffer As String
1060:   sBuffer = Space(strMAXPATH)
1061:   If GetTempPath(strMAXPATH, sBuffer) <> 0 Then
1062:     TempPathLocation = Left$(sBuffer, _
      InStr(sBuffer, vbNullChar) - 1)
1064:   Else
1065:     TempPathLocation = ""
1066:   End If


  Exit Function
ErrorHandler:
  HandleError True, "TempPathLocation " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function GetTheUserName() As String
  On Error GoTo ErrorHandler


  Dim sBuffer As String
  Dim sUName As String
  Dim lSize As Long
1082:   sBuffer = Space$(255)
1083:   lSize = Len(sBuffer)
1084:   Call GetUserName(sBuffer, lSize)
1085:   If lSize > 0 Then
1086:     sUName = Left$(sBuffer, lSize)
1087:   Else
1088:     sUName = vbNullString
1089:   End If
1090:   GetTheUserName = BasicTrimAvenue(sUName, "", Chr(0))     ' NEED TO PEEL OFF THAT LAST ODD CHARACTER


  Exit Function
ErrorHandler:
  HandleError True, "GetTheUserName " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function BasicTrimAvenue(aString As String, aTrimLeft As String, aTrimRight As String) As String
  On Error GoTo ErrorHandler

  
1102:   Do While (aString <> "") And (InStr(1, aTrimRight, Right(aString, 1), vbTextCompare) > 0)
1103:     aString = Left(aString, Len(aString) - 1)
1104:   Loop
1105:   Do While (aString <> "") And (InStr(1, aTrimLeft, Left(aString, 1), vbTextCompare) > 0)
1106:     aString = Right(aString, Len(aString) - 1)
1107:   Loop
  
1109:   BasicTrimAvenue = aString


  Exit Function
ErrorHandler:
  HandleError True, "BasicTrimAvenue " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function InsertCommas(InputValue As Variant) As String
  On Error GoTo ErrorHandler

  
  Dim theString As String
1122:   theString = CStr(InputValue)
  
  Dim theDecLocation As Long
1125:   theDecLocation = InStr(theString, ".")
  
  Dim HasDecimal As Boolean
1128:   HasDecimal = theDecLocation > 0
  
  Dim theLength As Long
1131:   theLength = Len(theString)
  
  Dim theBaseNumber As String
  Dim theRemainder As String
  
1136:   If HasDecimal Then
1137:     theRemainder = Right(theString, theLength - theDecLocation)
1138:     theBaseNumber = Left(theString, theDecLocation - 1)
1139:   Else
1140:     theRemainder = ""
1141:     theBaseNumber = theString
1142:   End If
  
  Dim theCount As Long
1145:   theCount = Len(theBaseNumber)
  
  Dim theCommaString As String
  
1149:   If theCount > 3 Then
    Dim anIndex As Long
1151:     For anIndex = (theCount - 2) To 1 Step -3
1152:       theCommaString = Mid(theBaseNumber, anIndex, 3) & "," & theCommaString
1153:       If anIndex < 4 Then
1154:         theCommaString = Left(theBaseNumber, anIndex - 1) & "," & theCommaString
1155:       End If
1156:     Next anIndex
    
1158:     Do While Right(theCommaString, 1) = ","
1159:       theCommaString = Left(theCommaString, Len(theCommaString) - 1)
1160:     Loop
1161:     Do While Left(theCommaString, 1) = ","
1162:       theCommaString = Right(theCommaString, Len(theCommaString) - 1)
1163:     Loop
1164:   Else
1165:     theCommaString = theBaseNumber
1166:   End If
  
1168:   If HasDecimal Then
1169:     theCommaString = theCommaString & "." & theRemainder
1170:   End If
  
1172:   InsertCommas = theCommaString


  Exit Function
ErrorHandler:
  HandleError True, "InsertCommas " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ClipExtension(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1187:   aml_func_mod.ParseString strPathName, strDirTokens, "."
1188:   strDirPath = strDirTokens(0)
  
1190:   If (UBound(strDirTokens) = 0) Then
1191:     ClipExtension = strDirPath
1192:   Else
    Dim anIndex As Long
1194:     For anIndex = 1 To (UBound(strDirTokens) - 1)
1195:       strDirPath = strDirPath & "." & strDirTokens(anIndex)
1196:     Next anIndex
1197:     ClipExtension = strDirPath
1198:   End If


  Exit Function
ErrorHandler:
  HandleError True, "ClipExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function FieldIsNumeric(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1211:   theFieldType = pTheField.Type
  
1213:   FieldIsNumeric = _
    (theFieldType = esriFieldTypeSmallInteger) Or (theFieldType = esriFieldTypeDouble) Or (theFieldType = esriFieldTypeInteger) Or _
          (theFieldType = esriFieldTypeSingle)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsNumeric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsString(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1227:   theFieldType = pTheField.Type
  
1229:   FieldIsString = (theFieldType = esriFieldTypeString)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsDate(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1241:   theFieldType = pTheField.Type
  
1243:   FieldIsDate = (theFieldType = esriFieldTypeDate)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsDate " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function FieldIsShape(pTheField As IField) As Boolean
  On Error GoTo ErrorHandler

  
  Dim theFieldType As esriFieldType
1255:   theFieldType = pTheField.Type
  
1257:   FieldIsShape = (theFieldType = esriFieldTypeGeometry)


  Exit Function
ErrorHandler:
  HandleError True, "FieldIsShape " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SetExtension(strPathName As String, strExtension As String) As String
  On Error GoTo ErrorHandler

  
  Dim theClippedPath As String
1270:   SetExtension = ClipExtension(strPathName) & "." & strExtension


  Exit Function
ErrorHandler:
  HandleError True, "SetExtension " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetExtensionText(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1285:   aml_func_mod.ParseString strPathName, strDirTokens, "."
1286:   If UBound(strDirTokens) = 0 Then
1287:     GetExtensionText = ""
1288:   Else
1289:     GetExtensionText = strDirTokens(UBound(strDirTokens))
1290:   End If


  Exit Function
ErrorHandler:
  HandleError True, "GetExtensionText " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function GetFullFileString(str83Type As String) As String
  On Error GoTo ErrorHandler

  
  ' ADAPTED FROM BRETT MERONEY'S POST ABOVE
  
  Dim lLen As Long
  Dim sBuffer As String
  
1307:   sBuffer = String$(strMAXPATH, 0)
1308:   lLen = GetLongPathName(str83Type, sBuffer, Len(sBuffer))
1309:   If lLen > 0 And err.Number = 0 Then
1310:     GetFullFileString = Left$(sBuffer, lLen)
1311:   Else
1312:     GetFullFileString = str83Type
1313:   End If


  Exit Function
ErrorHandler:
  HandleError True, "GetFullFileString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ReturnDir(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1328:   aml_func_mod.ParseString strPathName, strDirTokens, "\"
1329:   strDirPath = strDirTokens(0)
  
1331:   If (UBound(strDirTokens) = 0) Then
1332:     ReturnDir = strDirPath
1333:   Else
    Dim anIndex As Long
1335:     For anIndex = 1 To (UBound(strDirTokens) - 1)
1336:       strDirPath = strDirPath & "\" & strDirTokens(anIndex)
1337:     Next anIndex
1338:     ReturnDir = strDirPath
1339:   End If
  
1341:   ReturnDir = ReturnDir & "\"


  Exit Function
ErrorHandler:
  HandleError True, "ReturnDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function ReturnFilename(strPathName As String) As String
  On Error GoTo ErrorHandler


  Dim strDirPath As String
  Dim strDirTokens() As String
  
1357:   aml_func_mod.ParseString strPathName, strDirTokens, "\"
1358:   ReturnFilename = strDirTokens(UBound(strDirTokens))


  Exit Function
ErrorHandler:
  HandleError True, "ReturnFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Sub ParseString(str As String, strArray() As String, Delim As String)
  On Error GoTo ErrorHandler


' Populates a named string array with elements in a string. Each array element
' contains one word. Multiple words ' within single quotes ' are treated as
' one word. NOTE: Use parseStringR if you want both commas and blanks to be
' treated as delimiting characters.
'
' Before calling this Sub you must declare your array in the calling program
' As String, with no bounds.
'
' Dim myarray() As String
' mystring = "ARC YES, POLY NO, TICS YES"
' parseString (mystring), myarray, ","
' Returns:
' array(0) = ARC YES
' array(1) = POLY NO
' array(2) = TICS YES

' parseString(mystring),myarray," "
' Returns:
' array(0) = ARC
' array(1) = YES,
' array(2) = POLY
' array(3) = NO,
' array(4) = TICS
' array(5) = YES,

' mystring = "'Universe,Medium','Helvetica,Bold','Times,Medium'"
' parsestring(mystring),myarray,","
' Returns:
' array(0) = Universe,Medium
' array(1) = Helvetica,Bold
' array(2) = Times,Medium

' Dim counters

Dim i As Long
Dim tokenlen As Long
Dim tmpstr As String
Dim position As Long
Dim length As Long

'Dim variables to keep track of embedded quotes

Dim switch As Long
Dim position1 As Long
Dim position2 As Long
Dim pair As Long

On Error Resume Next

' If string contains no elements raise error
1419:   If Trim(Subst(str, Delim)) = "" Then
1420:     err.Raise vbObjectError + 1, "aml_func.ParseString", _
    "StringPassed"
    Exit Sub
1423:   End If

' intialize array. Warning: This will overwrite any data elements currently
' stored in this named array

  ReDim strArray(0)

'intializer counters and tracking variables

1432:   pair = False
1433:   switch = 0
1434:   length = Len(str)
1435:   position = 1
1436:   i = 0
1437:   tmpstr = str

'check each character in the array. If it is a quote, store if it is first or last
' 0 = havent read one yet
' 1 = read first single quote
' 2 = read second single quote

1444:   Do While position < length
1445:     If Mid(tmpstr, position, 1) = "'" Then
1446:       If Not (switch = 1) Then
1447:         switch = 1
1448:         position1 = position
1449:         pair = False
1450:       Else
1451:         switch = 2
1452:         position2 = position
1453:         pair = True
1454:       End If
1455:     End If

' if last char read was last in a pair of quotes, store contents between first and last
' in current array element and reset tracking variables

1460:     If pair = True Then
1461:       Mid(tmpstr, position1, 1) = " "
1462:       Mid(tmpstr, position2, 1) = " "
1463:       strArray(i) = Mid(tmpstr, position1, position2 - position1)
1464:       strArray(i) = Trim(strArray(i))
1465:       pair = False
1466:       switch = 0
      
' check to see if we are reading till the next single quote. If switch = 0, we are not
' if not check if the next character is a delimiter. if it is store everything to the left
' replace everything to the left of original str with blanks so we can safely use LEFT
' function then trim the blanks

1473:     Else
1474:       If switch = 0 Then
1475:         If Mid(tmpstr, position, 1) = Delim Then
1476:           strArray(i) = Left(tmpstr, position)
1477:           tokenlen = Len(strArray(i))
1478:           strArray(i) = Trim(strArray(i))
1479:           If Not (Len(strArray(i)) = 0) Then
1480:             Mid(tmpstr, 1, tokenlen) = String(tokenlen, " ")
            ReDim Preserve strArray(LBound(strArray) To UBound(strArray) + 1)
1482:             i = i + 1
1483:           End If
1484:         End If
1485:       End If
1486:     End If
1487:     position = position + 1
1488:   Loop
1489:   strArray(i) = Trim(tmpstr)

' we have populated our array, now remove the delimiters from each element

1493:   position = 1
1494:   For i = 0 To UBound(strArray)
1495:     position = Len(strArray(i))
1496:     If Mid(strArray(i), position, 1) = Delim Then
1497:       Mid(strArray(i), position, 1) = " "
1498:       strArray(i) = Trim(strArray(i))
1499:     End If
1500:   Next i


  Exit Sub
ErrorHandler:
  HandleError True, "ParseString " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function After(str As String, SearchStr As String) As String
  On Error GoTo ErrorHandler


' Returns the substring of Str to the right of the leftmost
' occurrence of the searchStr.
Dim position As Long
Dim length As Long

1517:   position = InStr(str, SearchStr)
1518:   length = Len(SearchStr)
1519:   If Not (position = 0) Then
1520:    After = Mid(str, position + length)
1521:   End If


  Exit Function
ErrorHandler:
  HandleError True, "After " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Before(str As String, SearchStr As String) As String
  On Error GoTo ErrorHandler


' Returns the substring of Str to the left of the leftmost
' occurrence of the searchStr.
Dim position As Long
Dim length As Long

1538:   position = InStr(str, SearchStr)
1539:   length = Len(SearchStr)
1540:   If Not (position = 0) Then
1541:    Before = Mid(str, 1, position - 1)
1542:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Before " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Function ExistFileDir(sTest As String) As Boolean
  On Error GoTo ErrorHandler


'Checks for the existance of a File or Directory
  Dim af As Long
1556:   af = -1
  On Error Resume Next
1558:   af = GetAttr(sTest)
1559:   ExistFileDir = (af <> -1)


  Exit Function
ErrorHandler:
  HandleError True, "ExistFileDir " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



Public Function MakeUniqueFilename(strFileName As String) As String
  On Error GoTo ErrorHandler


1573:   If Not ExistFileDir(strFileName) Then
1574:     MakeUniqueFilename = strFileName
    Exit Function
1576:   Else
    
    Dim theCounter As Long
1579:     theCounter = 1
    
    Dim theFilename As String
    Dim theBaseName As String
    Dim thePointPos As Long
    Dim theExtension As String
    
1586:     If InStr(1, Right(strFileName, 5), ".", vbTextCompare) > 0 Then
1587:       thePointPos = InStrRev(strFileName, ".", -1, vbTextCompare)
1588:       theExtension = Right(strFileName, Len(strFileName) - (thePointPos - 1))
1589:       theFilename = Left(strFileName, thePointPos - 1)
1590:     Else
1591:       theExtension = ""
1592:       theFilename = strFileName
1593:     End If
    
1595:     theBaseName = theFilename
    
1597:     Do While ExistFileDir(theFilename & theExtension)
1598:       theCounter = theCounter + 1
1599:       theFilename = theBaseName & "_" & CStr(theCounter)
1600:     Loop
    
1602:     MakeUniqueFilename = theFilename & theExtension
    
1604:   End If


  Exit Function
ErrorHandler:
  HandleError True, "MakeUniqueFilename " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Extract(ElemNum As Long, ElemList As String) As String
  On Error GoTo ErrorHandler


' extracts an element from a list of elements

Dim strArray() As String

1620:   ParseStringR (ElemList), strArray
1621:   If ElemNum > UBound(strArray) + 1 Then
   Exit Function
1623:   End If
  
1625:   If ElemNum = 0 Then
   Exit Function
1627:   End If
1628:   Extract = strArray(ElemNum - 1)


  Exit Function
ErrorHandler:
  HandleError True, "Extract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Index(str As String, SearchStr As String) As Long
  On Error GoTo ErrorHandler


' Returns the position of the leftmost occurrence of searcStr in str.

1642:   Index = InStr(str, SearchStr)


  Exit Function
ErrorHandler:
  HandleError True, "Index " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub ParseStringR(str As String, strArray() As String, Optional ReturnQuoted)
  On Error GoTo ErrorHandler


' Populates a named string arrary with elements in a string. Each array element
' contains one word. Multiple words ' within single quotes ' are treated as
' one word. Treats both blanks and commas as delimters NOTE: Use parseString
' to specify a specific delimiting character.

' ReturnQuoted - indicates if elements are to be returned quoted.
' FALSE - DEFAULT return elements unquoted
' TRUE - return elements quoted

' Before calling this function you must declare your array in the calling program
' As String, with no bounds.
'
' Dim myarray() As String
' mystring = "ARC YES, POLY NO, TICS YES"
' parseStringR(mystring),myarray

' Returns:
' array(0) = ARC
' array(1) = YES
' array(2) = POLY
' array(3) = NO
' array(4) = TICS
' array(5) = YES

' mystring = "'Universe,Medium','Helvetica,Bold','Times,Medium'"

' parsestringR(mystring),myarray
' Returns:
' array(0) = Universe,Medium
' array(1) = Helvetica,Bold
' array(2) = Times,Medium

' parsestring(mystring,myarray,TRUE)
' Returns:
' array(0) = 'Universe,Medium'
' array(1) = 'Helvetica,Bold'
' array(2) = 'Times,Medium'

' Dim counters

Dim i As Long
Dim tokenlen As Long
Dim switch As Long
Dim position1 As Long
Dim position2 As Long
Dim pair As Long
Dim parseAgain As Boolean
Dim tmpstr As String
Dim position As Long
Dim length As Long

On Error Resume Next

' If string contains no elements raise error
1708:   If Trim(Subst(str, ",")) = "" Then
1709:     err.Raise vbObjectError + 1, "aml_func.ParseStringR", _
    "StringPassed"
    Exit Sub
1712:   End If

' intialize counters and tracking variables
  ReDim strArray(0)
1716:   pair = False
1717:   switch = 0
1718:   length = Len(str)
1719:   position = 1
1720:   i = 0
1721:   tmpstr = str

1723:   If IsMissing(ReturnQuoted) Then
1724:     ReturnQuoted = False
1725:   End If
1726:   If Not (ReturnQuoted = False) Then
1727:     ReturnQuoted = True
1728:   End If

' check each character in the array. If it is a quote, store if it is first or last
' 0 = havent read one yet
' 1 = read first single quote
' 2 = just read second single quote

1735:   Do While position < length
1736:     If Mid(tmpstr, position, 1) = "'" Then
1737:       If Not (switch = 1) Then
1738:       switch = 1
1739:       position1 = position
1740:       pair = False
1741:     Else
1742:       switch = 2
1743:       position2 = position
1744:       pair = True
1745:       End If
1746:     End If
     
' if last char read was last in a pair of single quotes, store contents between first
' and last in current array element and reset tracking variables
 
1751:     If pair = True Then
1752:       Mid(tmpstr, position1, 1) = " "
1753:       Mid(tmpstr, position2, 1) = " "
1754:       strArray(i) = Mid(tmpstr, position1, position2 - position1)
1755:       strArray(i) = Trim(strArray(i))
1756:       pair = False
1757:       switch = 0
    
' check to see if we are reading till the next single quote. If switch = 0, we are not
' if not check if the next character is a delimiter. if it is store everything to the left
' replace everything to the left of original str with blanks so we can safely use LEFT
' function then trim the blanks
  
1764:     Else
1765:       If switch = 0 Then
1766:         If Mid(tmpstr, position, 1) = "," Or Mid(tmpstr, position, 1) = " " Then
1767:           strArray(i) = Left(tmpstr, position)
1768:           tokenlen = Len(strArray(i))
1769:           strArray(i) = Trim(strArray(i))
1770:           If Not (Len(strArray(i)) = 0) Then
1771:             Mid(tmpstr, 1, tokenlen) = String(tokenlen, " ")
            ReDim Preserve strArray(LBound(strArray) To UBound(strArray) + 1)
1773:             i = i + 1
1774:           End If
1775:         End If
1776:       End If
1777:     End If
1778:     position = position + 1
1779:   Loop
1780:     strArray(i) = Trim(tmpstr)

' we have populated our array, now remove the delimiters from each element
' set parseAgain flag if there are any blank elements

1785:   parseAgain = False
1786:   position = 1

1788:   For i = 0 To UBound(strArray)
1789:     position = Len(strArray(i))
1790:     If Mid(strArray(i), position, 1) = "," Then
1791:       Mid(strArray(i), position, 1) = " "
1792:       strArray(i) = Trim(strArray(i))
1793:       If Len(strArray(i)) = 0 Then
1794:         parseAgain = True
1795:       End If
1796:     End If
1797:   Next i
  
' now remove any blank elements
  
1801:   If parseAgain = True Then
1802:     tmpstr = ""
1803:     For i = 0 To UBound(strArray)
1804:       If Not (Len(strArray(i))) = 0 Then
1805:         tmpstr = tmpstr & "'" & strArray(i) & "'" & ","
1806:       End If
1807:     Next i
1808:     ParseString (tmpstr), strArray, ","
1809:   End If
      
1811:   If ReturnQuoted = True Then
1812:     For i = 0 To UBound(strArray)
1813:      strArray(i) = "'" & strArray(i) & "'"
1814:     Next i
1815:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "ParseStringR " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function Keyword(str As String, SearchStr As String) As Long
  On Error GoTo ErrorHandler


' Returns the position of a string within a list of keywords.
' converts Str and searchStr to upper case before comparing
'  0  if keyword not found
' -1  if string is ambiguous - mutiple occurances of same keyword
'  n  position of keyword in string

Dim strArray() As String
Dim i As Long
Dim keywordCnt As Long

1837:   ParseStringR (str), strArray
1838:   keywordCnt = 0
1839:     For i = 0 To UBound(strArray)
1840:     If UCase(SearchStr) = UCase(strArray(i)) Then
1841:       Keyword = i + 1
1842:       keywordCnt = keywordCnt + 1
1843:     End If
1844:     Next i
  
1846:   If keywordCnt > 1 Then
1847:     Keyword = -1
1848:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Keyword " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Search(str, SearchStr) As Long
  On Error GoTo ErrorHandler


' Returns the position of the first character in Str
' which occurs in searchStr.

Dim strArray() As String
Dim i As Long
Dim Index As Long
Dim firstchar
Dim InString As Boolean

1869:   Index = 1
  ReDim strArray(Len(str))

1872:   For i = 0 To Len(SearchStr) - 1
1873:     firstchar = Mid(SearchStr, i + 1, 1)
1874:     Index = InStr(str, firstchar)
1875:     strArray(Index) = i + 1
1876:   Next i
  
1878:   InString = False
  
1880:   For i = 1 To UBound(strArray)
1881:     If Not (strArray(i)) = "" Then
1882:     InString = True
1883:     Exit For
1884:     Else
1885:     End If
1886:   Next i
  
1888:   If InString = False Then
1889:     Search = 0
1890:   Else
1891:     Search = i
1892:   End If


  Exit Function
ErrorHandler:
  HandleError True, "Search " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Sort(str As String, Optional SortOption, Optional SortType) As String
  On Error GoTo ErrorHandler


' Returns a string of sorted elements

1906:   If IsMissing(SortOption) Then
1907:     SortOption = "-ASCEND"
1908:   ElseIf Not (UCase(SortOption) = "-DESCEND") Then
1909:     SortOption = "-ASCEND"
1910:   End If
  
1912:   If IsMissing(SortType) Then
1913:     SortType = "-CHARACTER"
1914:   ElseIf Not (UCase(SortType) = "-NUMERIC") Then
1915:     SortType = "-CHARACTER"
1916:   End If
  
1918:   If (UCase(SortType)) = "-NUMERIC" Then
1919:     Call Sort_Num(str, SortOption)
1920:   Else
1921:     Call Sort_Char(str, SortOption, True)
1922:   End If
1923:     Sort = str


  Exit Function
ErrorHandler:
  HandleError True, "Sort " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function Sort_Num(str As String, SortOption) As String
  On Error GoTo ErrorHandler


' Sort function - performs a numerical sort
' Ref: Selectionsort Chapter8 of VB Algorithms; Rod Stephens

Dim i As Long
Dim j As Long
Dim min As Long
Dim max As Long
Dim best_value As String
Dim best_j As Long
Dim sortArray() As String
Dim sorted As String

1947:   ParseStringR (str), sortArray
  
1949:   min = LBound(sortArray)
1950:   max = UBound(sortArray)
  
1952:   For i = min To max - 1
1953:     best_value = sortArray(i)
1954:     best_j = i
    
1956:     For j = i + 1 To max
1957:       If Val(sortArray(j)) < Val(best_value) Then
1958:         best_value = sortArray(j)
1959:         best_j = j
1960:       End If
1961:     Next j
      
1963:     sortArray(best_j) = sortArray(i)
1964:     sortArray(i) = best_value
1965:   Next i
    
1967:   If UCase(SortOption) = "-DESCEND" Then
1968:     For i = max To min Step -1
1969:       sorted = sorted & sortArray(i) & ","
1970:     Next i
1971:   Else
1972:     For i = min To max
1973:       sorted = sorted & sortArray(i) & ","
1974:     Next i
1975:   End If
  
1977:   Mid(sorted, Len(sorted), 1) = " "
1978:   str = sorted
1979:   Sort_Num = sorted


  Exit Function
ErrorHandler:
  HandleError False, "Sort_Num " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Function Sort_Char(str As String, SortOption, Optional ReturnQuoted) As String
  On Error GoTo ErrorHandler


' Sort function - performs a character sort
' Ref: Selectionsort Chapter8 of VB Algorithms; Rod Stephens

Dim i As Long
Dim j As Long
Dim min As Long
Dim max As Long
Dim best_value As String
Dim best_j As Long
Dim sortArray() As String
Dim sorted As String

2003: If IsMissing(ReturnQuoted) Then
2004:   ReturnQuoted = False
2005: End If
2006: If Not (ReturnQuoted = False) Then
2007:   ReturnQuoted = True
2008: End If

2010: ParseStringR (str), sortArray, ReturnQuoted

2012: min = LBound(sortArray)
2013: max = UBound(sortArray)

2015:   For i = min To max - 1
2016:     best_value = sortArray(i)
2017:     best_j = i
    
2019:     For j = i + 1 To max
2020:       If sortArray(j) < best_value Then
2021:       best_value = sortArray(j)
2022:       best_j = j
2023:       End If
2024:     Next j
    
2026:     sortArray(best_j) = sortArray(i)
2027:     sortArray(i) = best_value
2028:   Next i
  
2030:   If UCase(SortOption) = "-DESCEND" Then
2031:     For i = max To min Step -1
2032:       sorted = sorted & sortArray(i) & " "
2033:     Next i
2034:   Else
2035:     For i = min To max
2036:       sorted = sorted & sortArray(i) & " "
2037:     Next i
2038:   End If
2039:   Mid(sorted, Len(sorted), 1) = " "
  
2041:   str = sorted
2042:   Sort_Char = sorted


  Exit Function
ErrorHandler:
  HandleError False, "Sort_Char " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Subst(str As String, SearchChar As String, Optional ReplaceChar) As String
  On Error GoTo ErrorHandler


' Replaces all occurances of specified char in string.

Dim complete As Boolean
Dim i As Long
Dim first As String
Dim last As String
Dim tmpstr As String
Dim position As Long

2063: tmpstr = str

2065: position = 1
2066: complete = False

2068: If IsMissing(ReplaceChar) Then
2069:   Do Until complete = True
2070:     position = InStr(position, tmpstr, SearchChar)
2071:     If position = 0 Or position > Len(tmpstr) Then
2072:       complete = True
2073:     Else
2074:       first = Before(tmpstr, SearchChar)
2075:       last = After(tmpstr, SearchChar)
2076:       tmpstr = first & last
2077:     End If
2078:   Loop
2079: End If

2081: Do Until complete = True
2082:   position = InStr(position, tmpstr, SearchChar)
2083:   If position = 0 Or position > Len(tmpstr) Then
2084:     complete = True
2085:   Else
2086:     Mid(tmpstr, position, Len(ReplaceChar)) = ReplaceChar
2087:     position = position + Len(ReplaceChar)
2088:   End If
2089: Loop

Subst = tmpstr


  Exit Function
ErrorHandler:
  HandleError True, "Subst " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Substr(str As String, position As Long, Optional NumChars) As String
  On Error GoTo ErrorHandler


'extracts a substring starting at a specified character position.

2105: If IsMissing(NumChars) Then
2106:   If position = 0 Or position > Len(str) Then
2107:     Substr = ""
2108:   Else
2109:     Substr = Mid(str, position)
2110:   End If
2111: Else
2112:   If position = 0 Or position > Len(str) Then
2113:     Substr = ""
2114:   Else
2115:    Substr = Mid(str, position, NumChars)
2116:   End If
2117: End If


  Exit Function
ErrorHandler:
  HandleError True, "Substr " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function Token(ElemList As String, Arg As String, ParamArray OtherArgs()) As Variant
  On Error GoTo ErrorHandler


' Performs various functions for string manipulation

Dim strArray() As String
Dim i As Long
Dim temp As String
Dim from_elem  As Long
Dim to_elem As Long
Dim start_elem As Long
Dim insertStr As String
Dim DELETE As Long
Dim SearchStr As String

' Parse ElemList out to strarray
' Select TOKEN argument and perform function

2144:   ParseStringR (ElemList), strArray
2145:   Arg = Subst(Arg, "-")
  
  Select Case UCase(Arg)

  ' Count - returns the number of tokens in a list
    Case "COUNT"
2151:       Token = UBound(strArray) + 1
    
  ' Find <token> - returns the position of a token in a list
    Case "FIND"
2155:       SearchStr = OtherArgs(0)
2156:       Token = 0
2157:       For i = 0 To UBound(strArray)
2158:         If UCase(SearchStr) = UCase(strArray(i)) Then
2159:           Token = i + 1
2160:         End If
2161:       Next i
    
  ' Move <from_position> <to_position> - moves a token in the list
    Case "MOVE"
2165:       from_elem = OtherArgs(0) - 1
2166:       to_elem = OtherArgs(1)
2167:       temp = strArray(to_elem)
2168:       strArray(to_elem) = strArray(from_elem)
2169:       For i = from_elem To to_elem - 1
2170:         strArray(i) = strArray(i + 1)
2171:         Next i
2172:          strArray(i) = temp
2173:       For i = 0 To UBound(strArray) - 1
2174:         Token = Token & strArray(i) & ","
2175:       Next i
    
  ' Insert <position> - inserts a new token at <position> in the list
    Case "INSERT"
      ReDim Preserve strArray(UBound(strArray) + 1)
2180:       start_elem = OtherArgs(0) - 1
2181:       insertStr = OtherArgs(1)
2182:       For i = UBound(strArray) To start_elem Step -1
2183:         strArray(i) = strArray(i - 1)
2184:         Next i
2185:         strArray(start_elem) = insertStr
2186:         For i = 0 To UBound(strArray) - 1
2187:         Token = Token & strArray(i) & ","
2188:       Next i
    
  ' Delete <position> - removes the token at <position> from the list.
    Case "DELETE"
2192:       DELETE = OtherArgs(0) - 1
2193:       For i = DELETE To UBound(strArray) - 1
2194:         strArray(i) = strArray(i + 1)
2195:       Next i
2196:       For i = 0 To UBound(strArray) - 1
2197:         Token = Token & strArray(i) & ","
2198:       Next i
    
  ' Replace <position> <new_string> - replaces the token at <position> with the
  ' <new_string>.
    Case "REPLACE"
2203:       strArray(OtherArgs(1) - 1) = OtherArgs(0)
2204:       For i = 0 To UBound(strArray)
2205:         Token = Token & strArray(i) & ","
2206:       Next i
    
  ' Switch <position_1> <position_2> - moves token at <position_1> to <position_2> and moves
  ' token at <position_2> to <position_1>.
    Case "SWITCH"
2211:       from_elem = OtherArgs(0) - 1
2212:       to_elem = OtherArgs(1) - 1
2213:       temp = strArray(to_elem)
2214:       strArray(to_elem) = strArray(from_elem)
2215:       strArray(from_elem) = temp
2216:       For i = 0 To UBound(strArray)
2217:         Token = Token & strArray(i) & ","
2218:       Next i
    
    Case Else
2221:   End Select


  Exit Function
ErrorHandler:
  HandleError True, "Token " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function




