Attribute VB_Name = "MyGeometricOperations"
' MyGeometricOperations
'----------------------------------------------------------------------------------
'  Jeff Jenness
'  Jenness Enterprises
'  3020 N. Schevene Blvd.
'  Flagstaff, AZ  86004
'  http://www.jennessent.com
'  jeffj@jennessent.com
'  phone:  1-928-607-4638
'----------------------------------------------------------------------------------
'                 ArcCosJen - Returns ArcCos (Inverse Cosine)
'                 ArcSinJen - Returns ArcSin (Inverse Sine)
'                 AsDegrees - CONVERTS RADIANS TO DEGREES
'                 AsRadians - CONVERTS DEGREES TO RADIANS
'                     atan2 - Given DeltaY and DeltaX, returns ArcTangent that is sensitive to quadrant.
'          AzimuthHaversine - GIVEN TWO GEOGRAPHIC POINTS, RETURNS STARTING AZIMUTH POINTING IN GREAT CIRCLE OVER SPHERE.
'               CalcBearing - GIVEN TWO POINTS, CALCULATES THE CARTESIAN BEARING, WHERE 0 = NORTH, 360 DEGREES GOING CLOCKWISE
'              CalcBearing2 - SAME AS CalcBearing, EXCEPT IT RETURNS -999 IF THE TWO POINTS ARE COINCIDENT.
'        CalcCheckClockwise - CHECKS IF 3 CONSECUTIVE POINTS ARE ARRANGED COUNTERCLOCKWISE
'         CalcClosestPoints - GIVEN TWO GEOMETRIES AND OPTIONAL NUMBER OF TIMES TO GO BACK AND FORTH BETWEEN CURVES, RETURNS
'                             IArray CONTAINING:
'                             IStringArray containing either "Intersecting Shapes" or
'                                "Empty Shapes" + two booleans indicating which geometry is empty.
'                             -- OR --
'                             3 OBJECTS:
'                             0) Connector Line AS IPOLYLINE
'                             1) Closest Point on Geometry #1         AS IPOINT
'                             2) Closest Point on Geometry #2         AS IPOINT
'            CalcDistMatrix - GIVEN AN IARRAY OF SHAPES, RETURNS A COLLECTION WHERE:
'                             INDEX = IArrayIndex1 & "_" & IArrayIndex2, and Object =
'                             IArray of {Distance, Optional Line, Optional Azimuth}
'         CalcInternalAngle - GIVEN 3 POINTS, RETURNS THE INTERNAL ANGLE (IN DEGREES) AND OPTIONALLY THE ANGLE OF DEVIATION
'                             NOTE: ASSUMES PLANE, USES STANDARD TRIGONOMETRY
'             CalcPointLine - GIVEN POINT, DISTANCE, AZIMUTH, EMPTY ENDPOINT AND EMPTY POLYLINE, REPLACES EMPTY
'                             ENDPOINT WITH ACTUAL ENDPOINT AND OPTIONALLY RETURNS A POLYLINE CONNECTOR
'            CurveToPolygon - SIMILAR TO EllipticArcToPolygon2 EXCEPT THAT IT RETURNS AN IPolygon.  DOESN'T INSERT
'                             POINTS IF SEGMENT IS A LINE
'           CurveToPolyline - SIMILAR TO EllipticArcToPolygon2 EXCEPT THAT IT RETURNS AN IPolyline.  DOESN'T INSERT
'                             POINTS IF SEGMENT IS A LINE
'                  DegToRad - Given Degrees (double), returns Radians (double) (ACCIDENTALLY DUPLICATED THIS FUNCTION!  SEE AsRadians)
'         DistanceHaversine - GIVEN Point A, Point B and optional earth radius, returns distance between points in meters.
'                             Less accurate but faster than using Vincenty's functions.
'    DistanceVincentyPoints - Given 2 points and empty double variables for azimuths, returns distance in meters (using WGS84)
'                             and start and end azimuths of geodesic curve.
'   DistanceVincentyPoints2 - MODIFICATION OF DistanceVincentyPoints, TO ALLOW FOR ANY ELLIPSOID
'   DistanceVincentyNumbers - Given numeric values for latitude and longitude for 2 points, plus empty double variables
'                             for azimuths, returns distance in meters (using WGS84) and start and end azimuths of geodesic curve.
'  DistanceVincentyNumbers2 - MODIFICATION OF DistanceVincentyNumbers, TO ALLOW FOR ANY ELLIPSOID
'      EllipticArcToPolygon - Given a segment collection and number of vertices, returns a polygon4 simulating the ellipse
'                             by generating points along the arc and then calculating a convex hull around the points.
'     EllipticArcToPolygon2 - GIVEN A SEG COLLECTION AND NUMBER OF VERTICES, RETURNS A MULTIPOINT WITH APPROXIMATELY THE
'                             REQUESTED NUMBER OF POINTS DISTRIBUTED ALONG THE ARC.
'         EnvelopeToPolygon - GIVEN AN ENVELOPE, RETURNS A POLYGON
'FeaturePlanetOCentricToPlanetOGraphic - RETURNS A NEW GEOMETRY (POINT, POLYLINE, POLYGON OR MULTIPOINT) IN WHICH EACH VERTEX
'                             HAS BEEN SHIFTED FROM AN OCENTRIC TO AN OGRAPHIC LOCATION.
'FeaturePlanetOGraphicToPlanetOCentric - RETURNS A NEW GEOMETRY (POINT, POLYLINE, POLYGON OR MULTIPOINT) IN WHICH EACH VERTEX
'                             HAS BEEN SHIFTED FROM AN OGRAPHIC TO AN OCENTRIC LOCATION.
'  Graphic_MakeFromGeometry - GIVEN A MAP DOCUMENT, GEOMETRY AND OPTIONAL NAME AND SYMBOLOGY, ADDS GRAPHIC TO MAP.
'Graphic_ReturnElementFromGeometry - GIVEN MAP DOC, GEOMETRY, OPTIONAL NAME AND OPTIONAL ADD-TO-VIEW, RETURNS THE GRAPHIC
'                                    ELEMENT
'                   HArcCos - GIVEN A VALUE, RETURNS INVERSE HYPERBOLIC COSINE
'                   HArcSin - GIVEN A VALUE, RETURNS INVERSE HYPERBOLIC SINE
'                   HArcTan - GIVEN A VALUE, RETURNS INVERSE HYPERBOLIC TANGENT
'                      HCos - GIVEN A VALUE IN RADIANS, RETURNS HYPERBOLIC COSINE
'                      HSin - GIVEN A VALUE IN RADIANS, RETURNS HYPERBOLIC SINE
'                      HTan - GIVEN A VALUE IN RADIANS, RETURNS HYPERBOLIC TANGENT
'        MultipointCentroid - GIVEN AN IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE OF POINTS
'  MultipointCentroidSphere - GIVEN A GEOGRAPHIC IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE ON SPHERE
'MultipointCentroidSpheriod - GIVEN A GEOGRAPHIC IMULTIPOINT, RETURNS AN IPOINT REPRESENTING AVERAGE ON SPHEROID
'                  PointAdd - ADDS TWO POINTS
'         PointLineVincenty - Given Point, Distance and Azimuth, plus empty NewPoint and End Azimuth, plus optional
'                             number of vertices and empty Polyline, returns a new point the specified distance from the
'                             origin along geodesic, plus optional polyline with specified number of vertices.
'        PointLineVincenty2 - MODIFICATION OF PointLineVincenty, TO ALLOW FOR ANY ELLIPSOID
' PointLineVincentyPerPoint - Given pPoint, dblLength, dblAzimuth, empty NewPoint and empty End Azimuth,
'                             fills empty point and azimuth with correct values.
'PointLineVincentyPerPoint2 - MODIFICATION OF PointLineVincentyPerPoint, TO ALLOW FOR ANY ELLIPSOID
'             PointSubtract - SUBTRACTS POINT B FROM POINT A
'         PolygonToPolyline - GIVEN A POLYGON, RETURNS A POLYLINE
'                  RadToDeg - Given Radians (double), returns Degrees (double) (ACCIDENTALLY DUPLICATED THIS FUNCTION!  SEE AsDegrees)
'              ShowVertices - GIVEN A MAP DOC, GEOMETRY AND OPTIONAL NAME, ADDS POINT GRAPHICS TO SCREEN SHOWING WHERE
'                             VERTICES ARE
'    SphericalLatLongToCart - SUBROUTINE:  GIVEN LATITUDE AND LONGITUDE, AND OPTIONAL RADIUS, FILLS X, Y, Z VALUES
'    SphericalCartToLatLong - SUBROUTINE:  GIVEN X, Y, Z, FILLS LATITUDE AND LONGITUDE VALUES
'      SphericalPolygonArea - GIVEN A POLYGON, CALCULATES AREA USING SERIES OF SphericalTriangleArea CALLS
'     SphericalPolygonArea2 - MODIFICATION OF SphericalPolygonArea TO ALLOW USER TO SET CUSTOM ELLIPSOID MAJOR AND MINOR AXES
'   SpheroidalCartToLatLong - GIVEN X, Y, Z, OPTIONAL SPHEROID RADII AND HEIGHT ABOVE ELLPISOID, FILLS LATITUDE AND LONGITUDE VALUES
'   SpheroidalLatLongToCart - GIVEN LATITUDE, LONGITUDE, OPTIONAL SPHEROID RADII AND HEIGHT ABOVE ELLPISOID, FILLS X, Y, Z VALUES
'  SpheroidalPolylineLength - GIVEN A GEOGRAPHIC POLYLINE, RETURNS LENGTH IN METERS BASED ON VINCENTY'S EQUATIONS
' SpheroidalPolylineLength2 - MODIFICATION OF SpheroidalPolylineLength, TO ALLOW FOR ANY ELLIPSOID
'SpheroidalPolylineMidpoint - GIVEN A GEOGRAPHIC POLYLINE, DISTANCE VALUE, booAsRatio, RETURNS POINT AND OPTIONAL POLYLINE DISTANCE
'SpheroidalPolylineMidpoint2 - MODIFICATION OF SpheroidalPolylineMidpoint, TO ALLOW FOR ANY ELLIPSOID
'     SphericalTriangleArea - GIVEN 3 GEOGRAPHIC POINTS, CALCULATES SPHERICAL AREA
'    SphericalTriangleArea2 - MODIFICATION OF SphericalTriangleArea, TO ALLOW USER TO OPTIONALLY SEND CUSTOM MAJOR AND MINOR ELLIPSOID AXES
'        TriangleAreaPoints - GIVEN THREE POINTS, RETURNS AREA IN LOCAL UNITS
'      TriangleAreaPoints3D - GIVEN THREE 3-DIMENSIONAL POINTS, RETURNS AREA IN LOCAL UNITS
'        TriangleCentroid3D - GIVEN X,Y,Z COORDINATES FOR THREE 3D POINTS, RETURNS 3D CENTROID
'     TriangleCentroidPlane - GIVEN X,Y COORDINATES FOR THREE POINTS, RETURNS CENTROID
'      XYOCentricToOGraphic - GIVEN LONGITUDE, LATITUDE, ELLIPSOID MAJOR AND MINOR AXES, AND OPTIONAL LONGITUDE SHIFT, SETS NEW
'                             LATITUDE AND LONGITUDE VALUES BY CONVERTING FROM OCENTRIC TO OGRAPHIC.
'      XYOGraphicToOCentric - GIVEN LONGITUDE, LATITUDE, ELLIPSOID MAJOR AND MINOR AXES, AND OPTIONAL LONGITUDE SHIFT, SETS NEW
'                             LATITUDE AND LONGITUDE VALUES BY CONVERTING FROM OGRAPHIC TO OCENTRIC.

Option Explicit

Const dblPi As Double = 3.14159265358979
Const dblE As Double = 2.71828182845905
' Variables used by the Error handler function - DO NOT REMOVE
Const c_sModuleFileName As String = "D:\arcGIS_stuff\general_tools\Registration\MyGeometricOperations.bas"


Public Function DistanceHaversine(pPointA As IPoint, pPointB As IPoint, Optional dblRadius As Double = 6371000.79000915) As Double
  On Error GoTo ErrorHandler

  
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  
122:   dblLat1 = DegToRad(pPointA.Y)
123:   dblLat2 = DegToRad(pPointB.Y)
124:   dblLat = dblLat1 - dblLat2
125:   dblLong = DegToRad(pPointA.X - pPointB.X)
126:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
127:   DistanceHaversine = (2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))) * dblRadius


  Exit Function
ErrorHandler:
  HandleError True, "DistanceHaversine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MultipointCentroidSpheroid(pMultipoint As IMultipoint, Optional dblEquatorialRadius As Double = 6378137, _
    Optional dblPolarRadius As Double = 6356752.31424518, Optional dblHeightAboveEllipsoid As Double = 0) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim dblZ As Double
  Dim dblRunningX As Double
  Dim dblRunningY As Double
  Dim dblRunningZ As Double
  Dim pPointCollection As IPointCollection
148:   Set pPointCollection = pMultipoint
  Dim lngIndex As Long
150:   dblRunningX = 0
151:   dblRunningY = 0
152:   dblRunningZ = 0
153:   Set pPoint = New Point
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  
  Dim lngCounter As Long
158:   lngCounter = pPointCollection.PointCount
159:   For lngIndex = 0 To lngCounter - 1
160:     pPointCollection.QueryPoint lngIndex, pPoint
161:     SpheroidalLatLongToCart pPoint.X, pPoint.Y, dblX, dblY, dblZ, dblEquatorialRadius, dblPolarRadius, dblHeightAboveEllipsoid
162:     dblRunningX = dblRunningX + dblX
163:     dblRunningY = dblRunningY + dblY
164:     dblRunningZ = dblRunningZ + dblZ
165:   Next lngIndex
  
167:   dblX = dblRunningX / lngCounter
168:   dblY = dblRunningY / lngCounter
169:   dblZ = dblRunningZ / lngCounter
  
  ' CONVERT BACK TO GEOGRAPHIC COORDINATES
172:   SpheroidalCartToLatLong dblLongitude, dblLatitude, dblX, dblY, dblZ, dblEquatorialRadius, dblPolarRadius, dblHeightAboveEllipsoid
173:   Set MultipointCentroidSpheroid = New Point
174:   MultipointCentroidSpheroid.PutCoords dblLongitude, dblLatitude


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroidSpheroid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function MultipointCentroidSphere(pMultipoint As IMultipoint) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim dblZ As Double
  Dim dblRunningX As Double
  Dim dblRunningY As Double
  Dim dblRunningZ As Double
  Dim pPointCollection As IPointCollection
194:   Set pPointCollection = pMultipoint
  Dim lngIndex As Long
196:   dblRunningX = 0
197:   dblRunningY = 0
198:   dblRunningZ = 0
199:   Set pPoint = New Point
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  
  Dim lngCounter As Long
204:   lngCounter = pPointCollection.PointCount
205:   For lngIndex = 0 To lngCounter - 1
206:     pPointCollection.QueryPoint lngIndex, pPoint
207:     SphericalLatLongToCart pPoint.X, pPoint.Y, dblX, dblY, dblZ
208:     dblRunningX = dblRunningX + dblX
209:     dblRunningY = dblRunningY + dblY
210:     dblRunningZ = dblRunningZ + dblZ
211:   Next lngIndex
  
213:   dblX = dblRunningX / lngCounter
214:   dblY = dblRunningY / lngCounter
215:   dblZ = dblRunningZ / lngCounter
  
  ' CONVERT BACK TO GEOGRAPHIC COORDINATES
218:   SphericalCartToLatLong dblLongitude, dblLatitude, dblX, dblY, dblZ
219:   Set MultipointCentroidSphere = New Point
220:   MultipointCentroidSphere.PutCoords dblLongitude, dblLatitude


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroidSphere " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function MultipointCentroid(pMultipoint As IMultipoint) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim dblX As Double
  Dim dblY As Double
  Dim pPointCollection As IPointCollection
235:   Set pPointCollection = pMultipoint
  
  Dim lngIndex As Long
238:   dblX = 0
239:   dblY = 0
240:   Set pPoint = New Point
241:   For lngIndex = 0 To pPointCollection.PointCount - 1
242:     pPointCollection.QueryPoint lngIndex, pPoint
243:     dblX = dblX + pPoint.X
244:     dblY = dblY + pPoint.Y
245:   Next lngIndex
  
247:   Set MultipointCentroid = New Point
248:   MultipointCentroid.PutCoords dblX, dblY


  Exit Function
ErrorHandler:
  HandleError True, "MultipointCentroid " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function PolygonToPolyline(pPolygon As IPolygon2) As IPolyline
  On Error GoTo ErrorHandler

  
  Dim pNewGeometryCollection As IGeometryCollection
261:   Set pNewGeometryCollection = New Polyline
  Dim lngNumParts As Long
  Dim pPolyComponents() As IPolygon 'Declare an array of polygon
  Dim lngIndex As Long
  Dim pSubPolygon As IPolygon4
  Dim pTopoOp As ITopologicalOperator
  Dim pGeometryBag As IGeometryCollection
  Dim pNewSegCollection As ISegmentCollection
  Dim pOrigSegcollection As ISegmentCollection
  Dim pRing As IRing
  Dim pPath As IPath
  Dim pSpRef As ISpatialReference
  Dim lngIndex2 As Long
  Dim pPolyline As IPolyline
  
276:   Set pSpRef = pPolygon.SpatialReference
277:   Set pNewGeometryCollection = New Polyline
  
  ' GET CONNECTED COMPONENTS OF POLYGON
280:   lngNumParts = pPolygon.ExteriorRingCount
  ReDim pPolyComponents(lngNumParts - 1) 'Redimension the array of polygons with number of exterior rings
282:   pPolygon.GetConnectedComponents lngNumParts, pPolyComponents(0) 'Pass the first element of the array
  
284:   For lngIndex = 0 To lngNumParts - 1
285:     Set pSubPolygon = pPolyComponents(lngIndex)
286:     Set pTopoOp = pSubPolygon
287:     pTopoOp.Simplify
    
    ' GET SINGLE EXTERNAL RING AND ALL INTERNAL RINGS
290:     Set pGeometryBag = pSubPolygon.ExteriorRingBag
291:     Set pRing = pGeometryBag.Geometry(0)
292:     Set pOrigSegcollection = pRing
293:     Set pNewSegCollection = New Path
294:     pNewSegCollection.AddSegmentCollection pOrigSegcollection
295:     Set pPath = pNewSegCollection
296:     pNewGeometryCollection.AddGeometry pPath
    
    ' ADD ANY INTERNAL RINGS TO POLYLINE
299:     If pSubPolygon.InteriorRingCount(pRing) > 0 Then
300:       Set pGeometryBag = pSubPolygon.InteriorRingBag(pRing)
301:       For lngIndex2 = 0 To pGeometryBag.GeometryCount - 1
302:         Set pRing = pGeometryBag.Geometry(lngIndex2)
303:         Set pOrigSegcollection = pRing
304:         Set pNewSegCollection = New Path
305:         pNewSegCollection.AddSegmentCollection pOrigSegcollection
306:         Set pPath = pNewSegCollection
307:         pNewGeometryCollection.AddGeometry pPath
308:       Next lngIndex2
309:     End If
310:   Next lngIndex
  
  ' CLEAN NEW POLYLINE
313:   Set pPolyline = pNewGeometryCollection
314:   Set pTopoOp = pPolyline
315:   pTopoOp.Simplify
316:   Set pPolyline.SpatialReference = pSpRef

318:   Set PolygonToPolyline = pPolyline


  Exit Function
ErrorHandler:
  HandleError True, "PolygonToPolyline " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub TriangleCentroid3D(dblPX As Double, dblPY As Double, dblPZ As Double, _
                                           dblQX As Double, dblQY As Double, dblQZ As Double, _
                                           dblRX As Double, dblRY As Double, dblRZ As Double, _
                                           dblCentX As Double, dblCentY As Double, dblCentZ As Double)
  On Error GoTo ErrorHandler


333:   dblCentX = (dblPX + dblQX + dblRX) / 3
334:   dblCentY = (dblPY + dblQY + dblRY) / 3
335:   dblCentZ = (dblPZ + dblQZ + dblRZ) / 3


  Exit Sub
ErrorHandler:
  HandleError True, "TriangleCentroid3D " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub TriangleCentroidPlane(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
                                           dblRX As Double, dblRY As Double, dblCentX As Double, dblCentY As Double)
  On Error GoTo ErrorHandler


348:   dblCentX = (dblPX + dblQX + dblRX) / 3
349:   dblCentY = (dblPY + dblQY + dblRY) / 3


  Exit Sub
ErrorHandler:
  HandleError True, "TriangleCentroidPlane " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SphericalCartToLatLong(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double)
  On Error GoTo ErrorHandler


  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
369:   dblPhi = atan2(Sqr(X ^ 2 + Y ^ 2), Z)
370:   dblTheta = atan2(Y, X)
  
372:   dblLongitude = RadToDeg(dblTheta)
373:   dblLatitude = 90 - RadToDeg(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SphericalCartToLatLong " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SpheroidalCartToLatLong(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblEquatorialRadius As Double = 6378137, _
      Optional dblPolarRadius As Double = 6356752.31424518, Optional dblHeightAboveEllipsoid As Double = 0)
  On Error GoTo ErrorHandler


  ' IF SPHEROID PARAMETERS NOT INCLUDED, DEFAULTS TO WGS84
  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  ' MODIFIED FROM J.C. ILIFFE, CHAPTER 2
  ' NOTE:  ILIFFE USES PHI FOR LATITUDE DIRECTLY, RATHER THAN AS DISTANCE FROM POLES
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
  
  Dim dblP As Double
400:   dblP = Sqr(X ^ 2 + Y ^ 2)
  
  Dim dblU As Double
403:   dblU = atan2((Z * dblEquatorialRadius), (dblP * dblPolarRadius))
  
  Dim dblEccentSquared As Double
406:   dblEccentSquared = ((dblEquatorialRadius ^ 2) - (dblPolarRadius ^ 2)) / (dblEquatorialRadius ^ 2)
  
  Dim dblEpsilon As Double
409:   dblEpsilon = dblEccentSquared / (1 - dblEccentSquared)
  
411:   dblPhi = atan2(dblP - (dblEccentSquared * dblEquatorialRadius * (Cos(dblU) ^ 3)), _
                  Z + (dblEpsilon * dblPolarRadius * (Sin(dblU) ^ 3)))
413:   dblTheta = atan2(Y, X)
  
415:   dblLongitude = RadToDeg(dblTheta)
416:   dblLatitude = 90 - RadToDeg(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SpheroidalCartToLatLong " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub SphericalLatLongToCart(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblRadius = 1)
  On Error GoTo ErrorHandler


  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
435:   dblPhi = DegToRad(90 - dblLatitude)
436:   dblTheta = DegToRad(dblLongitude)
  
438:   X = dblRadius * Sin(dblPhi) * Cos(dblTheta)
439:   Y = dblRadius * Sin(dblPhi) * Sin(dblTheta)
440:   Z = dblRadius * Cos(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SphericalLatLongToCart " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub


Public Sub SpheroidalLatLongToCart(dblLongitude As Double, dblLatitude As Double, X As Double, _
      Y As Double, Z As Double, Optional dblEquatorialRadius As Double = 6378137, _
      Optional dblPolarRadius As Double = 6356752.31424518, Optional dblHeightAboveEllipsoid As Double = 0)
  On Error GoTo ErrorHandler

  
  ' IF SPHEROID PARAMETERS NOT INCLUDED, DEFAULTS TO WGS84

  ' Phi is angle from north pole down to Latitude
  ' Theta is angle from Greenwich
  
  ' MODIFIED FROM J.C. ILIFFE, CHAPTER 2
  ' NOTE:  ILIFFE USES PHI FOR LATITUDE DIRECTLY, RATHER THAN AS DISTANCE FROM POLES
  
  Dim dblPhi As Double
  Dim dblTheta As Double
  
466:   dblPhi = DegToRad(90 - dblLatitude)
467:   dblTheta = DegToRad(dblLongitude)
  
  Dim dblNu As Double
  Dim dblEccentSquared As Double
471:   dblEccentSquared = ((dblEquatorialRadius ^ 2) - (dblPolarRadius ^ 2)) / (dblEquatorialRadius ^ 2)
472:   dblNu = (dblEquatorialRadius) / Sqr(1 - (dblEccentSquared * (Cos(dblPhi) ^ 2)))
  
474:   X = (dblNu + dblHeightAboveEllipsoid) * Sin(dblPhi) * Cos(dblTheta)
475:   Y = (dblNu + dblHeightAboveEllipsoid) * Sin(dblPhi) * Sin(dblTheta)
476:   Z = (((1 - dblEccentSquared) * dblNu) + dblHeightAboveEllipsoid) * Cos(dblPhi)


  Exit Sub
ErrorHandler:
  HandleError True, "SpheroidalLatLongToCart " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function SpheroidalPolylineMidpoint(pPolyline As IPolyline, dblDistance As Double, _
    booIsRatio As Boolean, Optional dblPolylineLength As Double) As IPoint
  On Error GoTo ErrorHandler


  Dim pPoint As IPoint
  Dim pVarArray As esriSystem.IVariantArray
  Dim pDblArray As esriSystem.IDoubleArray
492:   Set pVarArray = New esriSystem.VarArray
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  Dim dblX1 As Double
  Dim dblY1 As Double
  
506:   dblTotalLength = 0
507:   Set pSegCollection = pPolyline
  
  ' GET TOTAL LENGTH AND FILL ARRAY WITH SEGMENT STATISTICS
510:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
511:     Set pSeg = pSegCollection.Segment(lngIndex)
512:     Set pPoint1 = pSeg.FromPoint
513:     Set pPoint2 = pSeg.ToPoint
514:     dblLength = DistanceVincentyPoints(pPoint1, pPoint2, dblAz1, dblAz2)
515:     dblTotalLength = dblTotalLength + dblLength
516:     Set pDblArray = New esriSystem.DoubleArray
517:     pDblArray.Add dblLength
518:     pDblArray.Add dblTotalLength
519:     pDblArray.Add dblAz1
520:     pDblArray.Add pPoint1.X
521:     pDblArray.Add pPoint1.Y
522:     pVarArray.Add pDblArray
523:   Next lngIndex
  
525:   dblPolylineLength = dblTotalLength
  
  Dim dblHalfLength As Double
528:   If booIsRatio Then
529:     If dblDistance > 1 Then dblDistance = 1
530:     If dblDistance < 0 Then dblDistance = 0
531:     dblHalfLength = dblPolylineLength * dblDistance
532:   Else
533:     dblHalfLength = dblDistance
534:   End If
    
536:   For lngIndex = 0 To pVarArray.Count - 1
537:     Set pDblArray = pVarArray.Element(lngIndex)
538:     dblTotalLength = pDblArray.Element(1)
539:     If dblTotalLength > dblHalfLength Then
540:       dblLength = pDblArray.Element(0)
541:       dblAz1 = pDblArray.Element(2)
542:       dblX1 = pDblArray.Element(3)
543:       dblY1 = pDblArray.Element(4)
544:       Exit For
545:     End If
546:   Next lngIndex
  
  Dim dblProperDistance As Double
549:   dblProperDistance = dblLength - (dblTotalLength - dblHalfLength)
  
551:   Set pPoint1 = New Point
552:   pPoint1.PutCoords dblX1, dblY1
553:   Set pPoint2 = New Point
    
555:   PointLineVincenty pPoint1, dblProperDistance, dblAz1, pPoint2, dblAz2
  
557:   Set SpheroidalPolylineMidpoint = pPoint2


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineMidpoint " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function SpheroidalPolylineMidpoint2(pPolyline As IPolyline, dblDistance As Double, _
    booIsRatio As Boolean, Optional dblPolylineLength As Double, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As IPoint
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF SpheroidalPolylineMidpoint, TO ALLOW FOR ANY ELLIPSOID
  
  Dim pPoint As IPoint
  Dim pVarArray As esriSystem.IVariantArray
  Dim pDblArray As esriSystem.IDoubleArray
575:   Set pVarArray = New esriSystem.VarArray
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  Dim dblX1 As Double
  Dim dblY1 As Double
  
589:   dblTotalLength = 0
590:   Set pSegCollection = pPolyline
  
  ' GET TOTAL LENGTH AND FILL ARRAY WITH SEGMENT STATISTICS
593:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
594:     Set pSeg = pSegCollection.Segment(lngIndex)
595:     Set pPoint1 = pSeg.FromPoint
596:     Set pPoint2 = pSeg.ToPoint
597:     dblLength = DistanceVincentyPoints2(pPoint1, pPoint2, dblAz1, dblAz2, dblEquatorialRadius, dblPolarRadius)
598:     dblTotalLength = dblTotalLength + dblLength
599:     Set pDblArray = New esriSystem.DoubleArray
600:     pDblArray.Add dblLength
601:     pDblArray.Add dblTotalLength
602:     pDblArray.Add dblAz1
603:     pDblArray.Add pPoint1.X
604:     pDblArray.Add pPoint1.Y
605:     pVarArray.Add pDblArray
606:   Next lngIndex
  
608:   dblPolylineLength = dblTotalLength
  
  Dim dblHalfLength As Double
611:   If booIsRatio Then
612:     If dblDistance > 1 Then dblDistance = 1
613:     If dblDistance < 0 Then dblDistance = 0
614:     dblHalfLength = dblPolylineLength * dblDistance
615:   Else
616:     dblHalfLength = dblDistance
617:   End If
    
619:   For lngIndex = 0 To pVarArray.Count - 1
620:     Set pDblArray = pVarArray.Element(lngIndex)
621:     dblTotalLength = pDblArray.Element(1)
622:     If dblTotalLength > dblHalfLength Then
623:       dblLength = pDblArray.Element(0)
624:       dblAz1 = pDblArray.Element(2)
625:       dblX1 = pDblArray.Element(3)
626:       dblY1 = pDblArray.Element(4)
627:       Exit For
628:     End If
629:   Next lngIndex
  
  Dim dblProperDistance As Double
632:   dblProperDistance = dblLength - (dblTotalLength - dblHalfLength)
  
634:   Set pPoint1 = New Point
635:   pPoint1.PutCoords dblX1, dblY1
636:   Set pPoint1.SpatialReference = pPolyline.SpatialReference
  
638:   Set pPoint2 = New Point
  
640:   PointLineVincenty2 pPoint1, dblProperDistance, dblAz1, pPoint2, dblAz2
  
642:   Set SpheroidalPolylineMidpoint2 = pPoint2


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineMidpoint2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function SpheroidalPolylineLength(pPolyline As IPolyline) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYLINE IS IN GEOGRAPHIC COORDINATES

  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  
665:   dblTotalLength = 0
666:   Set pSegCollection = pPolyline
  
668:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
669:     Set pSeg = pSegCollection.Segment(lngIndex)
670:     Set pPoint1 = pSeg.FromPoint
671:     Set pPoint2 = pSeg.ToPoint
672:     dblLength = DistanceVincentyPoints(pPoint1, pPoint2, dblAz1, dblAz2)
673:     dblTotalLength = dblTotalLength + dblLength
674:   Next lngIndex
  
676:   SpheroidalPolylineLength = dblTotalLength


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineLength " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SpheroidalPolylineLength2(pPolyline As IPolyline, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYLINE IS IN GEOGRAPHIC COORDINATES

  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim lngIndex As Long
  Dim dblLength As Double
  Dim dblTotalLength As Double
  Dim dblAz1 As Double
  Dim dblAz2 As Double
  
701:   dblTotalLength = 0
702:   Set pSegCollection = pPolyline
  
704:   For lngIndex = 0 To (pSegCollection.SegmentCount - 1)
705:     Set pSeg = pSegCollection.Segment(lngIndex)
706:     Set pPoint1 = pSeg.FromPoint
707:     Set pPoint2 = pSeg.ToPoint
708:     dblLength = DistanceVincentyPoints2(pPoint1, pPoint2, dblAz1, dblAz2, dblEquatorialRadius, dblPolarRadius)
709:     dblTotalLength = dblTotalLength + dblLength
710:   Next lngIndex
  
712:   SpheroidalPolylineLength2 = dblTotalLength


  Exit Function
ErrorHandler:
  HandleError True, "SpheroidalPolylineLength2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SphericalPolygonArea2(pPolygon As IPolygon, Optional booCalcCentroid As Boolean = False, _
      Optional dblCentX As Double, Optional dblCentY As Double, _
      Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' REMEMBER THAT THIS STILL CALCULATES ON THE SPHERE BECAUSE I HAVE NOT FIGURED OUT A WAY TO CALCULATE POLYGON AREAS
  ' ON AN ELLISOID.  THE SPHERE IS DEFINED AS THAT SPHERE WITH THE SAME VOLUME AS THE ELLIPSOID WITH THE SPECIFIED
  ' MAJOR AND MINOR AXES.
  '
  ' MODIFICATION OF SphericalPolygonArea TO ALLOW USER TO SET CUSTOM ELLIPSOID MAJOR AND MINOR AXES
  '
  ' ASSUMES POLYGON IS IN GEOGRAPHIC COORDINATES
  ' BREAK UP POLYGON INTO CONNECTED COMPONENTS
  
  Dim pPoly4 As IPolygon4
736:   Set pPoly4 = pPolygon
  Dim pConnected As IGeometryCollection
  Dim pRingBag As IGeometryCollection
  Dim pExtRing As IRing
740:   Set pConnected = pPoly4.ConnectedComponentBag
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim pPointCollection As IPointCollection
  
  Dim pArea As IArea
  Dim pCentroid As IPoint
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  
750:   If booCalcCentroid Then
'    Dim dbl1Long As Double
'    Dim dbl1Lat As Double
    Dim dbl1X As Double
    Dim dbl1Y As Double
    Dim dbl1Z As Double
'    Dim dbl2Long As Double
'    Dim dbl2Lat As Double
    Dim dbl2X As Double
    Dim dbl2Y As Double
    Dim dbl2Z As Double
    Dim dbl3X As Double
    Dim dbl3Y As Double
    Dim dbl3Z As Double
    Dim dblTempCentX As Double
    Dim dblTempCentY As Double
    Dim dblTempCentZ As Double
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    Dim dblRunningZ As Double
770:     dblRunningX = 0
771:     dblRunningY = 0
772:     dblRunningZ = 0
    Dim dblVectLength As Double
774:   End If
  
'  Dim dbl3Long As Double
'  Dim dbl3Lat As Double
  
  Dim pSubPolygon As IPolygon4
  Dim dblArea As Double
  Dim dblTriangleArea As Double
782:   dblArea = 0
  Dim dblMultiplier As Double
  Dim lngCountPos As Long
  Dim lngCountNeg As Long
  
  ' FOR TESTING CENTROID
'  Dim dblTestArea As Double
'  Dim dblTestRunningArea As Double
'  Dim dblTestX As Double
'  Dim dblTestY As Double
'  Dim dblTestRunningX As Double
'  Dim dblTestRunningY As Double
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
797:   For lngIndex = 0 To (pConnected.GeometryCount - 1)
798:     Set pSubPolygon = pConnected.Geometry(lngIndex)
799:     Set pArea = pSubPolygon
800:     Set pCentroid = pArea.Centroid
801:     Set pSegCollection = pSubPolygon
    
'    dbl3Long = pCentroid.X
'    dbl3Lat = pCentroid.Y
    
806:     For lngIndex2 = 0 To (pSegCollection.SegmentCount - 1)
807:       Set pSeg = pSegCollection.Segment(lngIndex2)
808:       Set pPoint1 = pSeg.FromPoint
809:       Set pPoint2 = pSeg.ToPoint
      
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint1, "delete_corridors"
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint2, "delete_corridors"
813:       dblTriangleArea = SphericalTriangleArea2(pPoint1, pPoint2, pCentroid, dblMultiplier, dblEquatorialRadius, dblPolarRadius)
814:       dblArea = dblArea + dblTriangleArea
      
      ' FOR TESTING CENTROID
'      dblTestArea = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, 1, pPoint2.X, pPoint2.Y, _
'                1, pCentroid.X, pCentroid.Y, 1)
'      dblTestRunningArea = dblTestRunningArea + dblTestArea
'      TriangleCentroidPlane pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, pCentroid.X, pCentroid.Y, dblTestX, dblTestY
'      dblTestRunningX = dblTestRunningX + (dblTestX * dblTestArea)
'      dblTestRunningY = dblTestRunningY + (dblTestY * dblTestArea)
      
824:       If booCalcCentroid Then
825:         SphericalLatLongToCart pPoint1.X, pPoint1.Y, dbl1X, dbl1Y, dbl1Z
826:         SphericalLatLongToCart pPoint2.X, pPoint2.Y, dbl2X, dbl2Y, dbl2Z
827:         SphericalLatLongToCart pCentroid.X, pCentroid.Y, dbl3X, dbl3Y, dbl3Z
828:         TriangleCentroid3D dbl1X, dbl1Y, dbl1Z, dbl2X, dbl2Y, dbl2Z, dbl3X, dbl3Y, dbl3Z, _
                dblTempCentX, dblTempCentY, dblTempCentZ
    
        ' NORMALIZE VECTOR
832:         dblVectLength = Sqr(dblTempCentX ^ 2 + dblTempCentY ^ 2 + dblTempCentZ ^ 2)
833:         dblTempCentX = dblTempCentX / dblVectLength
834:         dblTempCentY = dblTempCentY / dblVectLength
835:         dblTempCentZ = dblTempCentZ / dblVectLength
        
837:         dblRunningX = dblRunningX + (dblTempCentX * dblTriangleArea)
838:         dblRunningY = dblRunningY + (dblTempCentY * dblTriangleArea)
839:         dblRunningZ = dblRunningZ + (dblTempCentZ * dblTriangleArea)
840:       End If
841:     Next lngIndex2
842:   Next lngIndex
  
844:   If booCalcCentroid Then
845:     If dblArea > 0 Then
846:       dblRunningX = dblRunningX / dblArea
847:       dblRunningY = dblRunningY / dblArea
848:       dblRunningZ = dblRunningZ / dblArea
      
850:       SphericalCartToLatLong dblCentX, dblCentY, dblRunningX, dblRunningY, dblRunningZ
851:     Else
      
      ' IF AREA = 0 BUT HAS VERTICES, THEN CALCULATE CENTROID AS MULTIPOINT CENTROID?  NO; MIGHT HAVE OVERLAPPING VERTICES THAT WOULD SKEW
      ' IF HAS NO MASS, THEN CANNOT HAVE CENTER OF MASS
      
856:       dblCentX = -9999
857:       dblCentY = -9999
858:     End If
859:   End If
  
  ' FOR TESTING CENTROID
'  dblTestRunningX = dblTestRunningX / dblTestRunningArea
'  dblTestRunningY = dblTestRunningY / dblTestRunningArea
'  Debug.Print "Test Centroid:  X = " & dblTestRunningX & ",  Y = " & dblTestRunningY
  
866:   SphericalPolygonArea2 = dblArea


  Exit Function
ErrorHandler:
  HandleError True, "SphericalPolygonArea2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function SphericalPolygonArea(pPolygon As IPolygon, Optional booCalcCentroid As Boolean = False, _
      Optional dblCentX As Double, Optional dblCentY As Double) As Double
  On Error GoTo ErrorHandler


  ' ASSUMES POLYGON IS IN GEOGRAPHIC COORDINATES
  ' BREAK UP POLYGON INTO CONNECTED COMPONENTS
  Dim pPoly4 As IPolygon4
881:   Set pPoly4 = pPolygon
  Dim pConnected As IGeometryCollection
  Dim pRingBag As IGeometryCollection
  Dim pExtRing As IRing
885:   Set pConnected = pPoly4.ConnectedComponentBag
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  Dim pPointCollection As IPointCollection
  
  Dim pArea As IArea
  Dim pCentroid As IPoint
  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  
895:   If booCalcCentroid Then
'    Dim dbl1Long As Double
'    Dim dbl1Lat As Double
    Dim dbl1X As Double
    Dim dbl1Y As Double
    Dim dbl1Z As Double
'    Dim dbl2Long As Double
'    Dim dbl2Lat As Double
    Dim dbl2X As Double
    Dim dbl2Y As Double
    Dim dbl2Z As Double
    Dim dbl3X As Double
    Dim dbl3Y As Double
    Dim dbl3Z As Double
    Dim dblTempCentX As Double
    Dim dblTempCentY As Double
    Dim dblTempCentZ As Double
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    Dim dblRunningZ As Double
915:     dblRunningX = 0
916:     dblRunningY = 0
917:     dblRunningZ = 0
    Dim dblVectLength As Double
919:   End If
  
'  Dim dbl3Long As Double
'  Dim dbl3Lat As Double
  
  Dim pSubPolygon As IPolygon4
  Dim dblArea As Double
  Dim dblTriangleArea As Double
927:   dblArea = 0
  Dim dblMultiplier As Double
  Dim lngCountPos As Long
  Dim lngCountNeg As Long
  
  ' FOR TESTING CENTROID
'  Dim dblTestArea As Double
'  Dim dblTestRunningArea As Double
'  Dim dblTestX As Double
'  Dim dblTestY As Double
'  Dim dblTestRunningX As Double
'  Dim dblTestRunningY As Double
  
  Dim pSegCollection As ISegmentCollection
  Dim pSeg As ISegment
942:   For lngIndex = 0 To (pConnected.GeometryCount - 1)
943:     Set pSubPolygon = pConnected.Geometry(lngIndex)
944:     Set pArea = pSubPolygon
945:     Set pCentroid = pArea.Centroid
946:     Set pSegCollection = pSubPolygon
    
'    dbl3Long = pCentroid.X
'    dbl3Lat = pCentroid.Y
    
951:     For lngIndex2 = 0 To (pSegCollection.SegmentCount - 1)
952:       Set pSeg = pSegCollection.Segment(lngIndex2)
953:       Set pPoint1 = pSeg.FromPoint
954:       Set pPoint2 = pSeg.ToPoint
      
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint1, "delete_corridors"
'      ThisDocument.Graphic_MakeFromGeometry Document, pPoint2, "delete_corridors"
958:       dblTriangleArea = SphericalTriangleArea(pPoint1, pPoint2, pCentroid, dblMultiplier)
959:       dblArea = dblArea + dblTriangleArea
      
      ' FOR TESTING CENTROID
'      dblTestArea = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, 1, pPoint2.X, pPoint2.Y, _
'                1, pCentroid.X, pCentroid.Y, 1)
'      dblTestRunningArea = dblTestRunningArea + dblTestArea
'      TriangleCentroidPlane pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, pCentroid.X, pCentroid.Y, dblTestX, dblTestY
'      dblTestRunningX = dblTestRunningX + (dblTestX * dblTestArea)
'      dblTestRunningY = dblTestRunningY + (dblTestY * dblTestArea)
      
969:       If booCalcCentroid Then
970:         SphericalLatLongToCart pPoint1.X, pPoint1.Y, dbl1X, dbl1Y, dbl1Z
971:         SphericalLatLongToCart pPoint2.X, pPoint2.Y, dbl2X, dbl2Y, dbl2Z
972:         SphericalLatLongToCart pCentroid.X, pCentroid.Y, dbl3X, dbl3Y, dbl3Z
973:         TriangleCentroid3D dbl1X, dbl1Y, dbl1Z, dbl2X, dbl2Y, dbl2Z, dbl3X, dbl3Y, dbl3Z, _
                dblTempCentX, dblTempCentY, dblTempCentZ
    
        ' NORMALIZE VECTOR
977:         dblVectLength = Sqr(dblTempCentX ^ 2 + dblTempCentY ^ 2 + dblTempCentZ ^ 2)
978:         dblTempCentX = dblTempCentX / dblVectLength
979:         dblTempCentY = dblTempCentY / dblVectLength
980:         dblTempCentZ = dblTempCentZ / dblVectLength
        
982:         dblRunningX = dblRunningX + (dblTempCentX * dblTriangleArea)
983:         dblRunningY = dblRunningY + (dblTempCentY * dblTriangleArea)
984:         dblRunningZ = dblRunningZ + (dblTempCentZ * dblTriangleArea)
985:       End If
986:     Next lngIndex2
987:   Next lngIndex
  
989:   If booCalcCentroid Then
990:     If dblArea > 0 Then
991:       dblRunningX = dblRunningX / dblArea
992:       dblRunningY = dblRunningY / dblArea
993:       dblRunningZ = dblRunningZ / dblArea
      
995:       SphericalCartToLatLong dblCentX, dblCentY, dblRunningX, dblRunningY, dblRunningZ
996:     Else
      
      ' IF AREA = 0 BUT HAS VERTICES, THEN CALCULATE CENTROID AS MULTIPOINT CENTROID?  NO; MIGHT HAVE OVERLAPPING VERTICES THAT WOULD SKEW
      ' IF HAS NO MASS, THEN CANNOT HAVE CENTER OF MASS
      
1001:       dblCentX = -9999
1002:       dblCentY = -9999
1003:     End If
1004:   End If
  
  ' FOR TESTING CENTROID
'  dblTestRunningX = dblTestRunningX / dblTestRunningArea
'  dblTestRunningY = dblTestRunningY / dblTestRunningArea
'  Debug.Print "Test Centroid:  X = " & dblTestRunningX & ",  Y = " & dblTestRunningY
  
1011:   SphericalPolygonArea = dblArea


  Exit Function
ErrorHandler:
  HandleError True, "SphericalPolygonArea " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AzimuthHaversine(pPointA As IPoint, pPointB As IPoint) As Double
  On Error GoTo ErrorHandler


  ' WITH HELP FROM http://www.movable-type.co.uk/scripts/latlong.html
  ' Formula:    Theta = atan2( sin(Deltalong)*cos(lat2), cos(lat1)*sin(lat2) - sin(lat1)*cos(lat2)*cos(DeltaLong) )
  
  Dim DeltaLong As Double
  Dim PX As Double
  Dim PY As Double
  Dim QX As Double
  Dim QY As Double
  
1032:   PX = DegToRad(pPointA.X)
1033:   PY = DegToRad(pPointA.Y)
1034:   QX = DegToRad(pPointB.X)
1035:   QY = DegToRad(pPointB.Y)
  
1037:   DeltaLong = QX - PX
  Dim dblTheta As Double
1039:   dblTheta = atan2(Sin(DeltaLong) * Cos(QY), Cos(PY) * Sin(QY) - Sin(PY) * Cos(QY) * Cos(DeltaLong))
1040:   AzimuthHaversine = RadToDeg(dblTheta)
1041:   If AzimuthHaversine < 360 Then AzimuthHaversine = AzimuthHaversine + 360


  Exit Function
ErrorHandler:
  HandleError True, "AzimuthHaversine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function SphericalTriangleArea2(pPointA As IPoint, pPointB As IPoint, pPointC As IPoint, Optional dblMult As Double, _
    Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler


  ' MODIFICATION OF SphericalTriangleArea, TO ALLOW USER TO OPTIONALLY SEND CUSTOM MAJOR AND MINOR ELLIPSOID AXES
  
  ' BASED ON GIRARD'S FORMULA:  Area = R^2 * (A + B + C - Pi)
  '                          Where A = Angle 1
  '                                B = Angle 2
  '                                C = Angle 3
  '                   A + B + C - Pi = Spherical Excess
  '                                R = Sphere Radius
  ' Trick is to get Angles A, B and C from points.
  '
  ' ANOTHER FORMULATION, BASED ON DISTANCES:
  '                       Tan(E / 4) = sqrt(Tan(S / 2) * Tan((S - A) / 2) * Tan((S - B) / 2) * Tan((S - C) / 2))
  '                 Spherical Excess = E
  '                   where  a, b, c = sides of spherical triangle
  '                                S = (A + B + C) / 2
  ' INITAL AZIMUTH = atn( sin (Lo2 - Lo1) / (cos (Lo2 - Lo1) sin L1 - cos L1 tan L2)
  '         http://fer3.com/arc/m2.aspx?i=1688&y=200111
  
  ' FOR DEBUGGING
'  Static dblMax As Double
'  Static dblMin As Double
  
1076:   If Abs(pPointA.X - pPointB.X) < 0.000000000001 And Abs(pPointA.X - pPointC.X) < 0.000000000001 Then
1077:     SphericalTriangleArea2 = 0
    Exit Function
1079:   End If
  
  Dim dblAB As Double
  Dim dblBC As Double
  Dim dblCA As Double
  
  Dim dblR As Double         ' RADIUS
1086:   dblR = (dblEquatorialRadius ^ 2 * dblPolarRadius) ^ (1 / 3)   ' PROPER 3-AXIS GEOMETRIC MEAN; (a^2 * b) ^ (1/3)
  
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblC As Double
  Dim dblAzimuthAB As Double
  Dim dblAzimuthBC As Double
  Dim dblAzimuthAC As Double
  Dim dblAzimuthAA As Double
  Dim dblAzRev As Double
  Dim dblMultiplier As Double
  Dim dblLong2 As Double
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1103:   dblLat1 = DegToRad(pPointA.Y)
1104:   dblLat2 = DegToRad(pPointB.Y)
1105:   dblLat = dblLat1 - dblLat2
1106:   dblLong = DegToRad(pPointA.X - pPointB.X)
1107:   dblLong2 = -dblLong
1108:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1109:   dblAB = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO B
'  DistanceVincentyNumbers pPointA.X, pPointA.Y, pPointB.X, pPointB.Y, dblAzimuthAB, dblAzRev
'  dblAB = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointB.x, pPointB.y, dblAzimuthAB, dblAzRev) / dblR
'  Debug.Print "A to B:  Vincenty = " & dblAzimuthAB
1114:   dblAzimuthAB = atan2(Sin(-dblLong) * Cos(dblLat2), _
        Cos(dblLat1) * Sin(dblLat2) - Sin(dblLat1) * Cos(dblLat2) * Cos(-dblLong))
'  Debug.Print "A to B:  Simpler = " & dblAzimuthAB
'  dblAB = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE BC USING HAVERSINE FORMULA
1120:   dblLat1 = DegToRad(pPointB.Y)
1121:   dblLat2 = DegToRad(pPointC.Y)
1122:   dblLat = dblLat1 - dblLat2
1123:   dblLong = DegToRad(pPointB.X - pPointC.X)
1124:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1125:   dblBC = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1131:   dblLat1 = DegToRad(pPointC.Y)
1132:   dblLat2 = DegToRad(pPointA.Y)
1133:   dblLat = dblLat1 - dblLat2
1134:   dblLong = DegToRad(pPointC.X - pPointA.X)
  
1136:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1137:   dblCA = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO C
'  DistanceVincentyNumbers pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev
'  dblCA = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev) / dblR
'  Debug.Print "A to C:  Vincenty = " & dblAzimuth
'  dblAzimuth2 = RadToDeg(atan2(Sin(pPointC.x - pPointA.x), _
                    (Cos(pPointC.x - pPointA.x) * Sin(pPointA.y) - Cos(pPointA.y) * Tan(pPointC.y))))
'  dblCA = dblR * dblC
'  Debug.Print "Az1 = " & CStr(dblAzimuth1) & ",       Az2 = " & CStr(dblAzimuth2)

  ' NOTE:  Lat1 and Lat2 flipped in equation below because variables originally defined for Line CA, not Line AC
1148:   dblAzimuthAC = atan2(Sin(dblLong) * Cos(dblLat1), _
        Cos(dblLat2) * Sin(dblLat1) - Sin(dblLat2) * Cos(dblLat1) * Cos(dblLong))
'  Debug.Print "A to C:  Simpler = " & dblAzimuthAC
  
'  If dblAzimuthAB > dblMax Then dblMax = dblAzimuthAB
'  If dblAzimuthAB < dblMin Then dblMin = dblAzimuthAB
'  Debug.Print "Current Max = " & dblMax & ",   Current Min = " & dblMin
  
1156:   If dblAzimuthAB < 0 Then dblAzimuthAB = dblAzimuthAB + 2 * dblPi
1157:   If dblAzimuthAC < 0 Then dblAzimuthAC = dblAzimuthAC + 2 * dblPi
  
  Dim dblDiff As Double
1160:   dblDiff = dblAzimuthAC - dblAzimuthAB
1161:   If dblDiff > 0 Then              ' EITHER AC > AB or AC IS TO THE LEFT OF NORTH
1162:     If dblDiff > dblPi Then         ' THEN AC IS TO THE LEFT OF NORTH
1163:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1164:     Else                           ' THEN AC > AB
1165:       dblMultiplier = 1            ' CLOCKWISE
1166:     End If
1167:   Else                             ' EITHER AC < AB or AB IS TO THE LEFT OF NORTH
1168:     If Abs(dblDiff) > dblPi Then   ' THEN AB IS TO THE LEFT OF NORTH
1169:       dblMultiplier = 1            ' CLOCKWISE
1170:     Else                           ' THEN AC < AB
1171:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1172:     End If
1173:   End If
    
  
'  If ((dblDiff > dblPi) And (dblAzimuthAB > dblAzimuthAC)) Or ((dblDiff < dblPi) And (dblAzimuthAB < dblAzimuthAC)) Then
'    ' IS CLOCKWISE
'    dblMultiplier = 1
'  Else
'    ' IS COUNTERCLOCKWISE
'    dblMultiplier = -1
'  End If

'  If Abs(dblAzimuthAC - dblAzimuthAB) > dblPi Then dblAzimuthAB = dblAzimuth
'  dblAzimuthAB = dblAzimuthAB + 360
'  dblAzimuthAC = dblAzimuthAC + 360
'  If dblAzimuthAB > dblAzimuthAC Then
'    dblMultiplier = -1
'  Else
'    dblMultiplier = 1
'  End If
  
'  Dim booIsClockwise As Boolean
'  booIsClockwise = CalcCheckClockwise(pPointA, pPointB, pPointC)
'  If booIsClockwise Then
'    If dblMultiplier = -1 Then
'      Debug.Print "-1, 1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",   AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  Else
'    If dblMultiplier = 1 Then
'      Debug.Print "1, -1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",  AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  End If
  
  Dim dblS As Double
1212:   dblS = (dblAB + dblBC + dblCA) / 2
  
  Dim dblTanEOver4 As Double
1215:   dblTanEOver4 = Sqr(Tan(dblS / 2) * Tan((dblS - dblAB) / 2) * Tan((dblS - dblBC) / 2) * Tan((dblS - dblCA) / 2))
  Dim dblE As Double
1217:   dblE = Atn(dblTanEOver4) * 4
  
1219:   dblMult = dblMultiplier
1220:   SphericalTriangleArea2 = dblR ^ 2 * dblE * dblMultiplier


  Exit Function
ErrorHandler:
  HandleError True, "SphericalTriangleArea2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function SphericalTriangleArea(pPointA As IPoint, pPointB As IPoint, pPointC As IPoint, Optional dblMult As Double) As Double
  On Error GoTo ErrorHandler

  
  
  ' BASED ON GIRARD'S FORMULA:  Area = R^2 * (A + B + C - Pi)
  '                          Where A = Angle 1
  '                                B = Angle 2
  '                                C = Angle 3
  '                   A + B + C - Pi = Spherical Excess
  '                                R = Sphere Radius
  ' Trick is to get Angles A, B and C from points.
  '
  ' ANOTHER FORMULATION, BASED ON DISTANCES:
  '                       Tan(E / 4) = sqrt(Tan(S / 2) * Tan((S - A) / 2) * Tan((S - B) / 2) * Tan((S - C) / 2))
  '                 Spherical Excess = E
  '                   where  a, b, c = sides of spherical triangle
  '                                S = (A + B + C) / 2
  ' INITAL AZIMUTH = atn( sin (Lo2 - Lo1) / (cos (Lo2 - Lo1) sin L1 - cos L1 tan L2)
  '         http://fer3.com/arc/m2.aspx?i=1688&y=200111
  
  ' FOR DEBUGGING
'  Static dblMax As Double
'  Static dblMin As Double
  
1254:   If Abs(pPointA.X - pPointB.X) < 0.000000000001 And Abs(pPointA.X - pPointC.X) < 0.000000000001 Then
1255:     SphericalTriangleArea = 0
    Exit Function
1257:   End If
  
  Dim dblAB As Double
  Dim dblBC As Double
  Dim dblCA As Double
  
  Dim dblR As Double         ' RADIUS
'  dblR = (6378137 + 6356752.31424518) / 2 ' AVERAGE OF WGS ELLIPSOID MAJOR AND MINOR AXES
'  dblR = Sqr(6378137 * 6356752.31424518)  ' GEOMETRIC MEAN OF WGS ELLIPSOID MAJOR AND MINOR AXES
1266:   dblR = (6378137 ^ 2 * 6356752.31424518) ^ (1 / 3)   ' PROPER 3-AXIS GEOMETRIC MEAN; (a^2 * b) ^ (1/3)
  
  Dim dblLat As Double
  Dim dblLong As Double
  Dim dblTemp As Double
  Dim dblLat1 As Double
  Dim dblLat2 As Double
  Dim dblC As Double
  Dim dblAzimuthAB As Double
  Dim dblAzimuthBC As Double
  Dim dblAzimuthAC As Double
  Dim dblAzimuthAA As Double
  Dim dblAzRev As Double
  Dim dblMultiplier As Double
  Dim dblLong2 As Double
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1283:   dblLat1 = DegToRad(pPointA.Y)
1284:   dblLat2 = DegToRad(pPointB.Y)
1285:   dblLat = dblLat1 - dblLat2
1286:   dblLong = DegToRad(pPointA.X - pPointB.X)
1287:   dblLong2 = -dblLong
1288:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1289:   dblAB = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO B
'  DistanceVincentyNumbers pPointA.X, pPointA.Y, pPointB.X, pPointB.Y, dblAzimuthAB, dblAzRev
'  dblAB = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointB.x, pPointB.y, dblAzimuthAB, dblAzRev) / dblR
'  Debug.Print "A to B:  Vincenty = " & dblAzimuthAB
1294:   dblAzimuthAB = atan2(Sin(-dblLong) * Cos(dblLat2), _
        Cos(dblLat1) * Sin(dblLat2) - Sin(dblLat1) * Cos(dblLat2) * Cos(-dblLong))
'  Debug.Print "A to B:  Simpler = " & dblAzimuthAB
'  dblAB = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE BC USING HAVERSINE FORMULA
1300:   dblLat1 = DegToRad(pPointB.Y)
1301:   dblLat2 = DegToRad(pPointC.Y)
1302:   dblLat = dblLat1 - dblLat2
1303:   dblLong = DegToRad(pPointB.X - pPointC.X)
1304:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1305:   dblBC = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = DistanceVincentyNumbers(pPointB.x, pPointB.y, pPointC.x, pPointC.y, dblAzimuthBC, dblAzRev) / dblR
'  dblBC = dblR * dblC
  
  ' CALCULATE LENGTH OF GEOCURVE AB USING HAVERSINE FORMULA
1311:   dblLat1 = DegToRad(pPointC.Y)
1312:   dblLat2 = DegToRad(pPointA.Y)
1313:   dblLat = dblLat1 - dblLat2
1314:   dblLong = DegToRad(pPointC.X - pPointA.X)
  
1316:   dblTemp = (Sin(dblLat / 2)) ^ 2 + Cos(dblLat1) * Cos(dblLat2) * (Sin(dblLong / 2)) ^ 2
1317:   dblCA = 2 * atan2(Sqr(dblTemp), Sqr(1 - dblTemp))
  ' AZIMUTH FROM A TO C
'  DistanceVincentyNumbers pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev
'  dblCA = DistanceVincentyNumbers(pPointA.x, pPointA.y, pPointC.x, pPointC.y, dblAzimuthAC, dblAzRev) / dblR
'  Debug.Print "A to C:  Vincenty = " & dblAzimuth
'  dblAzimuth2 = RadToDeg(atan2(Sin(pPointC.x - pPointA.x), _
                    (Cos(pPointC.x - pPointA.x) * Sin(pPointA.y) - Cos(pPointA.y) * Tan(pPointC.y))))
'  dblCA = dblR * dblC
'  Debug.Print "Az1 = " & CStr(dblAzimuth1) & ",       Az2 = " & CStr(dblAzimuth2)

  ' NOTE:  Lat1 and Lat2 flipped in equation below because variables originally defined for Line CA, not Line AC
1328:   dblAzimuthAC = atan2(Sin(dblLong) * Cos(dblLat1), _
        Cos(dblLat2) * Sin(dblLat1) - Sin(dblLat2) * Cos(dblLat1) * Cos(dblLong))
'  Debug.Print "A to C:  Simpler = " & dblAzimuthAC
  
'  If dblAzimuthAB > dblMax Then dblMax = dblAzimuthAB
'  If dblAzimuthAB < dblMin Then dblMin = dblAzimuthAB
'  Debug.Print "Current Max = " & dblMax & ",   Current Min = " & dblMin
  
1336:   If dblAzimuthAB < 0 Then dblAzimuthAB = dblAzimuthAB + 2 * dblPi
1337:   If dblAzimuthAC < 0 Then dblAzimuthAC = dblAzimuthAC + 2 * dblPi
  
  Dim dblDiff As Double
1340:   dblDiff = dblAzimuthAC - dblAzimuthAB
1341:   If dblDiff > 0 Then              ' EITHER AC > AB or AC IS TO THE LEFT OF NORTH
1342:     If dblDiff > dblPi Then         ' THEN AC IS TO THE LEFT OF NORTH
1343:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1344:     Else                           ' THEN AC > AB
1345:       dblMultiplier = 1            ' CLOCKWISE
1346:     End If
1347:   Else                             ' EITHER AC < AB or AB IS TO THE LEFT OF NORTH
1348:     If Abs(dblDiff) > dblPi Then   ' THEN AB IS TO THE LEFT OF NORTH
1349:       dblMultiplier = 1            ' CLOCKWISE
1350:     Else                           ' THEN AC < AB
1351:       dblMultiplier = -1           ' COUNTERCLOCKWISE
1352:     End If
1353:   End If
    
  
'  If ((dblDiff > dblPi) And (dblAzimuthAB > dblAzimuthAC)) Or ((dblDiff < dblPi) And (dblAzimuthAB < dblAzimuthAC)) Then
'    ' IS CLOCKWISE
'    dblMultiplier = 1
'  Else
'    ' IS COUNTERCLOCKWISE
'    dblMultiplier = -1
'  End If

'  If Abs(dblAzimuthAC - dblAzimuthAB) > dblPi Then dblAzimuthAB = dblAzimuth
'  dblAzimuthAB = dblAzimuthAB + 360
'  dblAzimuthAC = dblAzimuthAC + 360
'  If dblAzimuthAB > dblAzimuthAC Then
'    dblMultiplier = -1
'  Else
'    dblMultiplier = 1
'  End If
  
'  Dim booIsClockwise As Boolean
'  booIsClockwise = CalcCheckClockwise(pPointA, pPointB, pPointC)
'  If booIsClockwise Then
'    If dblMultiplier = -1 Then
'      Debug.Print "-1, 1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",   AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  Else
'    If dblMultiplier = 1 Then
'      Debug.Print "1, -1:  AB = " & CStr(RadToDeg(dblAzimuthAB)) & ",  AC = " & CStr(RadToDeg(dblAzimuthAC)) & vbCrLf & _
'                  "  --> Point A:  X = " & pPointA.x & ",    Y = " & pPointA.y & vbCrLf & _
'                  "  --> Point B:  X = " & pPointB.x & ",    Y = " & pPointB.y & vbCrLf & _
'                  "  --> Point C:  X = " & pPointC.x & ",    Y = " & pPointC.y
'    End If
'  End If
  
  Dim dblS As Double
1392:   dblS = (dblAB + dblBC + dblCA) / 2
  
  Dim dblTanEOver4 As Double
1395:   dblTanEOver4 = Sqr(Tan(dblS / 2) * Tan((dblS - dblAB) / 2) * Tan((dblS - dblBC) / 2) * Tan((dblS - dblCA) / 2))
  Dim dblE As Double
1397:   dblE = Atn(dblTanEOver4) * 4
  
1399:   dblMult = dblMultiplier
1400:   SphericalTriangleArea = dblR ^ 2 * dblE * dblMultiplier


  Exit Function
ErrorHandler:
  HandleError True, "SphericalTriangleArea " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function ArcSinJen(dblValue As Double) As Double
  On Error GoTo ErrorHandler


1412:   ArcSinJen = Atn(dblValue / Sqr(-dblValue * dblValue + 1))


  Exit Function
ErrorHandler:
  HandleError True, "ArcSinJen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function ArcCosJen(dblValue As Double) As Double
  On Error GoTo ErrorHandler


1423:   ArcCosJen = Atn(-dblValue / Sqr(-dblValue * dblValue + 1)) + 2 * Atn(1)


  Exit Function
ErrorHandler:
  HandleError True, "ArcCosJen " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyPoints(pPoint1 As IPoint, pPoint2 As IPoint, dblAz1 As Double, dblAz2 As Double) As Double
  On Error GoTo ErrorHandler


1435:   DistanceVincentyPoints = DistanceVincentyNumbers(pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, dblAz1, dblAz2)


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyPoints2(pPoint1 As IPoint, pPoint2 As IPoint, dblAz1 As Double, dblAz2 As Double, _
  Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF DistanceVincentyPoints, TO ALLOW FOR ANY ELLIPSOID
  
1450:   DistanceVincentyPoints2 = DistanceVincentyNumbers2(pPoint1.X, pPoint1.Y, pPoint2.X, pPoint2.Y, dblAz1, dblAz2, _
        dblEquatorialRadius, dblPolarRadius)


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyPoints2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub PointLineVincenty(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, pNewPoint As IPoint, _
      dblAz2 As Double, Optional lngNumVertices As Long, Optional pPolyline As IPolyline)
  On Error GoTo ErrorHandler

      
  Dim pWGS84 As IGeographicCoordinateSystem
  Dim pSpatRefFact As ISpatialReferenceFactory
  'Set the spatial reference factory to a new spatial reference environment
1467:   Set pSpatRefFact = New SpatialReferenceEnvironment
1468:   Set pWGS84 = pSpatRefFact.CreateGeographicCoordinateSystem(esriSRGeoCS_WGS1984)
  
1470:   If lngNumVertices > 0 Then
1471:     If lngNumVertices = 1 Then lngNumVertices = 2
    
    Dim dblShort As Double
1474:     dblShort = dblLength / (lngNumVertices - 1)
    
    Dim pPointCollection As IPointCollection
1477:     If pPolyline Is Nothing Then
1478:       Set pPolyline = New Polyline
1479:     End If
1480:     Set pPointCollection = pPolyline
    
    ' ADD FIRST VERTEX
1483:     pPointCollection.AddPoint pPoint
    
    ' ADD INTERNAL VERTICES
1486:     If lngNumVertices > 2 Then
      Dim lngCounter As Long
      Dim dblCurrentDistance As Double
      Dim pTempPoint As IPoint
1490:       For lngCounter = 1 To (lngNumVertices - 2)
1491:         Set pTempPoint = New Point
1492:         PointLineVincentyPerPoint pPoint, lngCounter * dblShort, dblAzimuth, pTempPoint, dblAz2
1493:         pPointCollection.AddPoint pTempPoint
1494:       Next lngCounter
1495:     End If
    
    ' ADD LAST VERTEX AND SET FINAL POINT AND AZIMUTH VALUES
1498:     PointLineVincentyPerPoint pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2
1499:     pPointCollection.AddPoint pNewPoint
1500:     Set pPolyline.SpatialReference = pWGS84
1501:     Set pNewPoint.SpatialReference = pWGS84
1502:   Else
1503:     PointLineVincentyPerPoint pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2
1504:     Set pNewPoint.SpatialReference = pWGS84
1505:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincenty " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub PointLineVincenty2(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, pNewPoint As IPoint, _
      dblAz2 As Double, Optional lngNumVertices As Long, Optional pPolyline As IPolyline)
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF PointLineVincenty, TO ALLOW FOR ANY ELLIPSOID
  ' ASSUMES INCOMING POINT IS IN GEOGRAPHIC COORDINATE SYSTEM
  
  Dim pGCS As IGeographicCoordinateSystem
  Dim pSpRef As ISpatialReference
  
1524:   Set pSpRef = pPoint.SpatialReference
1525:   If TypeOf pSpRef Is IGeographicCoordinateSystem Then
1526:     Set pGCS = pSpRef
1527:   Else
    Dim pPrjCS As IProjectedCoordinateSystem
1529:     Set pPrjCS = pSpRef
1530:     Set pGCS = pPrjCS.GeographicCoordinateSystem
1531:   End If
  
  Dim dblEquatorialRadius As Double
  Dim dblPolarRadius As Double
  Dim pEllipsoid As ISpheroid
  
1537:   Set pEllipsoid = pGCS.Datum.Spheroid
1538:   dblEquatorialRadius = pEllipsoid.SemiMajorAxis
1539:   dblPolarRadius = pEllipsoid.SemiMinorAxis
  
1541:   If lngNumVertices > 0 Then
1542:     If lngNumVertices = 1 Then lngNumVertices = 2
    
    Dim dblShort As Double
1545:     dblShort = dblLength / (lngNumVertices - 1)
    
    Dim pPointCollection As IPointCollection
1548:     If pPolyline Is Nothing Then
1549:       Set pPolyline = New Polyline
1550:     End If
1551:     Set pPointCollection = pPolyline
    
    ' ADD FIRST VERTEX
1554:     pPointCollection.AddPoint pPoint
    
    ' ADD INTERNAL VERTICES
1557:     If lngNumVertices > 2 Then
      Dim lngCounter As Long
      Dim dblCurrentDistance As Double
      Dim pTempPoint As IPoint
1561:       For lngCounter = 1 To (lngNumVertices - 2)
1562:         Set pTempPoint = New Point
1563:         PointLineVincentyPerPoint2 pPoint, lngCounter * dblShort, dblAzimuth, pTempPoint, dblAz2, dblEquatorialRadius, dblPolarRadius
1564:         pPointCollection.AddPoint pTempPoint
1565:       Next lngCounter
1566:     End If
    
    ' ADD LAST VERTEX AND SET FINAL POINT AND AZIMUTH VALUES
1569:     PointLineVincentyPerPoint2 pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2, dblEquatorialRadius, dblPolarRadius
1570:     pPointCollection.AddPoint pNewPoint
1571:     Set pPolyline.SpatialReference = pGCS
1572:     Set pNewPoint.SpatialReference = pGCS
1573:   Else
1574:     PointLineVincentyPerPoint2 pPoint, dblLength, dblAzimuth, pNewPoint, dblAz2, dblEquatorialRadius, dblPolarRadius
1575:     Set pNewPoint.SpatialReference = pGCS
1576:   End If


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincenty2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Sub PointLineVincentyPerPoint2(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, _
      pNewPoint As IPoint, dblAz2 As Double, _
      Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518)
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF PointLineVincentyPerPoint, TO ALLOW FOR ANY ELLIPSOID
  
  ' ASSUMES pPoint IS GEOGRAPHIC

  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  Dim dblPX As Double
1601:   dblPX = pPoint.X
  Dim dblPY As Double
1603:   dblPY = pPoint.Y
  
1605:   If dblLength = 0 Then    ' SAME POINT
1606:     pNewPoint.X = dblPX
1607:     pNewPoint.Y = dblPY
1608:     dblAz2 = dblAzimuth
    Exit Sub
1610:   End If
  
  ' WGS84 PARAMETERS ----------------------------------------
  Dim A As Double
  Dim B As Double
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim f As Double
  Dim dblA1 As Double
  Dim dblB1 As Double
  
  Dim dblTanU1 As Double
  Dim dblSinU1 As Double
  Dim dblCosU1 As Double
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
'  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
'  U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
1630:   A = dblEquatorialRadius   ' SPHEROID; EQUATORIAL RADIUS
1631:   B = dblPolarRadius        ' SPHEROID; POLAR RADIUS
1632:   f = (A - B) / B           ' FLATTENING
  
1634:   dblTanU1 = (1 - f) * (Tan(DegToRad(dblPY)))
1635:   U1 = Atn(dblTanU1)
1636:   dblCosU1 = Cos(U1)
1637:   dblSinU1 = Sin(U1)
  
  Dim S As Double
1640:   S = dblLength
  
  Dim Sigma1 As Double
  Dim tanSigma1 As Double
  Dim cosAlpha1 As Double
  Dim sinAlpha1 As Double
'  Dim dblAlpha As Double                      ' AZIMUTH AT EQUATOR
  Dim cosAlpha As Double
  Dim cosSqAlpha As Double
  
1650:   cosAlpha1 = Cos(DegToRad(dblAzimuth))
1651:   sinAlpha1 = Sin(DegToRad(dblAzimuth))
1652:   tanSigma1 = dblTanU1 / cosAlpha1                                                                    ' [1]
1653:   Sigma1 = atan2(dblTanU1, cosAlpha1)
  Dim sinAlpha As Double
1655:   sinAlpha = dblCosU1 * sinAlpha1                                                                  ' [2]
1656:   cosSqAlpha = 1 - (sinAlpha ^ 2)                                                                  ' TRIG IDENTITY
1657:   cosAlpha = Sqr(cosSqAlpha)
'  dblAlpha = ArcSinJen(sinAlpha)
  
1660:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
1661:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
1662:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                                      ' [3]
1663:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
1664:   dblB = (uSq / 1024) * (256 + dblB1)                                                              ' [4]
  
  Dim Sigma As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim DeltaSigma As Double
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  Dim l As Double
  
  Dim lngIterations As Long
1678:   lngIterations = 40
  
  Dim SigmaCompare As Double
1681:   SigmaCompare = 2 * dblPi
1682:   Sigma = S / (B * dblA)                  ' FIRST ESTIMATION
  
1684:   Do While (Abs(Sigma - SigmaCompare) > 0.000000000001) And (lngIterations > 0)
1685:     cos2SigmaM = Cos(2 * Sigma1 + Sigma)                                                             ' [5]
1686:     sinSigma = Sin(Sigma)
1687:     cosSigma = Cos(Sigma)
1688:     DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
1689:     DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
1690:     DeltaSigma3 = cos2SigmaM + DeltaSigma2
1691:     DeltaSigma = dblB * sinSigma * DeltaSigma3                                                       ' [6]
1692:     SigmaCompare = Sigma
1693:     Sigma = (S / (B * dblA)) + DeltaSigma                                                            ' [7]
    
1695:     If lngIterations = 0 Then
1696:       MsgBox "Vincenty Formula failed to converge!"
      Exit Sub
1698:     End If
1699:     lngIterations = lngIterations - 1
1700:   Loop
  
1702:   cos2SigmaM = Cos(2 * Sigma1 + Sigma)
1703:   sinSigma = Sin(Sigma)
1704:   cosSigma = Cos(Sigma)
  Dim dblLat2Denom As Double
  Dim dblLat2Temp As Double
1707:   dblLat2Temp = dblSinU1 * sinSigma - dblCosU1 * cosSigma * cosAlpha1
1708:   dblLat2Denom = (1 - f) * (Sqr(sinAlpha ^ 2 + dblLat2Temp ^ 2))
  
  ' CALCULATE LATITUDE FOR NEW POINT
  Dim dblLat2 As Double
1712:   dblLat2 = atan2(dblSinU1 * cosSigma + dblCosU1 * sinSigma * cosAlpha1, dblLat2Denom)                ' [8]
  
  ' CALCULATE LONGITUDE FOR NEW POINT
  Dim dblLambda As Double
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
1718:   dblLambda = atan2(sinSigma * sinAlpha1, dblCosU1 * cosSigma - dblSinU1 * sinSigma * cosAlpha1)      ' [9]
1719:   C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                                      ' [10]
1720:   dblLambda1 = cos2SigmaM + C * cosSigma * (-1# + 2# * cos2SigmaM ^ 2#)
1721:   dblLambda1a = C * sinSigma * dblLambda1
  Dim dblLambda2 As Double
1723:   dblLambda2 = Sigma + dblLambda1a
1724:   l = dblLambda - ((1 - C) * f * sinAlpha * dblLambda2)                                               ' [11]
  
1726:   pNewPoint.X = dblPX + RadToDeg(l)
1727:   pNewPoint.Y = RadToDeg(dblLat2)
  
1729:   dblAz2 = RadToDeg(atan2(sinAlpha, -dblLat2Temp))
1730:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincentyPerPoint2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub PointLineVincentyPerPoint(pPoint As IPoint, dblLength As Double, dblAzimuth As Double, _
      pNewPoint As IPoint, dblAz2 As Double)
  On Error GoTo ErrorHandler

      
  ' ASSUMES pPoint IS GEOGRAPHIC

  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  Dim dblPX As Double
1753:   dblPX = pPoint.X
  Dim dblPY As Double
1755:   dblPY = pPoint.Y
  
1757:   If dblLength = 0 Then    ' SAME POINT
1758:     pNewPoint.X = dblPX
1759:     pNewPoint.Y = dblPY
1760:     dblAz2 = dblAzimuth
    Exit Sub
1762:   End If
  
  ' WGS84 PARAMETERS ----------------------------------------
  Dim A As Double
  Dim B As Double
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim f As Double
  Dim dblA1 As Double
  Dim dblB1 As Double
  
  Dim dblTanU1 As Double
  Dim dblSinU1 As Double
  Dim dblCosU1 As Double
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
'  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
'  U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
1782:   A = 6378137               ' WGS84 SPHEROID; EQUATORIAL RADIUS
1783:   B = 6356752.31424518      ' WGS84 SPHEROID; POLAR RADIUS
1784:   f = 1 / 298.257223563     ' WGS84 SPHEROID; FLATTENING
  
1786:   dblTanU1 = (1 - f) * (Tan(DegToRad(dblPY)))
1787:   U1 = Atn(dblTanU1)
1788:   dblCosU1 = Cos(U1)
1789:   dblSinU1 = Sin(U1)
  
  Dim S As Double
1792:   S = dblLength
  
  Dim Sigma1 As Double
  Dim tanSigma1 As Double
  Dim cosAlpha1 As Double
  Dim sinAlpha1 As Double
'  Dim dblAlpha As Double                      ' AZIMUTH AT EQUATOR
  Dim cosAlpha As Double
  Dim cosSqAlpha As Double
  
1802:   cosAlpha1 = Cos(DegToRad(dblAzimuth))
1803:   sinAlpha1 = Sin(DegToRad(dblAzimuth))
1804:   tanSigma1 = dblTanU1 / cosAlpha1                                                                    ' [1]
1805:   Sigma1 = atan2(dblTanU1, cosAlpha1)
  Dim sinAlpha As Double
1807:   sinAlpha = dblCosU1 * sinAlpha1                                                                  ' [2]
1808:   cosSqAlpha = 1 - (sinAlpha ^ 2)                                                                  ' TRIG IDENTITY
1809:   cosAlpha = Sqr(cosSqAlpha)
'  dblAlpha = ArcSinJen(sinAlpha)
  
1812:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
1813:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
1814:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                                      ' [3]
1815:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
1816:   dblB = (uSq / 1024) * (256 + dblB1)                                                              ' [4]
  
  Dim Sigma As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim DeltaSigma As Double
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  Dim l As Double
  
  Dim lngIterations As Long
1830:   lngIterations = 40
  
  Dim SigmaCompare As Double
1833:   SigmaCompare = 2 * dblPi
1834:   Sigma = S / (B * dblA)                  ' FIRST ESTIMATION
  
1836:   Do While (Abs(Sigma - SigmaCompare) > 0.000000000001) And (lngIterations > 0)
1837:     cos2SigmaM = Cos(2 * Sigma1 + Sigma)                                                             ' [5]
1838:     sinSigma = Sin(Sigma)
1839:     cosSigma = Cos(Sigma)
1840:     DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
1841:     DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
1842:     DeltaSigma3 = cos2SigmaM + DeltaSigma2
1843:     DeltaSigma = dblB * sinSigma * DeltaSigma3                                                       ' [6]
1844:     SigmaCompare = Sigma
1845:     Sigma = (S / (B * dblA)) + DeltaSigma                                                            ' [7]
    
1847:     If lngIterations = 0 Then
1848:       MsgBox "Vincenty Formula failed to converge!"
      Exit Sub
1850:     End If
1851:     lngIterations = lngIterations - 1
1852:   Loop
  
1854:   cos2SigmaM = Cos(2 * Sigma1 + Sigma)
1855:   sinSigma = Sin(Sigma)
1856:   cosSigma = Cos(Sigma)
  Dim dblLat2Denom As Double
  Dim dblLat2Temp As Double
1859:   dblLat2Temp = dblSinU1 * sinSigma - dblCosU1 * cosSigma * cosAlpha1
1860:   dblLat2Denom = (1 - f) * (Sqr(sinAlpha ^ 2 + dblLat2Temp ^ 2))
  
  ' CALCULATE LATITUDE FOR NEW POINT
  Dim dblLat2 As Double
1864:   dblLat2 = atan2(dblSinU1 * cosSigma + dblCosU1 * sinSigma * cosAlpha1, dblLat2Denom)                ' [8]
  
  ' CALCULATE LONGITUDE FOR NEW POINT
  Dim dblLambda As Double
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
1870:   dblLambda = atan2(sinSigma * sinAlpha1, dblCosU1 * cosSigma - dblSinU1 * sinSigma * cosAlpha1)      ' [9]
1871:   C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                                      ' [10]
1872:   dblLambda1 = cos2SigmaM + C * cosSigma * (-1# + 2# * cos2SigmaM ^ 2#)
1873:   dblLambda1a = C * sinSigma * dblLambda1
  Dim dblLambda2 As Double
1875:   dblLambda2 = Sigma + dblLambda1a
1876:   l = dblLambda - ((1 - C) * f * sinAlpha * dblLambda2)                                               ' [11]
  
1878:   pNewPoint.X = dblPX + RadToDeg(l)
1879:   pNewPoint.Y = RadToDeg(dblLat2)
  
1881:   dblAz2 = RadToDeg(atan2(sinAlpha, -dblLat2Temp))
1882:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2


  Exit Sub
ErrorHandler:
  HandleError True, "PointLineVincentyPerPoint " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function DistanceVincentyNumbers(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
        dblAz1 As Double, dblAz2 As Double) As Double
  On Error GoTo ErrorHandler

  
  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  
1903:   If dblPX = dblQX And dblPY = dblQY Then      ' SAME POINT
1904:     DistanceVincentyNumbers = 0
1905:     dblAz1 = 0
1906:     dblAz2 = 0
    Exit Function
1908:   End If
  
  
  Dim A As Double
  Dim B As Double
1913:   A = 6378137               ' WGS84 SPHEROID; EQUATORIAL RADIUS
1914:   B = 6356752.31424518      ' WGS84 SPHEROID; POLAR RADIUS
  
  Dim f As Double
1917:   f = 1 / 298.257223563     ' WGS84 SPHEROID; FLATTENING
  
  Dim l As Double           ' DIFFERENCE IN LONGITUDE
1920:   l = DegToRad(dblQX - dblPX)
  
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
1925:   U1 = Atn((1 - f) * (Tan(DegToRad(dblPY))))
1926:   U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
  Dim dblSinU1 As Double
  Dim dblSinU2 As Double
  Dim dblCosU1 As Double
  Dim dblCosU2 As Double
  
1933:   dblSinU1 = Sin(U1)
1934:   dblCosU1 = Cos(U1)
1935:   dblSinU2 = Sin(U2)
1936:   dblCosU2 = Cos(U2)
  
  Dim dblLambda As Double, dblLambdaComp As Double
1939:   dblLambda = l                     ' FIRST ESTIMATE OF LAMBDA
1940:   dblLambdaComp = 2 * dblPi
  Dim lngIterations As Long
1942:   lngIterations = 40
  
  Dim sinLambda As Double
  Dim cosLambda As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim Sigma As Double
  Dim sinAlpha As Double
  Dim cosSqAlpha As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
  
1957:   Do While (Abs(dblLambda - dblLambdaComp) > 0.000000000001) And (lngIterations > 0)       ' VINCENTY EQUATION NUMBERS
1958:     sinLambda = Sin(dblLambda)                                                          '  |
1959:     cosLambda = Cos(dblLambda)                                                          '  |
1960:     sinSigma = Sqr((dblCosU2 * sinLambda) ^ 2 + ((dblCosU1 * dblSinU2) - _
            (dblSinU1 * dblCosU2 * cosLambda)) ^ 2)                                     ' [14]
1962:     cosSigma = (dblSinU1 * dblSinU2) + (dblCosU1 * dblCosU2 * cosLambda)                ' [15]
1963:     Sigma = atan2(sinSigma, cosSigma)                                                   ' [16]
1964:     sinAlpha = (dblCosU1 * dblCosU2 * sinLambda) / sinSigma                             ' [17]
1965:     cosSqAlpha = 1 - (sinAlpha ^ 2)                                                     ' TRIG IDENTITY
1966:     If cosSqAlpha = 0 Then
1967:       cos2SigmaM = 0                                                                    ' ADAPTED FROM VENESS
1968:     Else
1969:       cos2SigmaM = cosSigma - ((2 * dblSinU1 * dblSinU2) / cosSqAlpha)                  ' [18]
1970:     End If
    
1972:     C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                      ' [10]
1973:     dblLambdaComp = dblLambda
1974:     dblLambda1 = cos2SigmaM + C * cosSigma * (-1 + (2 * cos2SigmaM * cos2SigmaM))
1975:     dblLambda1a = C * sinSigma * dblLambda1
    ' VINCENTY WRITES EQUATION AS "L = dblLambda - ((1 - C)...
1977:     dblLambda = l + ((1 - C) * f * sinAlpha * (Sigma + dblLambda1a))                    ' [11]
    
1979:     If lngIterations = 0 Then
1980:       MsgBox "Vincenty Formula failed to converge!"
1981:       DistanceVincentyNumbers = -9999
      Exit Function
1983:     End If
1984:     lngIterations = lngIterations - 1
1985:   Loop
  
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim DeltaSigma As Double
  Dim S As Double
  
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  
1997:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
  
  Dim dblA1 As Double
  Dim dblB1 As Double
  
2002:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
2003:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                           ' [3]
2004:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
2005:   dblB = (uSq / 1024) * (256 + dblB1)              ' [4]
  
2007:   DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
2008:   DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
2009:   DeltaSigma3 = cos2SigmaM + DeltaSigma2
2010:   DeltaSigma = dblB * sinSigma * DeltaSigma3                                                                 ' [6]
2011:   S = B * dblA * (Sigma - DeltaSigma)
  
'  var uSq = cosSqAlpha * (a*a - b*b) / (b*b);
'  var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
'  var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
'  var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-
'    B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
'  var s = b*A*(sigma-deltaSigma);
'
'  s = s.toFixed(3); // round to 1mm precision
'  return s;
'}
  
  
2025:   dblAz1 = RadToDeg(atan2(dblCosU2 * sinLambda, (dblCosU1 * dblSinU2) - (dblSinU1 * dblCosU2 * cosLambda)))
2026:   dblAz2 = RadToDeg(atan2(dblCosU1 * sinLambda, -(dblSinU1 * dblCosU2) + (dblCosU1 * dblSinU2 * cosLambda)))
  
2028:   If dblAz1 < 0 Then dblAz1 = 360 + dblAz1
2029:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2
2030:   DistanceVincentyNumbers = S


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyNumbers " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function DistanceVincentyNumbers2(dblPX As Double, dblPY As Double, dblQX As Double, dblQY As Double, _
        dblAz1 As Double, dblAz2 As Double, _
        Optional dblEquatorialRadius As Double = 6378137, Optional dblPolarRadius As Double = 6356752.31424518) As Double
  On Error GoTo ErrorHandler

  
  ' MODIFICATION OF DistanceVincentyNumbers TO ALLOW FOR ANY ELLIPSOID
  
  ' ADAPTED FROM Vincenty, T. (1975). Direct and inverse solutions of geodesics on the
  '                                    ellipsoid with application of nested equations. Surv. Rev., XXII(176),
  '                                    8893.
  ' ADAPTED FROM CHRIS VENESS; http://www.movable-type.co.uk/scripts/latlong-vincenty-direct.html
  
  ' POINT 1 = dblPX, dblPY
  ' POINT 2 = dblQX, dblQY
  
2054:   If dblPX = dblQX And dblPY = dblQY Then      ' SAME POINT
2055:     DistanceVincentyNumbers2 = 0
2056:     dblAz1 = 0
2057:     dblAz2 = 0
    Exit Function
2059:   End If
  
  
  Dim A As Double
  Dim B As Double
2064:   A = dblEquatorialRadius   ' SPHEROID; EQUATORIAL RADIUS
2065:   B = dblPolarRadius        ' SPHEROID; POLAR RADIUS
  
  Dim f As Double
2068:   f = (A - B) / B           ' FLATTENING
  
  Dim l As Double           ' DIFFERENCE IN LONGITUDE
2071:   l = DegToRad(dblQX - dblPX)
  
  Dim U1 As Double          ' REDUCED LATITUDE FOR POINT 1;  dblPY
  Dim U2 As Double          ' REDUCED LATITUDE FOR POINT 2;  dblQY
  
2076:   U1 = Atn((1 - f) * (Tan(DegToRad(dblPY))))
2077:   U2 = Atn((1 - f) * (Tan(DegToRad(dblQY))))
  
  Dim dblSinU1 As Double
  Dim dblSinU2 As Double
  Dim dblCosU1 As Double
  Dim dblCosU2 As Double
  
2084:   dblSinU1 = Sin(U1)
2085:   dblCosU1 = Cos(U1)
2086:   dblSinU2 = Sin(U2)
2087:   dblCosU2 = Cos(U2)
  
  Dim dblLambda As Double, dblLambdaComp As Double
2090:   dblLambda = l                     ' FIRST ESTIMATE OF LAMBDA
2091:   dblLambdaComp = 2 * dblPi
  Dim lngIterations As Long
2093:   lngIterations = 40
  
  Dim sinLambda As Double
  Dim cosLambda As Double
  Dim sinSigma As Double
  Dim cosSigma As Double
  Dim Sigma As Double
  Dim sinAlpha As Double
  Dim cosSqAlpha As Double
  Dim cos2SigmaM As Double
  Dim C As Double
  
  Dim dblLambda1 As Double
  Dim dblLambda1a As Double
  
2108:   Do While (Abs(dblLambda - dblLambdaComp) > 0.000000000001) And (lngIterations > 0)       ' VINCENTY EQUATION NUMBERS
2109:     sinLambda = Sin(dblLambda)                                                          '  |
2110:     cosLambda = Cos(dblLambda)                                                          '  |
2111:     sinSigma = Sqr((dblCosU2 * sinLambda) ^ 2 + ((dblCosU1 * dblSinU2) - _
            (dblSinU1 * dblCosU2 * cosLambda)) ^ 2)                                     ' [14]
2113:     cosSigma = (dblSinU1 * dblSinU2) + (dblCosU1 * dblCosU2 * cosLambda)                ' [15]
2114:     Sigma = atan2(sinSigma, cosSigma)                                                   ' [16]
2115:     sinAlpha = (dblCosU1 * dblCosU2 * sinLambda) / sinSigma                             ' [17]
2116:     cosSqAlpha = 1 - (sinAlpha ^ 2)                                                     ' TRIG IDENTITY
2117:     If cosSqAlpha = 0 Then
2118:       cos2SigmaM = 0                                                                    ' ADAPTED FROM VENESS
2119:     Else
2120:       cos2SigmaM = cosSigma - ((2 * dblSinU1 * dblSinU2) / cosSqAlpha)                  ' [18]
2121:     End If
    
2123:     C = (f / 16) * cosSqAlpha * (4 + (f * (4 - (3 * cosSqAlpha))))                      ' [10]
2124:     dblLambdaComp = dblLambda
2125:     dblLambda1 = cos2SigmaM + C * cosSigma * (-1 + (2 * cos2SigmaM * cos2SigmaM))
2126:     dblLambda1a = C * sinSigma * dblLambda1
    ' VINCENTY WRITES EQUATION AS "L = dblLambda - ((1 - C)...
2128:     dblLambda = l + ((1 - C) * f * sinAlpha * (Sigma + dblLambda1a))                    ' [11]
    
2130:     If lngIterations = 0 Then
2131:       MsgBox "Vincenty Formula failed to converge!"
2132:       DistanceVincentyNumbers2 = -9999
      Exit Function
2134:     End If
2135:     lngIterations = lngIterations - 1
2136:   Loop
  
  Dim uSq As Double
  Dim dblA As Double
  Dim dblB As Double
  Dim DeltaSigma As Double
  Dim S As Double
  
  Dim DeltaSigma1 As Double
  Dim DeltaSigma2 As Double
  Dim DeltaSigma3 As Double
  
2148:   uSq = (cosSqAlpha * (A ^ 2 - B ^ 2)) / (B ^ 2)
  
  Dim dblA1 As Double
  Dim dblB1 As Double
  
2153:   dblA1 = (uSq * (-768 + (uSq * (320 - (175 * uSq)))))
2154:   dblA = 1 + ((uSq / 16384) * (4096 + dblA1))                                           ' [3]
2155:   dblB1 = (uSq * (-128 + (uSq * (74 - (uSq * 47)))))
2156:   dblB = (uSq / 1024) * (256 + dblB1)              ' [4]
  
2158:   DeltaSigma1 = ((dblB / 6) * cos2SigmaM * (-3 + 4 * sinSigma ^ 2) * (-3 + 4 * cos2SigmaM ^ 2))
2159:   DeltaSigma2 = ((dblB / 4) * (cosSigma * (-1 + 2 * cos2SigmaM ^ 2) - DeltaSigma1))
2160:   DeltaSigma3 = cos2SigmaM + DeltaSigma2
2161:   DeltaSigma = dblB * sinSigma * DeltaSigma3                                                                 ' [6]
2162:   S = B * dblA * (Sigma - DeltaSigma)
  
'  var uSq = cosSqAlpha * (a*a - b*b) / (b*b);
'  var A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
'  var B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
'  var deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-
'    B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
'  var s = b*A*(sigma-deltaSigma);
'
'  s = s.toFixed(3); // round to 1mm precision
'  return s;
'}
  
  
2176:   dblAz1 = RadToDeg(atan2(dblCosU2 * sinLambda, (dblCosU1 * dblSinU2) - (dblSinU1 * dblCosU2 * cosLambda)))
2177:   dblAz2 = RadToDeg(atan2(dblCosU1 * sinLambda, -(dblSinU1 * dblCosU2) + (dblCosU1 * dblSinU2 * cosLambda)))
  
2179:   If dblAz1 < 0 Then dblAz1 = 360 + dblAz1
2180:   If dblAz2 < 0 Then dblAz2 = 360 + dblAz2
2181:   DistanceVincentyNumbers2 = S


  Exit Function
ErrorHandler:
  HandleError True, "DistanceVincentyNumbers2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function RadToDeg(dblRad As Double) As Double
  On Error GoTo ErrorHandler


2193:   RadToDeg = dblRad * 180 / dblPi


  Exit Function
ErrorHandler:
  HandleError True, "RadToDeg " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function DegToRad(dblDeg As Double) As Double
  On Error GoTo ErrorHandler


2204:   DegToRad = dblDeg * dblPi / 180


  Exit Function
ErrorHandler:
  HandleError True, "DegToRad " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function atan2(Y As Double, X As Double) As Double
  On Error GoTo ErrorHandler


2216:   If X > 0 Then
2217:     atan2 = Atn(Y / X)
2218:   ElseIf X < 0 Then
2219:     If Y = 0 Then
2220:       atan2 = (dblPi - Atn(Abs(Y / X)))
2221:     Else
2222:       atan2 = Sgn(Y) * (dblPi - Atn(Abs(Y / X)))
2223:     End If
2224:   Else    ' IF X = 0
2225:     If Y = 0 Then
2226:       atan2 = 0
2227:     Else
2228:       atan2 = Sgn(Y) * dblPi / 2
2229:     End If
2230:   End If


  Exit Function
ErrorHandler:
  HandleError True, "atan2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaLegs(dblLeg1 As Double, dblLeg2 As Double, dblLeg3 As Double) As Double
  On Error GoTo ErrorHandler


  Dim dblS As Double
2242:   dblS = (dblLeg1 + dblLeg2 + dblLeg3) / 2
2243:   TriangleAreaLegs = Sqr(dblS * (dblS - dblLeg1) * (dblS - dblLeg2) * (dblS - dblLeg3))


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaLegs " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaPoints(pPoint1 As IPoint, pPoint2 As IPoint, pPoint3 As IPoint) As Double
  On Error GoTo ErrorHandler


2254:   TriangleAreaPoints = Abs(((((pPoint2.X - pPoint1.X) * (pPoint3.Y - pPoint1.Y)) - ((pPoint3.X - pPoint1.X) * (pPoint2.Y - pPoint1.Y))) / 2))


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function TriangleAreaPoints3D(pPoint1 As IPoint, pPoint2 As IPoint, pPoint3 As IPoint) As Double
  On Error GoTo ErrorHandler

  
2265:   TriangleAreaPoints3D = TriangleAreaPoints3DValues(pPoint1.X, pPoint1.Y, pPoint1.Z, pPoint2.X, pPoint2.Y, pPoint2.Z, _
                                                  pPoint3.X, pPoint3.Y, pPoint3.Z)


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints3D " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function



Public Function TriangleAreaPoints3DValues(dblPX As Double, dblPY As Double, dblPZ As Double, _
                                           dblQX As Double, dblQY As Double, dblQZ As Double, _
                                           dblRX As Double, dblRY As Double, dblRZ As Double) As Double
  On Error GoTo ErrorHandler


  Dim dblI As Double
  Dim dblJ As Double
  Dim dblK As Double
  
2286:   dblI = (((dblQY - dblPY) * (dblRZ - dblPZ)) - ((dblRY - dblPY) * (dblQZ - dblPZ))) ^ 2
2287:   dblJ = (((dblQX - dblPX) * (dblRZ - dblPZ)) - ((dblRX - dblPX) * (dblQZ - dblPZ))) ^ 2
2288:   dblK = (((dblQX - dblPX) * (dblRY - dblPY)) - ((dblRX - dblPX) * (dblQY - dblPY))) ^ 2

2290:   TriangleAreaPoints3DValues = (Sqr(dblI + dblJ + dblK)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "TriangleAreaPoints3DValues " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
                                           

Public Function EnvelopeToPolygon(pEnv As IEnvelope) As IPolygon
  On Error GoTo ErrorHandler


  Dim pPtColl As IPointCollection

2305:   Set pPtColl = New Polygon
2306:   With pPtColl
2307:       .AddPoint pEnv.LowerLeft
2308:       .AddPoint pEnv.UpperLeft
2309:       .AddPoint pEnv.UpperRight
2310:       .AddPoint pEnv.LowerRight
      'Close the polygon
2312:       .AddPoint pEnv.LowerLeft
2313:   End With
  
  Dim pPolygon As IPolygon
2316:   Set pPolygon = pPtColl
2317:   Set pPolygon.SpatialReference = pEnv.SpatialReference
  Dim pTopoOp As ITopologicalOperator
2319:   Set pTopoOp = pPolygon
2320:   pTopoOp.Simplify
    
2322:   Set EnvelopeToPolygon = pPtColl


  Exit Function
ErrorHandler:
  HandleError True, "EnvelopeToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function EllipticArcToPolygon2(SegCollection As ISegmentCollection, NumVertices As Long) As IMultipoint
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
      
On Error GoTo ERH

  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
2344:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
2346:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
2350:   For anIndex = 0 To lngSegCount
2351:     theTestLength = SegCollection.Segment(anIndex).length
2352:     theLength = theLength + theTestLength
2353:     lngLengths(anIndex) = theTestLength
2354:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
2360:   For anIndex = 0 To lngSegCount
2361:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
2362:     If lngNumVertices < 8 Then lngNumVertices = 8
2363:     lngVertices(anIndex) = lngNumVertices
2364:   Next anIndex
  
  Dim pMpt As IPointCollection
2367:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
2369:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
2375:   For anIndex = 0 To lngSegCount
2376:     lngNumVertices = lngVertices(anIndex)
2377:     pRatio = 1 / lngNumVertices
2378:     Set pCurve = SegCollection.Segment(anIndex)
    
2380:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
2382:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
2383:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
2387:       pMpt.AddPoint pClone.Clone
2388:     Next anIndex2
2389:   Next anIndex
  
2391:   Set EllipticArcToPolygon2 = pMpt
    Exit Function
  
ERH:
2395:     MsgBox "Failed in EllipticArcToPolygon2: " & err.Description
End Function

Public Function CurveToPolygon(pOrigGeometry As IGeometry, NumVertices As Long) As IPolygon
On Error GoTo ERH
  
  Dim pGeometryCollection As IGeometryCollection
2402:   Set pGeometryCollection = pOrigGeometry
  Dim pSpRef As ISpatialReference
2404:   Set pSpRef = pOrigGeometry.SpatialReference
  
  Dim pOrigPolygon As IPolycurve
2407:   Set pOrigPolygon = pOrigGeometry
  
  Dim dblFullLength As Double
2410:   dblFullLength = pOrigPolygon.length
  
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  Dim pPolygon As IPointCollection
  Dim pRing As IRing
  Dim pSegment As ISegment
  Dim pStartPoint As IPoint
2418:   Set pStartPoint = New Point
  Dim pEndPoint As IPoint
2420:   Set pEndPoint = New Point
  Dim pClone As IClone
  Dim booFoundCurve As Boolean
  
  Dim lngRingCount As Long
  Dim lngNumVertices As Long
  Dim pRatio As Double
  Dim anIndex As Long
  Dim anIndex2 As Long
  Dim anIndex3 As Long
  Dim lngSegCount As Long
  
  Dim SegCollection As ISegmentCollection
  Dim pNewSegCollection As ISegmentCollection
  
  Dim pPathSegColl As ISegmentCollection
  Dim pNewSegment As ISegment
  Dim pNewLine As esriGeometry.ILine
  
  Dim pNewPolyGeoColl As IGeometryCollection
2440:   Set pNewPolyGeoColl = New Polygon
  Dim pNewRingGeometry As IGeometry
  Dim pPath As IPath
  Dim pSegmentCollection As ISegmentCollection
  Dim pNewSegCol As ISegmentCollection
  
2446:   lngRingCount = pGeometryCollection.GeometryCount - 1
2447:   For anIndex = 0 To lngRingCount
2448:     If TypeOf pOrigGeometry Is IPolyline Then
2449:       Set pPath = pGeometryCollection.Geometry(anIndex)
2450:       Set pSegmentCollection = pPath
2451:       Set pNewSegCol = New Ring
2452:       pNewSegCol.AddSegmentCollection pSegmentCollection
2453:       Set pRing = pNewSegCol
2454:       pRing.Close
2455:     Else
2456:       Set pRing = pGeometryCollection.Geometry(anIndex)
2457:     End If
2458:     Set SegCollection = pRing
2459:     Set pNewSegCollection = New Ring
2460:     lngSegCount = SegCollection.SegmentCount - 1
2461:     For anIndex2 = 0 To lngSegCount
2462:       Set pSegment = SegCollection.Segment(anIndex2)
2463:       Set pGeometry = pSegment
2464:       If pGeometry.GeometryType <> esriGeometryLine Then ' IF SEGMENT IS CURVE
2465:         booFoundCurve = True
2466:         lngNumVertices = Int((pSegment.length / dblFullLength) * NumVertices)
2467:         If lngNumVertices < 8 Then lngNumVertices = 8
2468:         pRatio = 1 / lngNumVertices
       
2470:         Set pCurve = pSegment
2471:         Set pPathSegColl = New Path
2472:         Set pNewSegment = New esriGeometry.Line
2473:         Set pStartPoint = pCurve.FromPoint
2474:         For anIndex3 = 1 To lngNumVertices
2475:           pCurve.QueryPoint 0, (pRatio * anIndex3), True, pEndPoint
2476:           pNewSegment.FromPoint = pStartPoint
2477:           pNewSegment.ToPoint = pEndPoint
          
2479:           Set pClone = pNewSegment
2480:           pPathSegColl.AddSegment pClone.Clone
          
'          If anIndex3 < 4 Then
'            MsgBox "Start Point:  X = " & CStr(pStartPoint.X) & ", Y = " & CStr(pStartPoint.Y) & vbCrLf & _
'              "End Point:  X = " & CStr(pEndPoint.X) & ", Y = " & CStr(pEndPoint.Y) & vbCrLf & _
'              "Segment Length = " & CStr(pNewSegment.length) & vbCrLf & _
'              "Segment Collection Count = " & CStr(pPathSegColl.SegmentCount)
'          End If
          
2489:           Set pClone = pEndPoint
2490:           Set pStartPoint = pClone.Clone
          
2492:         Next anIndex3
2493:         pNewSegCollection.AddSegmentCollection pPathSegColl

2495:       Else      ' IF SEGMENT IS ACTUALLY LINE, DON'T ADD MIDPOINTS
2496:         Set pClone = pSegment
2497:         pNewSegCollection.AddSegment pClone.Clone
2498:       End If
2499:     Next anIndex2
2500:     Set pNewRingGeometry = pNewSegCollection
2501:     pNewPolyGeoColl.AddGeometry pNewRingGeometry

2503:   Next anIndex
  
  Dim pNewPolygon As IPolygon
  
2507:   If booFoundCurve Or (TypeOf pOrigGeometry Is IPolyline) Then
  
2509:     Set pNewPolygon = pNewPolyGeoColl
    Dim pTopoOp As ITopologicalOperator
2511:     Set pTopoOp = pNewPolygon
2512:     pTopoOp.Simplify
2513:     Set pNewPolygon.SpatialReference = pSpRef

2515:   Else
2516:     Set pNewPolygon = pOrigGeometry
2517:     Set pNewPolygon.SpatialReference = pSpRef
2518:   End If
    
2520:   Set CurveToPolygon = pNewPolygon
  Exit Function
  
ERH:
2524:     MsgBox "Failed in CurveToPolygon: " & vbCrLf & "Error = " & err.Description & vbCrLf & "Line Number = " & CStr(Erl)
End Function
Public Function CurveToPolyline(pOrigGeometry As IGeometry, NumVertices As Long) As IPolyline
On Error GoTo ERH

  Dim pGeometryCollection As IGeometryCollection
2530:   Set pGeometryCollection = pOrigGeometry
  Dim pSpRef As ISpatialReference
2532:   Set pSpRef = pOrigGeometry.SpatialReference
  
  Dim pOrigPolyline As IPolycurve
  
2536:   Set pOrigPolyline = pOrigGeometry
  
  Dim dblFullLength As Double
2539:   dblFullLength = pOrigPolyline.length

  Dim pPath As IPath
  
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  Dim pSegment As ISegment
  Dim pStartPoint As IPoint
2547:   Set pStartPoint = New Point
  Dim pEndPoint As IPoint
2549:   Set pEndPoint = New Point
  Dim pClone As IClone
  Dim booFoundCurve As Boolean
  
  Dim lngPathCount As Long
  Dim lngNumVertices As Long
  Dim pRatio As Double
  Dim anIndex As Long
  Dim anIndex2 As Long
  Dim anIndex3 As Long
  Dim lngSegCount As Long
  
  Dim SegCollection As ISegmentCollection
  Dim pNewSegCollection As ISegmentCollection
  
  Dim pPathSegColl As ISegmentCollection
  Dim pNewSegment As ISegment
  Dim pNewLine As esriGeometry.ILine
  
  Dim pNewPolyGeoColl As IGeometryCollection
2569:   Set pNewPolyGeoColl = New Polyline
  Dim pNewPathGeometry As IGeometry
  
  Dim pRing As IRing
  
2574:   lngPathCount = pGeometryCollection.GeometryCount - 1
2575:   For anIndex = 0 To lngPathCount
2576:     If TypeOf pOrigGeometry Is IPolygon Then
2577:       Set pRing = pGeometryCollection.Geometry(anIndex)
2578:       Set pPath = pRing
2579:     Else
2580:       Set pPath = pGeometryCollection.Geometry(anIndex)
2581:     End If
2582:     Set SegCollection = pPath
2583:     Set pNewSegCollection = New Path
2584:     lngSegCount = SegCollection.SegmentCount - 1
2585:     For anIndex2 = 0 To lngSegCount
2586:       Set pSegment = SegCollection.Segment(anIndex2)
2587:       Set pGeometry = pSegment
2588:       If pGeometry.GeometryType <> esriGeometryLine Then ' IF SEGMENT IS CURVE
2589:         booFoundCurve = True
2590:         lngNumVertices = Int((pSegment.length / dblFullLength) * NumVertices)
2591:         If lngNumVertices < 8 Then lngNumVertices = 8
2592:         pRatio = 1 / lngNumVertices
       
2594:         Set pCurve = pSegment
2595:         Set pPathSegColl = New Path
2596:         Set pNewSegment = New esriGeometry.Line
2597:         Set pStartPoint = pCurve.FromPoint
2598:         For anIndex3 = 1 To lngNumVertices
2599:           pCurve.QueryPoint 0, (pRatio * anIndex3), True, pEndPoint
2600:           pNewSegment.FromPoint = pStartPoint
2601:           pNewSegment.ToPoint = pEndPoint
          
2603:           Set pClone = pNewSegment
2604:           pPathSegColl.AddSegment pClone.Clone
2605:           Set pClone = pEndPoint
2606:           Set pStartPoint = pClone.Clone
2607:         Next anIndex3
2608:         pNewSegCollection.AddSegmentCollection pPathSegColl

2610:       Else      ' IF SEGMENT IS ACTUALLY LINE, DON'T ADD MIDPOINTS
2611:         Set pClone = pSegment
2612:         pNewSegCollection.AddSegment pClone.Clone
2613:       End If
2614:     Next anIndex2
2615:     Set pNewPathGeometry = pNewSegCollection
2616:     pNewPolyGeoColl.AddGeometry pNewPathGeometry

2618:   Next anIndex

  
  Dim pNewPolyline As IPolyline
  
2623:   If booFoundCurve Or (TypeOf pOrigGeometry Is IPolygon) Then
  
2625:     Set pNewPolyline = pNewPolyGeoColl
    Dim pTopoOp As ITopologicalOperator
2627:     Set pTopoOp = pNewPolyline
2628:     pTopoOp.Simplify
2629:     Set pNewPolyline.SpatialReference = pSpRef

2631:   Else
2632:     Set pNewPolyline = pOrigGeometry
2633:     Set pNewPolyline.SpatialReference = pSpRef
2634:   End If
    
2636:   Set CurveToPolyline = pNewPolyline
  Exit Function
  
ERH:
2640:     MsgBox "Failed in CurveToPolyline: " & vbCrLf & "Error = " & err.Description & vbCrLf & "Line Number = " & CStr(Erl)
End Function
Public Sub Graphic_MakeFromGeometry(ByRef pMxDoc As IMxDocument, ByRef pGeometry As IGeometry, Optional strName As String)
  On Error GoTo ErrorHandler

  
  Dim pMxDocument As esriArcMapUI.IMxDocument
  Dim pActiveView As esriCarto.IActiveView
  
  Dim pGContainer As IGraphicsContainer
2650:   Set pGContainer = pMxDoc.FocusMap
  
  Dim pElement As IElement
  Dim pPolygonElement As IPolygonElement
  Dim pSpatialReference As ISpatialReference
  Dim pGraphicElement As IGraphicElement
  Dim pElementProperties As IElementProperties
  
  Dim pClone As IClone
2659:   Set pClone = pGeometry
  Dim pNewGeometry As IGeometry
2661:   Set pNewGeometry = pClone.Clone
  
  Dim pGeometryType As esriGeometryType
2664:   pGeometryType = pNewGeometry.GeometryType
  
  'ADD GEOMETRY, NAME AND SPATIAL REFERENCE TO GRAPHIC ELEMENT
  Select Case pGeometryType
    Case 0:
2669:       MsgBox "Null Geometry!  No graphic added..."
    Case 1:
2671:       Set pElement = New MarkerElement
    Case 3, 6, 13, 14, 15, 16:
2673:       Set pElement = New LineElement
    Case 4, 11:
2675:       Set pElement = New PolygonElement
    Case 5:
2677:       Set pElement = New RectangleElement
    Case Else:
2679:       MsgBox "Unexpected Shape Type:  Unable to add graphic..."
2680:   End Select
    
2682:   pElement.Geometry = pNewGeometry
2683:   Set pGraphicElement = pElement
2684:   Set pSpatialReference = pGeometry.SpatialReference
2685:   Set pGraphicElement.SpatialReference = pSpatialReference
2686:   Set pElementProperties = pElement
2687:   pElementProperties.Name = strName

  ' ADD GRAPHIC TO GRAPHICS CONTAINER
2690:   pGContainer.AddElement pElement, 0

  'Draw
2693:   pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing


  Exit Sub
ErrorHandler:
  HandleError True, "Graphic_MakeFromGeometry " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function Graphic_ReturnElementFromGeometry(ByRef pMxDoc As IMxDocument, ByRef pGeometry As IGeometry, _
    Optional strName As String, Optional AddToView As Boolean) As IElement
  On Error GoTo ErrorHandler

  
  Dim pMxDocument As esriArcMapUI.IMxDocument
  Dim pActiveView As esriCarto.IActiveView
  
  Dim pGContainer As IGraphicsContainer
2709:   Set pGContainer = pMxDoc.FocusMap
  
  Dim pElement As IElement
  Dim pPolygonElement As IPolygonElement
  Dim pSpatialReference As ISpatialReference
  Dim pGraphicElement As IGraphicElement
  Dim pElementProperties As IElementProperties
  
  Dim pClone As IClone
2718:   Set pClone = pGeometry
  Dim pNewGeometry As IGeometry
2720:   Set pNewGeometry = pClone.Clone
  
  Dim pGeometryType As esriGeometryType
2723:   pGeometryType = pNewGeometry.GeometryType
  
  'ADD GEOMETRY, NAME AND SPATIAL REFERENCE TO GRAPHIC ELEMENT
  Select Case pGeometryType
    Case 0:
2728:       MsgBox "Null Geometry!  No graphic added..."
    Case 1:
2730:       Set pElement = New MarkerElement
    Case 3, 6, 13, 14, 15, 16:
2732:       Set pElement = New LineElement
    Case 4, 11:
2734:       Set pElement = New PolygonElement
    Case 5:
2736:       Set pElement = New RectangleElement
    Case Else:
2738:       MsgBox "Unexpected Shape Type:  Unable to add graphic..."
      Exit Function
2740:   End Select
  
2742:   pElement.Geometry = pNewGeometry
2743:   Set pGraphicElement = pElement
2744:   Set pSpatialReference = pGeometry.SpatialReference
2745:   Set pGraphicElement.SpatialReference = pSpatialReference
2746:   Set pElementProperties = pElement
2747:   pElementProperties.Name = strName
  
2749:   If AddToView Then
    ' ADD GRAPHIC TO GRAPHICS CONTAINER
2751:     pGContainer.AddElement pElement, 0
    'Draw
2753:     pMxDoc.ActiveView.PartialRefresh esriViewGraphics, Nothing, Nothing
2754:   End If
  
2756:   Set Graphic_ReturnElementFromGeometry = pElement


  Exit Function
ErrorHandler:
  HandleError True, "Graphic_ReturnElementFromGeometry " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub ShowVertices(pMxDoc As IMxDocument, pGeometry As IGeometry, Optional strName As String, _
        Optional DeleteCurrentGraphicsWithName As Boolean)
  On Error GoTo ErrorHandler


  Dim pPoint1 As IPoint
  Dim pPoint2 As IPoint
  Dim pPoly As IPolygon
  Dim pLine As IPolyline
  Dim pOutVertex As IPoint, lOutPart As Long, lOutVertex As Long
  
2775:   If DeleteCurrentGraphicsWithName And (strName <> "") Then
2776:     Call DeleteGraphicsByName(pMxDoc, "DeleteMe")
2777:   End If
  
  Dim pArray As IArray
2780:   Set pArray = New esriSystem.Array
  
  Dim pPointCollection As IPointCollection
2783:   Set pPointCollection = pGeometry
  
  Dim pPointEnum As IEnumVertex
2786:   Set pPointEnum = pPointCollection.EnumVertices
  
2788:   pPointEnum.Reset
  
  Dim pVertex As IPoint
2791:   Set pVertex = New Point
  'Query the next vertex - have to cocreate the point
  'QueryNext is faster than Next, because the method doesn't have
  'to create the point each time
2795:   pPointEnum.QueryNext pVertex, lOutPart, lOutVertex
  
2797:   Do While Not pVertex.IsEmpty
2798:     Graphic_MakeFromGeometry pMxDoc, pVertex, strName
2799:     pPointEnum.QueryNext pVertex, lOutPart, lOutVertex
'    Debug.Print lOutPart & ",    " & lOutVertex & ",  " & pVertex.IsEmpty
2801:   Loop


  Exit Sub
ErrorHandler:
  HandleError True, "ShowVertices " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function CalcBearing(ByRef Point1 As IPoint, ByRef Point2 As IPoint) As Double
  On Error GoTo ErrorHandler


  Dim dblBearing As Double

  Dim xDist As Double
  Dim yDist As Double
  Dim xyTanDeg As Double
  
2818:   xDist = (Point1.X - Point2.X)
2819:   yDist = (Point1.Y - Point2.Y)
2820:   If yDist = 0 Then
2821:     If xDist < 0 Then
2822:       xyTanDeg = -90
2823:     ElseIf xDist = 0 Then
2824:       xyTanDeg = 0
2825:     Else
2826:       xyTanDeg = 90
2827:     End If
2828:   Else
2829:     xyTanDeg = AsDegrees(Atn(xDist / yDist))
2830:   End If

2832:   If (yDist >= 0) Then
2833:     dblBearing = 180 + xyTanDeg
2834:   Else
2835:     If (xDist <= 0) Then
2836:       dblBearing = xyTanDeg
2837:     Else
2838:       dblBearing = 360 + xyTanDeg
2839:     End If
2840:   End If ' END CALCULATING BEARING
  
2842:   dblBearing = Abs(dblBearing)
2843:   CalcBearing = dblBearing


  Exit Function
ErrorHandler:
  HandleError True, "CalcBearing " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcBearing2(ByRef Point1 As IPoint, ByRef Point2 As IPoint) As Double
  On Error GoTo ErrorHandler


  Dim dblBearing As Double

  Dim xDist As Double
  Dim yDist As Double
  Dim xyTanDeg As Double
  
2861:   xDist = (Point1.X - Point2.X)
2862:   yDist = (Point1.Y - Point2.Y)
  
2864:   If xDist = 0 And yDist = 0 Then
2865:     CalcBearing2 = -9999
2866:   Else
2867:     If yDist = 0 Then
2868:       If xDist < 0 Then
2869:         xyTanDeg = -90
2870:       ElseIf xDist = 0 Then
2871:         xyTanDeg = 0
2872:       Else
2873:         xyTanDeg = 90
2874:       End If
2875:     Else
2876:       xyTanDeg = AsDegrees(Atn(xDist / yDist))
2877:     End If
  
2879:     If (yDist >= 0) Then
2880:       dblBearing = 180 + xyTanDeg
2881:     Else
2882:       If (xDist <= 0) Then
2883:         dblBearing = xyTanDeg
2884:       Else
2885:         dblBearing = 360 + xyTanDeg
2886:       End If
2887:     End If ' END CALCULATING BEARING
    
2889:     dblBearing = Abs(dblBearing)
2890:     CalcBearing2 = dblBearing
2891:   End If


  Exit Function
ErrorHandler:
  HandleError True, "CalcBearing2 " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcDistMatrix(pArray As esriSystem.IArray, Optional IncludeLine As Boolean, _
    Optional IncludeBearing As Boolean, Optional pApp As IApplication) As Collection
  On Error GoTo ErrorHandler


2904:   Screen.MousePointer = vbHourglass
  
  ' RETURNS A COLLECTION OF IVariantArray OBJECTS
  ' EACH IVariantArray IDENTIFIED BY STRING; CONCATENATION OF [ORIGIN INDEX] & "_" & [DESTINATION INDEX]
  ' EACH IVariantArray OBJECT CONTAINS:
  '       0) ORIGIN SHAPE INDEX VALUE
  '       1) DESTINATION SHAPE INDEX VALUE
  '       2) DISTANCE
  '       3) CONNECTOR POLYLINE:  OPTIONAL; CONTAINS BOOLEAN "FALSE" IF NOT REQUESTED
  '       4) BEARING:             OPTIONAL; CONTAINS BOOLEAN "FALSE" IF NOT REQUESTED
  
  Dim pCollection As Collection
2916:   Set pCollection = New Collection
  
  Dim pProxOp As IProximityOperator
  
  Dim lngIndex As Long
  Dim lngIndex2 As Long
  
  Dim lngArrayMaxIndex As Long
2924:   lngArrayMaxIndex = pArray.Count - 1
  
  Dim pGeometry1 As IGeometry
  Dim pGeometry2 As IGeometry
  
  Dim dblDistance As Double
  Dim dblBearing As Double
  Dim dblRevBearing As Double
  Dim strID As String
  Dim strRevID As String
  Dim pConnector As IPolyline
  Dim pRevConnector As IPolyline
  
  Dim pVarArray As IVariantArray
  Dim pRevVarArray As IVariantArray
  
2940:   If Not pApp Is Nothing Then
      ' PROGRESS BAR STUFF
    Dim psbar As IStatusBar
2943:     Set psbar = pApp.StatusBar
    Dim pPro As IStepProgressor
2945:     Set pPro = psbar.ProgressBar
    Dim lngCounter As Long
2947:     lngCounter = 0
    Dim lngTotalCount As Long
2949:     lngTotalCount = (((lngArrayMaxIndex + 1) * lngArrayMaxIndex) / 2)
    Dim strTotalCount As String
2951:     strTotalCount = CStr(lngTotalCount)
2952:     pPro.position = 1
2953:     psbar.ShowProgressBar "Building Preliminary Distance Matrix:  Step 1 of " & strTotalCount & "...", 1, _
            lngTotalCount, 1, True
2955:   End If
  
  Dim pOutputPointCollection As IPointCollection
  
2959:   For lngIndex = 0 To lngArrayMaxIndex
2960:     Set pGeometry1 = pArray.Element(lngIndex)
    
2962:     For lngIndex2 = lngIndex To lngArrayMaxIndex
      
2964:       strID = CStr(lngIndex) & "_" & CStr(lngIndex2)
2965:       strRevID = CStr(lngIndex2) & "_" & CStr(lngIndex)
      
2967:       Set pVarArray = New VarArray
2968:       Set pRevVarArray = New VarArray
      
      ' FIRST ELEMENT
2971:       pVarArray.Add lngIndex         ' FIRST VALUES IN THE ARRAY ARE ORIGIN NODES
2972:       pRevVarArray.Add lngIndex2
      
      ' SECOND ELEMENT
2975:       pVarArray.Add lngIndex2        ' SECOND VALUES IN THE ARRAY ARE "TO" NODES
2976:       pRevVarArray.Add lngIndex
      
2978:       If lngIndex = lngIndex2 Then   ' IF MEASURING DISTANCE TO ITSELF
2979:         Set pConnector = New Polyline
2980:         pConnector.SetEmpty
2981:         Set pRevConnector = New Polyline
2982:         pRevConnector.SetEmpty
        
        ' THIRD ELEMENT
2985:         pVarArray.Add 0              ' THIRD VALUE IS DISTANCE
        
        ' FOURTH ELEMENT
2988:         If IncludeLine Then
2989:           pVarArray.Add pConnector   ' FOURTH VALUE IS CONNECTION LINE
2990:         Else
2991:           pVarArray.Add False        ' FOURTH VALUE:  JUST ADDING SMALL PLACEHOLDER ELEMENT
2992:         End If
        
        ' FIFTH ELEMENT
2995:         If IncludeBearing Then
2996:           pVarArray.Add -999         ' FIFTH VALUE IS BEARING
2997:         Else
2998:           pVarArray.Add False
2999:         End If
        
        ' ADD VARARRAY TO ORIGINAL COLLECTION
3002:         pCollection.Add pVarArray, strID
      
3004:       Else
        
3006:         If Not pApp Is Nothing Then
3007:           lngCounter = lngCounter + 1
3008:           pPro.Message = "Building Preliminary Distance Matrix:  Step " & CStr(lngCounter) & " of " & strTotalCount & "..."
3009:           psbar.StepProgressBar
3010:         End If
        
3012:         Set pGeometry2 = pArray.Element(lngIndex2)
        
3014:         If IncludeLine Or IncludeBearing Then
          Dim pLineArray As IArray
3016:           Set pLineArray = CalcClosestPoints(pGeometry1, pGeometry2, 10)
          
3018:           If TypeOf pLineArray.Element(0) Is esriSystem.IStringArray Then      ' FUNCTION FAILED FOR SOME REASON
            Dim pStrArray As IStringArray
3020:             Set pStrArray = pLineArray.Element(0)
3021:             MsgBox "Failed to connect:" & vbCrLf & "Message = " & pStrArray.Element(0) & vbCrLf & _
                   "Index 1 = " & CStr(lngIndex) & " of " & CStr(lngArrayMaxIndex) & vbCrLf & _
                   "Index 2 = " & CStr(lngIndex2) & " of " & CStr(lngArrayMaxIndex)
3024:             Set pConnector = New Polyline
3025:             pConnector.SetEmpty
3026:             Set pRevConnector = New Polyline
3027:             pRevConnector.SetEmpty
            
            ' THIRD ELEMENT
3030:             pVarArray.Add 0              ' THIRD VALUE IS DISTANCE
            
            ' FOURTH ELEMENT
3033:             If IncludeLine Then
3034:               pVarArray.Add pConnector   ' FOURTH VALUE IS CONNECTION LINE
3035:             Else
3036:               pVarArray.Add False        ' FOURTH VALUE:  JUST ADDING SMALL PLACEHOLDER ELEMENT
3037:             End If
            
            ' FIFTH ELEMENT
3040:             If IncludeBearing Then
3041:               pVarArray.Add -999         ' FIFTH VALUE IS BEARING
3042:             Else
3043:               pVarArray.Add False
3044:             End If
3045:           Else
          
3047:             If IncludeLine Then
3048:               Set pConnector = pLineArray.Element(0)
3049:               Set pRevConnector = New Polyline
3050:               Set pOutputPointCollection = pRevConnector
3051:               pOutputPointCollection.AddPoint pLineArray.Element(2)
3052:               pOutputPointCollection.AddPoint pLineArray.Element(1)
              
              ' THIRD ELEMENT
3055:               pVarArray.Add pConnector.length
3056:               pRevVarArray.Add pConnector.length
              
              ' FOURTH ELEMENT
3059:               pVarArray.Add pConnector
3060:               pRevVarArray.Add pRevConnector
3061:             Else
              ' FOURTH ELEMENT
3063:               pVarArray.Add False        ' JUST ADDING SMALL PLACEHOLDER ELEMENT
3064:               pRevVarArray.Add False     ' JUST ADDING SMALL PLACEHOLDER ELEMENT
3065:             End If
3066:             If IncludeBearing Then
3067:               dblBearing = CalcBearing(pLineArray.Element(1), pLineArray.Element(2))
3068:               If dblBearing < 180 Then
3069:                 dblRevBearing = dblBearing + 180
3070:               Else
3071:                 dblRevBearing = dblBearing - 180
3072:               End If
              ' FIFTH ELEMENT
3074:               pVarArray.Add dblBearing
3075:               pRevVarArray.Add dblRevBearing
3076:             Else
              ' FIFTH ELEMENT
3078:               pVarArray.Add False
3079:               pRevVarArray.Add False
3080:             End If
3081:           End If
3082:         Else
          ' THIRD ELEMENT
3084:           Set pProxOp = pGeometry1
3085:           dblDistance = pProxOp.ReturnDistance(pGeometry2)
3086:           pVarArray.Add dblDistance
3087:           pRevVarArray.Add dblDistance
          ' FOURTH ELEMENT   (DISTANCE)
3089:           pVarArray.Add False
3090:           pRevVarArray.Add False
          ' FIFTH ELEMENT   (BEARING)
3092:           pVarArray.Add False
3093:           pRevVarArray.Add False
3094:         End If
        
3096:         pCollection.Add pVarArray, strID
3097:         pCollection.Add pRevVarArray, strRevID

3099:       End If
3100:     Next lngIndex2
3101:   Next lngIndex
  
3103:   Set CalcDistMatrix = pCollection

3105:   Screen.MousePointer = vbDefault
3106:   If Not pApp Is Nothing Then
3107:     pPro.position = 1
3108:     psbar.HideProgressBar
3109:   End If


  Exit Function
ErrorHandler:
  HandleError True, "CalcDistMatrix " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function
Public Function CalcClosestPoints(ByVal Shape1 As IGeometry, ByVal shape2 As IGeometry, Optional intMaxCurveRepeat As Integer) As IArray
  On Error GoTo ErrorHandler


' CalcClosestPoints
' Jenness Enterprises (www.jennessent.com)
' Given two shapes, this script returns an IARRAY object containing the line connecting the closest points on each shape, plus the connection points
' CURRENTLY DOES NOT GUARANTEE SUCCESS WITH TRUE CURVES BECAUSE VERTICES ARE NOT GOOD QUERY POINTS; ATTEMPTS SEVERAL RUNS BACK AND FORTH

' Dim pRelationalOperator As IRelationalOperator
Dim pGeometryType1 As esriGeometryType
Dim pGeometryType2 As esriGeometryType
Dim pGeometry1 As IGeometry
Dim pGeometry2 As IGeometry

3131: Set pGeometry1 = Shape1
3132: Set pGeometry2 = shape2

3134: pGeometryType2 = shape2.GeometryType

' IF SHAPE #2 HAPPENS TO BE POINT, SET THAT ONE FIRST
Dim ShouldReverse As Boolean
3138: ShouldReverse = False
3139: If pGeometryType2 = esriGeometryPoint Then
3140:   Set pGeometry1 = shape2
3141:   Set pGeometry2 = Shape1
3142:   pGeometryType2 = pGeometry2.GeometryType
3143:   ShouldReverse = True
3144: End If
  
3146: pGeometryType1 = pGeometry1.GeometryType

Dim pArray As IArray
3149: Set pArray = New esriSystem.Array
Dim pOutputLine As IPolyline
3151: Set pOutputLine = New Polyline
Dim pOutputPointCollection As IPointCollection
3153: Set pOutputPointCollection = pOutputLine

Dim pStartPoint As IPoint
Dim pEndPoint As IPoint

Dim pPoint1 As IPoint
Dim pPoint2 As IPoint
3160: Set pPoint2 = New Point

Dim pProximityOp As IProximityOperator
Dim pStringArray As IStringArray

' CHECK FOR NULL SHAPES
' CHECK FOR INTERSECTING SHAPES
' NOT SURE IF THIS WILL WORK WITH MULTIPOINTS

3169: If pGeometry1.IsEmpty Or pGeometry2.IsEmpty Then
3170:   Set pStringArray = New strArray
3171:   pStringArray.Add "Empty Shapes"
3172:   pStringArray.Add CStr(pGeometry1.IsEmpty)
3173:   pStringArray.Add CStr(pGeometry2.IsEmpty)
3174:   pArray.Add pStringArray
3175:   Set CalcClosestPoints = pArray
  Exit Function
3177: Else
3178:   Set pProximityOp = pGeometry1
3179:   If pProximityOp.ReturnDistance(pGeometry2) = 0 Then
3180:     Set pStringArray = New strArray
3181:     pStringArray.Add "Intersecting Shapes"
3182:     pArray.Add pStringArray
3183:     Set CalcClosestPoints = pArray
    Exit Function
3185:   End If
3186: End If

3188: If pGeometryType1 = esriGeometryPoint Then
3189:   Set pPoint1 = pGeometry1
  
3191:   If pGeometryType2 = esriGeometryPoint Then
3192:     Set pPoint2 = pGeometry2
    
3194:     If pPoint1.X = pPoint2.X And pPoint1.Y = pPoint2.Y Then
3195:       If ShouldReverse Then
3196:         pArray.Add pOutputLine
3197:         pArray.Add pPoint2
3198:         pArray.Add pPoint1
3199:       Else
3200:         pArray.Add pOutputLine
3201:         pArray.Add pPoint1
3202:         pArray.Add pPoint2
3203:       End If
3204:     Else
3205:       If ShouldReverse Then
3206:         pOutputPointCollection.AddPoint pPoint2
3207:         pOutputPointCollection.AddPoint pPoint1
3208:         pArray.Add pOutputLine
3209:         pArray.Add pPoint2
3210:         pArray.Add pPoint1
3211:       Else
3212:         pOutputPointCollection.AddPoint pPoint1
3213:         pOutputPointCollection.AddPoint pPoint2
3214:         pArray.Add pOutputLine
3215:         pArray.Add pPoint1
3216:         pArray.Add pPoint2
3217:       End If
3218:     End If
3219:   Else
    
3221:     Set pProximityOp = pGeometry2
    
3223:     pProximityOp.QueryNearestPoint pPoint1, esriNoExtension, pPoint2
    
3225:     If ShouldReverse Then
3226:       pOutputPointCollection.AddPoint pPoint2
3227:       pOutputPointCollection.AddPoint pPoint1
3228:       pArray.Add pOutputLine
3229:       pArray.Add pPoint2
3230:       pArray.Add pPoint1
3231:     Else
3232:       pOutputPointCollection.AddPoint pPoint1
3233:       pOutputPointCollection.AddPoint pPoint2
3234:       pArray.Add pOutputLine
3235:       pArray.Add pPoint1
3236:       pArray.Add pPoint2
3237:     End If
    
    
3240:   End If
3241: Else
  Dim dblTestDistance As Double
  Dim pEnvelope As IEnvelope
  Dim pEnvelope2 As IEnvelope
3245:   Set pEnvelope = pGeometry1.Envelope
3246:   Set pEnvelope2 = pGeometry2.Envelope
3247:   pEnvelope.Union pEnvelope2
3248:   dblTestDistance = (pEnvelope.Height * pEnvelope.Width)
  Dim dblMaxDistance As Double
3250:   dblMaxDistance = dblTestDistance
  
  Dim pPointCollection1 As IPointCollection
  Dim pPointCollection2 As IPointCollection
  
3255:   If pGeometry1.GeometryType = esriGeometryEnvelope Then
    Dim pTempEnv As IEnvelope
3257:     Set pTempEnv = pGeometry1
    Dim pTempPoly1 As IPolygon
    Dim pTempPoint1 As IPoint
3260:     Set pTempPoly1 = New Polygon
3261:     Set pPointCollection1 = pTempPoly1
    Dim dXmin1 As Double
    Dim dYmin1 As Double
    Dim dXmax1 As Double
    Dim dYmax1 As Double
3266:     pTempEnv.QueryCoords dXmin1, dYmin1, dXmax1, dYmax1
3267:     Set pTempPoint1 = New Point
3268:     pTempPoint1.X = dXmin1
3269:     pTempPoint1.Y = dYmin1
3270:     pPointCollection1.AddPoint pTempPoint1
    
3272:     Set pTempPoint1 = New Point
3273:     pTempPoint1.X = dXmin1
3274:     pTempPoint1.Y = dYmax1
3275:     pPointCollection1.AddPoint pTempPoint1
    
3277:     Set pTempPoint1 = New Point
3278:     pTempPoint1.X = dXmax1
3279:     pTempPoint1.Y = dYmax1
3280:     pPointCollection1.AddPoint pTempPoint1
    
3282:     Set pTempPoint1 = New Point
3283:     pTempPoint1.X = dXmax1
3284:     pTempPoint1.Y = dYmin1
3285:     pPointCollection1.AddPoint pTempPoint1
3286:   Else
3287:     Set pPointCollection1 = pGeometry1
3288:   End If
  
3290:   If pGeometry2.GeometryType = esriGeometryEnvelope Then
    Dim pTempEnv2 As IEnvelope
3292:     Set pTempEnv2 = pGeometry2
    Dim pTempPoly2 As IPolygon
    Dim pTempPoint2 As IPoint
3295:     Set pTempPoly2 = New Polygon
3296:     Set pPointCollection2 = pTempPoly2
    Dim dXmin2 As Double
    Dim dYmin2 As Double
    Dim dXmax2 As Double
    Dim dYmax2 As Double
3301:     pTempEnv2.QueryCoords dXmin2, dYmin2, dXmax2, dYmax2
3302:     Set pTempPoint2 = New Point
3303:     pTempPoint2.X = dXmin2
3304:     pTempPoint2.Y = dYmin2
3305:     pPointCollection2.AddPoint pTempPoint2
    
3307:     Set pTempPoint2 = New Point
3308:     pTempPoint2.X = dXmin2
3309:     pTempPoint2.Y = dYmax2
3310:     pPointCollection2.AddPoint pTempPoint2
    
3312:     Set pTempPoint2 = New Point
3313:     pTempPoint2.X = dXmax2
3314:     pTempPoint2.Y = dYmax2
3315:     pPointCollection2.AddPoint pTempPoint2
    
3317:     Set pTempPoint2 = New Point
3318:     pTempPoint2.X = dXmax2
3319:     pTempPoint2.Y = dYmin2
3320:     pPointCollection2.AddPoint pTempPoint2
3321:   Else
3322:     Set pPointCollection2 = pGeometry2
3323:   End If
  
  Dim pClone As IClone
  
  Dim pVertex As IPoint
3328:   Set pVertex = New Point
  
  Dim pPointEnum As IEnumVertex
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
3334:   Set pPointEnum = pPointCollection1.EnumVertices
3335:   pPointEnum.Reset
3336:   pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
  
  ' CHECK IF CURVES; THIS CODE JUST CHECKS FIRST SEGMENT FOR CURVATURE
  Dim booWorkingWithCurves As Boolean
  Dim pSegmentCollection1 As ISegmentCollection
3341:   Set pSegmentCollection1 = pGeometry1
  Dim pSegment1 As ISegment
3343:   Set pSegment1 = pSegmentCollection1.Segment(0)
  Dim pGeometryTypeA As esriGeometryType
3345:   pGeometryTypeA = pSegment1.GeometryType
  
  Dim pSegmentCollection2 As ISegmentCollection
3348:   Set pSegmentCollection2 = pGeometry2
  Dim pSegment2 As ISegment
3350:   Set pSegment2 = pSegmentCollection2.Segment(0)
  Dim pGeometryTypeB As esriGeometryType
3352:   pGeometryTypeB = pSegment2.GeometryType
  
3354:   booWorkingWithCurves = (pGeometryTypeA = esriGeometryBezier3Curve) Or _
    (pGeometryTypeA = esriGeometryCircularArc) Or _
    (pGeometryTypeA = esriGeometryEllipticArc) Or _
    (pGeometryTypeB = esriGeometryBezier3Curve) Or _
    (pGeometryTypeB = esriGeometryCircularArc) Or _
    (pGeometryTypeB = esriGeometryEllipticArc)

3361:   Do While Not pVertex.IsEmpty
3362:     Set pProximityOp = pGeometry2
3363:     dblTestDistance = pProximityOp.ReturnDistance(pVertex)
3364:     If dblTestDistance < dblMaxDistance Then
3365:       dblMaxDistance = dblTestDistance
3366:       Set pClone = pVertex
3367:       Set pPoint1 = pClone.Clone
3368:       pProximityOp.QueryNearestPoint pVertex, esriNoExtension, pPoint2
3369:     End If
3370:     pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
3371:   Loop
  
3373:   Set pPointEnum = pPointCollection2.EnumVertices
3374:   pPointEnum.Reset
3375:   pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
  
3377:   Do While Not pVertex.IsEmpty
3378:     Set pProximityOp = pGeometry1
3379:     dblTestDistance = pProximityOp.ReturnDistance(pVertex)
3380:     If dblTestDistance < dblMaxDistance Then
3381:       dblMaxDistance = dblTestDistance
3382:       Set pClone = pVertex
3383:       Set pPoint2 = pClone.Clone
3384:       pProximityOp.QueryNearestPoint pVertex, esriNoExtension, pPoint1
3385:     End If
3386:     pPointEnum.QueryNext pVertex, lngOutPart, lngOutVertex
3387:   Loop
  
  ' FOR DEBUGGING
'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  ' IF WORKING WITH CURVES, GO BACK AND FORTH A FEW TIMES
3393:   If booWorkingWithCurves Then
    Dim intRepeat As Integer
    Dim pPoint1Temp As IPoint, pPoint2Temp As IPoint
    
3397:     Do Until (intRepeat = intMaxCurveRepeat)
 '     Graphic_MakeFromGeometry pMxDoc, pPoint1, "DeleteMe"
 '     Graphic_MakeFromGeometry pMxDoc, pPoint2, "DeleteMe"
      
3401:       Set pProximityOp = pGeometry2
3402:       pProximityOp.QueryNearestPoint pPoint1, esriNoExtension, pPoint2
      
3404:       Set pProximityOp = pGeometry1
3405:       pProximityOp.QueryNearestPoint pPoint2, esriNoExtension, pPoint1
      
3407:       intRepeat = intRepeat + 1
    
3409:     Loop
  
3411:   End If
  
3413:   If ShouldReverse Then
3414:     pOutputPointCollection.AddPoint pPoint2
3415:     pOutputPointCollection.AddPoint pPoint1
3416:     pArray.Add pOutputLine
3417:     pArray.Add pPoint2
3418:     pArray.Add pPoint1
3419:   Else
3420:     pOutputPointCollection.AddPoint pPoint1
3421:     pOutputPointCollection.AddPoint pPoint2
3422:     pArray.Add pOutputLine
3423:     pArray.Add pPoint1
3424:     pArray.Add pPoint2
3425:   End If
  
3427: End If

3429: Set CalcClosestPoints = pArray


  Exit Function
ErrorHandler:
  HandleError True, "CalcClosestPoints " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcCheckClockwise(theP As IPoint, theQ As IPoint, theR As IPoint) As Boolean
 
On Error GoTo err
' CalcCheckClockwise
' Jenness Enterprises <www.jennessent.com)>
' Given 3 consecutive points, this scripts calculates whether the third point lies to the right
' (clockwise) or to the left (counter-clockwise) of the line connecting the first point to
' the second point.

' CLOCKWISE IF TRUE
3447: CalcCheckClockwise = ((theQ.X * (theR.Y - theP.Y)) + (theQ.Y * (theP.X - theR.X)) - ((theP.X) * (theR.Y)) _
      + ((theP.Y) * (theR.X)) < 0)
    Exit Function
err:
3451:   MsgBox "Messed up CalcCheckClockwise..."
End Function

Public Function PointAdd(pPointA As IPoint, pPointB As IPoint) As IPoint
  On Error GoTo ErrorHandler


3458:   Set PointAdd = New Point
3459:   PointAdd.PutCoords pPointA.X + pPointB.X, pPointA.Y + pPointB.Y


  Exit Function
ErrorHandler:
  HandleError True, "PointAdd " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function PointSubtract(pPointA As IPoint, pPointB As IPoint) As IPoint
  On Error GoTo ErrorHandler


3471:   Set PointSubtract = New Point
3472:   PointSubtract.PutCoords pPointA.X - pPointB.X, pPointA.Y - pPointB.Y


  Exit Function
ErrorHandler:
  HandleError True, "PointSubtract " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AsRadians(theDegrees As Double) As Double
  On Error GoTo ErrorHandler


3484:   AsRadians = dblPi * (theDegrees / 180)


  Exit Function
ErrorHandler:
  HandleError True, "AsRadians " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function AsDegrees(theRadians As Double) As Double
  On Error GoTo ErrorHandler


3496:   AsDegrees = (theRadians * 180) / dblPi


  Exit Function
ErrorHandler:
  HandleError True, "AsDegrees " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub CalcPointLine(ptOrigin As IPoint, theLength As Double, dblAzimuth As Double, ptEndPoint As IPoint, _
    Optional pLine As IPolyline)
  On Error GoTo ErrorHandler


' Jenness Enterprises <www.jennessent.com>
' Given an origin point, distance and bearing, this script will return a new point at that distance and bearing, and a line
' connecting that new point to the origin point

'' MAKE SURE AZIMUTH IS BETWEEN 0 AND 360
Dim theAzimuth As Double
3515: theAzimuth = dblAzimuth

3517: Set ptEndPoint = New Point

3519: Do While theAzimuth < 0
3520:   theAzimuth = theAzimuth + 360
3521: Loop
3522: Do While theAzimuth > 360
3523:   theAzimuth = theAzimuth - 360
3524: Loop
'theAzimuth = theAzimuth Mod 360
'
'' NEW SEGMENT AND POINT DISTANCE NORTH/SOUTH AND EAST/WEST BASED ON DISTANCE AND BEARING FROM ORIGIN.
'' THERE ARE EIGHT DIFFERENT POSSIBILITIES:  THE BEARING COULD BE ONE OF THE FOUR CARDINAL DIRECTIONS OR IT
'' COULD BE IN ONE OF THE FOUR QUADRANTS.  THE BEARING IS TREATED DIFFERENTLY IN EACH OF THESE CIRCUMSTANCES.
'' THE CALCULATION TO DETERMINE THE NEW POINT LOCATION IS ESSENTIALLY A MATTER OF TAKING THE SINE OR THE
'' COSINE OF THE ANGLE (AFTER CONVERTING IT TO RADIANS), AND MULTIPLYING THAT SINE OR COSINE BY THE MEASURED
'' DISTANCE.  PLEASE CONTACT THE AUTHOR IF THIS DOESN'T MAKE SENSE, OR IF YOU WOULD LIKE FURTHER EXPLANATION.
Dim NorthSouthDistance As Double
Dim EastWestDistance As Double
Dim EastWest As Integer
Dim NorthSouth As Integer

3538: If theAzimuth = 0 Or theAzimuth = 360 Then
3539:   NorthSouthDistance = theLength
3540:   NorthSouth = 1
3541:   EastWestDistance = 0
3542:   EastWest = 1
3543: ElseIf (theAzimuth = 180) Then
3544:   NorthSouthDistance = theLength
3545:   NorthSouth = -1
3546:   EastWestDistance = 0
3547:   EastWest = 1
3548: ElseIf (theAzimuth = 90) Then
3549:   NorthSouthDistance = 0
3550:   NorthSouth = 1
3551:   EastWestDistance = theLength
3552:   EastWest = 1
3553: ElseIf (theAzimuth = 270) Then
3554:   NorthSouthDistance = 0
3555:   NorthSouth = 1
3556:   EastWestDistance = theLength
3557:   EastWest = -1
3558: ElseIf ((theAzimuth > 0) And (theAzimuth < 90)) Then
3559:   NorthSouthDistance = Cos(AsRadians(theAzimuth)) * theLength
3560:   NorthSouth = 1
3561:   EastWestDistance = Sin(AsRadians(theAzimuth)) * theLength
3562:   EastWest = 1
3563: ElseIf ((theAzimuth > 90) And (theAzimuth < 180)) Then
3564:   NorthSouthDistance = (Sin(AsRadians(theAzimuth - 90))) * theLength
3565:   NorthSouth = -1
3566:   EastWestDistance = (Cos(AsRadians(theAzimuth - 90))) * theLength
3567:   EastWest = 1
3568: ElseIf ((theAzimuth > 180) And (theAzimuth < 270)) Then
3569:   NorthSouthDistance = (Cos(AsRadians(theAzimuth - 180))) * theLength
3570:   NorthSouth = -1
3571:   EastWestDistance = (Sin(AsRadians(theAzimuth - 180))) * theLength
3572:   EastWest = -1
3573: ElseIf ((theAzimuth > 270) And (theAzimuth < 360)) Then
3574:   NorthSouthDistance = (Sin(AsRadians(theAzimuth - 270))) * theLength
3575:   NorthSouth = 1
3576:   EastWestDistance = (Cos(AsRadians(theAzimuth - 270))) * theLength
3577:   EastWest = -1
3578: End If

Dim theMovementNorth As Double
Dim theMovementWest As Double

3583: theMovementNorth = NorthSouthDistance * NorthSouth
3584: theMovementWest = EastWestDistance * EastWest

Dim startX As Double
Dim startY As Double

3589: ptOrigin.QueryCoords startX, startY
3590: ptEndPoint.PutCoords startX + theMovementWest, startY + theMovementNorth

3592: If Not pLine Is Nothing Then
  Dim pPointColl As IPointCollection
3594:   pLine.SetEmpty
3595:   Set pPointColl = pLine
3596:   pPointColl.AddPoint ptOrigin
3597:   pPointColl.AddPoint ptEndPoint
3598: End If


  Exit Sub
ErrorHandler:
  HandleError True, "CalcPointLine " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub
Public Function EllipticArcToPolygon(SegCollection As ISegmentCollection, NumVertices As Long) As IPolygon4
  On Error GoTo ErrorHandler


'  Dim pMxDoc As IMxDocument
'  Set pMxDoc = ThisDocument
  
'  Dim pEllArc As IEllipticArc
  Dim pCurve As ICurve
  Dim pGeometry As IGeometry
  
  Dim anIndex As Long
  Dim lngSegCount As Long
3618:   lngSegCount = SegCollection.SegmentCount - 1
  Dim theLength As Double
3620:   theLength = 0
  Dim theTestLength As Double
  Dim lngLengths() As Long
  ReDim lngLengths(lngSegCount)
3624:   For anIndex = 0 To lngSegCount
3625:     theTestLength = SegCollection.Segment(anIndex).length
3626:     theLength = theLength + theTestLength
3627:     lngLengths(anIndex) = theTestLength
3628:   Next anIndex
  
  Dim pProportion As Double
  Dim lngVertices() As Long
  Dim lngNumVertices As Long
  ReDim lngVertices(lngSegCount)
3634:   For anIndex = 0 To lngSegCount
3635:     lngNumVertices = Int((lngLengths(anIndex) / theLength) * NumVertices)
3636:     If lngNumVertices < 8 Then lngNumVertices = 8
3637:     lngVertices(anIndex) = lngNumVertices
3638:   Next anIndex
  
  Dim pMpt As IPointCollection
3641:   Set pMpt = New Multipoint
  Dim pPoint As IPoint
3643:   Set pPoint = New Point
  Dim pClone As IClone
  
  Dim pRatio As Double
  Dim anIndex2 As Long
  
3649:   For anIndex = 0 To lngSegCount
3650:     lngNumVertices = lngVertices(anIndex)
3651:     pRatio = 1 / lngNumVertices
3652:     Set pCurve = SegCollection.Segment(anIndex)
    
3654:     For anIndex2 = 0 To lngNumVertices
'      If pGeometry.GeometryType = esriGeometryEllipticArc Then
3656:       pCurve.QueryPoint 0, (pRatio * anIndex2), True, pPoint
3657:       Set pClone = pPoint
        
 '   Graphic_MakeFromGeometry pMxDoc, pPoint, "DeleteMe"
    
3661:       pMpt.AddPoint pClone.Clone
3662:     Next anIndex2
3663:   Next anIndex
  
  Dim pPoly4 As IPolygon4
  Dim pTopoOp2 As ITopologicalOperator2
  Dim pTopoOp3 As ITopologicalOperator3
3668:   Set pTopoOp2 = pMpt
3669:   Set pPoly4 = pTopoOp2.ConvexHull
3670:   Set pTopoOp3 = pPoly4
3671:   pTopoOp3.IsKnownSimple = False
3672:   pTopoOp3.Simplify
  
3674:   Set EllipticArcToPolygon = pPoly4


  Exit Function
ErrorHandler:
  HandleError True, "EllipticArcToPolygon " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FeaturePlanetOGraphicToPlanetOCentric(pGeometry As IGeometry, Optional dblMajorAxis As Double = -999, _
    Optional dblMinorAxis As Double = -999, Optional dblLongShift As Double = 0) As IGeometry
  On Error GoTo ErrorHandler

  


  ' ASSUMES pGeometry IS IN A GEOGRAPHIC PROJECTION AND IS EITHER A POINT, POLYLINE, POLYGON OR MULTIPOINT
  
3692:   If dblMajorAxis <= 0 Or dblMinorAxis <= 0 Then
3693:     If Not TypeOf pGeometry.SpatialReference Is IGeographicCoordinateSystem Then
3694:       MsgBox "Unexpected Spatial Reference:" & vbCrLf & _
          "The function 'FeaturePlanetOgraphicToPlanetOcentric' only accepts geometries with geographic projections." & vbCrLf & _
          "This geometry has spatial reference '" & pGeometry.SpatialReference.Name & "..."
3697:       Set FeaturePlanetOGraphicToPlanetOCentric = Nothing
3698:     End If
    Dim pGCS As IGeographicCoordinateSystem
3700:     Set pGCS = pGeometry.SpatialReference
    
    Dim pEllipsoid As ISpheroid
3703:     Set pEllipsoid = pGCS.Datum.Spheroid
    
  '  Dim dblFlattening As Double
3706:     dblMajorAxis = pEllipsoid.SemiMajorAxis
3707:     dblMinorAxis = pEllipsoid.SemiMinorAxis
  '  dblFlattening = pEllipsoid.Flattening
3709:   End If
  
  Dim pPointColl As IPointCollection
  
  Dim pOutput As IPointCollection
  Dim pPoint As IPoint
  Dim pClone As IClone
  
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  Dim dblNewLongitude As Double
  Dim dblNewLatitude As Double
  
  Dim pEnumVertex As IEnumVertex2
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
  Dim lngIndex As Long
  
  ' IF A POINT, JUST CONVERT LATITUDE AND LONGITUDE AND RETURN NEW POINT
  ' IF POLYLINE, POLYGON OR MULTIPOINT, THEN CREATE NEW IPointCollection BY CLONING ORIGINAL SHAPE.  THEN JUST
  ' ADJUST EACH POINT IN POINT COLLECTION
  
3732:   If TypeOf pGeometry Is IPoint Then
    Dim pNewPoint As IPoint
3734:     Set pNewPoint = New Point
3735:     Set pNewPoint.SpatialReference = pGeometry.SpatialReference
    
3737:     Set pPoint = pGeometry
3738:     dblLongitude = pPoint.X
3739:     dblLatitude = pPoint.Y
    
3741:     XYOGraphicToOCentric dblLongitude, dblLatitude, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3742:     pNewPoint.PutCoords dblNewLongitude, dblNewLatitude
        
3744:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewPoint
    
3746:   ElseIf TypeOf pGeometry Is IPolyline Then
    
3748:     Set pClone = pGeometry
    Dim pNewPolyline As IPolyline
3750:     Set pNewPolyline = pClone.Clone
3751:     Set pPointColl = pNewPolyline
3752:     Set pEnumVertex = pPointColl.EnumVertices
3753:     Set pPoint = New Point
    
3755:     pEnumVertex.Reset
3756:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3758:     Do While lngOutVertex > -1

3760:       XYOGraphicToOCentric pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3761:       pEnumVertex.put_Y dblNewLatitude
3762:       pEnumVertex.put_X dblNewLongitude
      
3764:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3766:     Loop
    
3768:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewPolyline
    
3770:   ElseIf TypeOf pGeometry Is IPolygon Then
    
3772:     Set pClone = pGeometry
    Dim pNewPolygon As IPolygon
3774:     Set pNewPolygon = pClone.Clone
    
3776:     Set pPointColl = pNewPolygon
3777:     Set pEnumVertex = pPointColl.EnumVertices
3778:     Set pPoint = New Point
    
3780:     pEnumVertex.Reset
3781:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3783:     Do While lngOutVertex > -1

3785:       XYOGraphicToOCentric pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3786:       pEnumVertex.put_Y dblNewLatitude
3787:       pEnumVertex.put_X dblNewLongitude
      
3789:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3791:     Loop
    
3793:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewPolygon
    
3795:   ElseIf TypeOf pGeometry Is IMultipoint Then
    
3797:     Set pClone = pGeometry
    Dim pNewMultipoint As IMultipoint
3799:     Set pNewMultipoint = pClone.Clone
3800:     Set pPointColl = pNewMultipoint
3801:     Set pEnumVertex = pPointColl.EnumVertices
3802:     Set pPoint = New Point
    
3804:     pEnumVertex.Reset
3805:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3807:     Do While lngOutVertex > -1

3809:       XYOGraphicToOCentric pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3810:       pEnumVertex.put_Y dblNewLatitude
3811:       pEnumVertex.put_X dblNewLongitude
      
3813:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3815:     Loop
    
3817:     Set FeaturePlanetOGraphicToPlanetOCentric = pNewMultipoint
  
3819:   Else
3820:     MsgBox "Unexpected Geometry Type:" & vbCrLf & _
        "The function 'PlanetOCentricToPlanetOGraphic' only accepts points, polylines, polygons or multipoints."
3822:     Set FeaturePlanetOGraphicToPlanetOCentric = Nothing
3823:   End If


  Exit Function
ErrorHandler:
  HandleError True, "FeaturePlanetOGraphicToPlanetOCentric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function


Public Function FeaturePlanetOCentricToPlanetOGraphic(pGeometry As IGeometry, Optional dblMajorAxis As Double = -999, _
    Optional dblMinorAxis As Double = -999, Optional dblLongShift As Double = 0) As IGeometry
  On Error GoTo ErrorHandler


  ' ASSUMES pGeometry IS IN A GEOGRAPHIC PROJECTION AND IS EITHER A POINT, POLYLINE, POLYGON OR MULTIPOINT
  
3839:   If dblMajorAxis <= 0 Or dblMinorAxis <= 0 Then
3840:     If Not TypeOf pGeometry.SpatialReference Is IGeographicCoordinateSystem Then
3841:       MsgBox "Unexpected Spatial Reference:" & vbCrLf & _
          "The function 'FeaturePlanetOCentricToPlanetOGraphic' only accepts geometries with geographic projections." & vbCrLf & _
          "This geometry has spatial reference '" & pGeometry.SpatialReference.Name & "..."
3844:       Set FeaturePlanetOCentricToPlanetOGraphic = Nothing
3845:     End If
    Dim pGCS As IGeographicCoordinateSystem
3847:     Set pGCS = pGeometry.SpatialReference
    
    Dim pEllipsoid As ISpheroid
3850:     Set pEllipsoid = pGCS.Datum.Spheroid
    
  '  Dim dblFlattening As Double
3853:     dblMajorAxis = pEllipsoid.SemiMajorAxis
3854:     dblMinorAxis = pEllipsoid.SemiMinorAxis
  '  dblFlattening = pEllipsoid.Flattening
3856:   End If
  
  Dim pPointColl As IPointCollection
  
  Dim pOutput As IPointCollection
  Dim pPoint As IPoint
  Dim pClone As IClone
  
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  Dim dblNewLongitude As Double
  Dim dblNewLatitude As Double
  
  Dim pEnumVertex As IEnumVertex2
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
  Dim lngIndex As Long
  
  ' IF A POINT, JUST CONVERT LATITUDE AND LONGITUDE AND RETURN NEW POINT
  ' IF POLYLINE, POLYGON OR MULTIPOINT, THEN CREATE NEW IPointCollection BY CLONING ORIGINAL SHAPE.  THEN JUST
  ' ADJUST EACH POINT IN POINT COLLECTION
  
3879:   If TypeOf pGeometry Is IPoint Then
    Dim pNewPoint As IPoint
3881:     Set pNewPoint = New Point
3882:     Set pNewPoint.SpatialReference = pGeometry.SpatialReference
    
3884:     Set pPoint = pGeometry
3885:     dblLongitude = pPoint.X
3886:     dblLatitude = pPoint.Y
    
3888:     XYOCentricToOGraphic dblLongitude, dblLatitude, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3889:     pNewPoint.PutCoords dblNewLongitude, dblNewLatitude
        
3891:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewPoint
    
3893:   ElseIf TypeOf pGeometry Is IPolyline Then
    
3895:     Set pClone = pGeometry
    Dim pNewPolyline As IPolyline
3897:     Set pNewPolyline = pClone.Clone
3898:     Set pPointColl = pNewPolyline
3899:     Set pEnumVertex = pPointColl.EnumVertices
3900:     Set pPoint = New Point
    
3902:     pEnumVertex.Reset
3903:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3905:     Do While lngOutVertex > -1

3907:       XYOCentricToOGraphic pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3908:       pEnumVertex.put_Y dblNewLatitude
3909:       pEnumVertex.put_X dblNewLongitude
      
3911:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3913:     Loop
    
3915:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewPolyline
    
3917:   ElseIf TypeOf pGeometry Is IPolygon Then
    
3919:     Set pClone = pGeometry
    Dim pNewPolygon As IPolygon
3921:     Set pNewPolygon = pClone.Clone
    
3923:     Set pPointColl = pNewPolygon
3924:     Set pEnumVertex = pPointColl.EnumVertices
3925:     Set pPoint = New Point
    
3927:     pEnumVertex.Reset
3928:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3930:     Do While lngOutVertex > -1

3932:       XYOCentricToOGraphic pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3933:       pEnumVertex.put_Y dblNewLatitude
3934:       pEnumVertex.put_X dblNewLongitude
      
3936:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3938:     Loop
    
3940:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewPolygon
    
3942:   ElseIf TypeOf pGeometry Is IMultipoint Then
    
3944:     Set pClone = pGeometry
    Dim pNewMultipoint As IMultipoint
3946:     Set pNewMultipoint = pClone.Clone
3947:     Set pPointColl = pNewMultipoint
3948:     Set pEnumVertex = pPointColl.EnumVertices
3949:     Set pPoint = New Point
    
3951:     pEnumVertex.Reset
3952:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3954:     Do While lngOutVertex > -1

3956:       XYOCentricToOGraphic pPoint.X, pPoint.Y, dblMajorAxis, dblMinorAxis, dblLongShift, dblNewLongitude, dblNewLatitude
3957:       pEnumVertex.put_Y dblNewLatitude
3958:       pEnumVertex.put_X dblNewLongitude
      
3960:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
3962:     Loop
    
3964:     Set FeaturePlanetOCentricToPlanetOGraphic = pNewMultipoint
  
3966:   Else
3967:     MsgBox "Unexpected Geometry Type:" & vbCrLf & _
        "The function 'PlanetOCentricToPlanetOGraphic' only accepts points, polylines, polygons or multipoints."
3969:     Set FeaturePlanetOCentricToPlanetOGraphic = Nothing
3970:   End If


  Exit Function
ErrorHandler:
  HandleError True, "FeaturePlanetOCentricToPlanetOGraphic " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function FeatureLongitudeShift(pGeometry As IGeometry, dblLongShift As Double) As IGeometry
  On Error GoTo ErrorHandler

  
  Dim pPointColl As IPointCollection
  
  Dim pOutput As IPointCollection
  Dim pPoint As IPoint
  Dim pClone As IClone
  
  Dim dblLongitude As Double
  Dim dblLatitude As Double
  Dim dblNewLongitude As Double
  Dim dblNewLatitude As Double
  
  Dim pEnumVertex As IEnumVertex2
  Dim lngOutPart As Long
  Dim lngOutVertex As Long
  
  Dim lngIndex As Long
  
  ' IF A POINT, JUST CONVERT LATITUDE AND LONGITUDE AND RETURN NEW POINT
  ' IF POLYLINE, POLYGON OR MULTIPOINT, THEN CREATE NEW IPointCollection BY CLONING ORIGINAL SHAPE.  THEN JUST
  ' ADJUST EACH POINT IN POINT COLLECTION
  
4003:   If TypeOf pGeometry Is IPoint Then
    Dim pNewPoint As IPoint
4005:     Set pNewPoint = New Point
4006:     Set pNewPoint.SpatialReference = pGeometry.SpatialReference
    
4008:     Set pPoint = pGeometry
4009:     dblLongitude = pPoint.X + dblLongShift
4010:     dblLatitude = pPoint.Y
    
4012:     pNewPoint.PutCoords dblLatitude, dblLongitude
        
4014:     Set FeatureLongitudeShift = pNewPoint
    
4016:   ElseIf TypeOf pGeometry Is IPolyline Then
    
4018:     Set pClone = pGeometry
    Dim pNewPolyline As IPolyline
4020:     Set pNewPolyline = pClone.Clone
4021:     Set pPointColl = pNewPolyline
4022:     Set pEnumVertex = pPointColl.EnumVertices
4023:     Set pPoint = New Point
    
4025:     pEnumVertex.Reset
4026:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4028:     Do While lngOutVertex > -1

4030:       pEnumVertex.put_Y pPoint.Y
4031:       pEnumVertex.put_X pPoint.X + dblLongShift
      
4033:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4035:     Loop
    
4037:     Set FeatureLongitudeShift = pNewPolyline
    
4039:   ElseIf TypeOf pGeometry Is IPolygon Then
    
4041:     Set pClone = pGeometry
    Dim pNewPolygon As IPolygon
4043:     Set pNewPolygon = pClone.Clone
    
4045:     Set pPointColl = pNewPolygon
4046:     Set pEnumVertex = pPointColl.EnumVertices
4047:     Set pPoint = New Point
    
4049:     pEnumVertex.Reset
4050:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4052:     Do While lngOutVertex > -1

4054:       pEnumVertex.put_Y pPoint.Y
4055:       pEnumVertex.put_X pPoint.X + dblLongShift
      
4057:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4059:     Loop
    
4061:     Set FeatureLongitudeShift = pNewPolygon
    
4063:   ElseIf TypeOf pGeometry Is IMultipoint Then
    
4065:     Set pClone = pGeometry
    Dim pNewMultipoint As IMultipoint
4067:     Set pNewMultipoint = pClone.Clone
4068:     Set pPointColl = pNewMultipoint
4069:     Set pEnumVertex = pPointColl.EnumVertices
4070:     Set pPoint = New Point
    
4072:     pEnumVertex.Reset
4073:     pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4075:     Do While lngOutVertex > -1

4077:       pEnumVertex.put_Y pPoint.Y
4078:       pEnumVertex.put_X pPoint.X + dblLongShift
      
4080:       pEnumVertex.QueryNext pPoint, lngOutPart, lngOutVertex
    
4082:     Loop
    
4084:     Set FeatureLongitudeShift = pNewMultipoint
  
4086:   Else
4087:     MsgBox "Unexpected Geometry Type:" & vbCrLf & _
        "The function 'FeatureLongitudeShift' only accepts points, polylines, polygons or multipoints."
4089:     Set FeatureLongitudeShift = Nothing
4090:   End If


  Exit Function
ErrorHandler:
  HandleError True, "FeatureLongitudeShift " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Sub XYOCentricToOGraphic(dblLongitude As Double, dblLatitude As Double, dblMajorAxis As Double, dblMinorAxis As Double, _
    dblLongitudeShift As Double, dblNewLongitude As Double, dblNewLatitude As Double)
  On Error GoTo ErrorHandler

  
  ' ORIGINAL AVENUE CODE FROM View.Ocentric2Ographic
'  theLon = pt.GetX
'  theLon = theLon + theLonShift
'
'  theLat = pt.GetY
'  theLat = theLat * Number.GetPi / 180
'  theLat = ((((theMajorAx / theMinorAx)^2) * (theLat.tan))).atan
'  theLat = theLat * (180 / Number.GetPi)

4112:   dblNewLongitude = dblLongitude + dblLongitudeShift
4113:   dblNewLatitude = AsDegrees(Atn(((dblMajorAxis / dblMinorAxis) ^ 2) * (Tan(AsRadians(dblLatitude)))))


  Exit Sub
ErrorHandler:
  HandleError True, "XYOCentricToOGraphic " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Sub XYOGraphicToOCentric(dblLongitude As Double, dblLatitude As Double, dblMajorAxis As Double, dblMinorAxis As Double, _
    dblLongitudeShift As Double, dblNewLongitude As Double, dblNewLatitude As Double)
  On Error GoTo ErrorHandler

  
  ' ORIGINAL AVENUE CODE FROM View.Ographic2Ocentric
'  theLon = pt.GetX
'  theLon = theLon + theLonShift
'
'  theLat = pt.GetY
'  theLat = theLat * Number.GetPi / 180
'  theLat = (((theLat.tan) / ((theMajorAx / theMinorAx)^2))).atan
'  theLat = theLat * (180 / Number.GetPi)


4136:   dblNewLongitude = dblLongitude + dblLongitudeShift
4137:   dblNewLatitude = AsDegrees(Atn(Tan(AsRadians(dblLatitude)) / ((dblMajorAxis / dblMinorAxis) ^ 2)))


  Exit Sub
ErrorHandler:
  HandleError True, "XYOGraphicToOCentric " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function WrapToBoundary(pGeometry As IGeometry, dblXMin As Double, dblXMax As Double, dblYMin As Double, dblYMax As Double, _
      Optional pMxDoc As IMxDocument) As IGeometry
  On Error GoTo ErrorHandler

  
  Dim dblXRange As Double
  Dim dblYRange As Double
4152:   dblXRange = dblXMax - dblXMin
4153:   dblYRange = dblYMax - dblYMin
  
  Dim dblTestX As Double
  Dim dblTestY As Double
  Dim pPoint As IPoint
  Dim pNewPoint As IPoint
  
  Dim pTopoOp As ITopologicalOperator3
  Dim pSpRef As ISpatialReference
4162:   Set pSpRef = pGeometry.SpatialReference
    
  Dim lngIndex As Long
  
4166:   If TypeOf pGeometry Is IPoint Then
    
4168:     Set pPoint = pGeometry
4169:     dblTestX = pPoint.X
4170:     dblTestY = pPoint.Y
    
4172:     Do Until dblTestX <= dblXMax
4173:       dblTestX = dblTestX - dblXRange
4174:     Loop
4175:     Do Until dblTestX >= dblXMin
4176:       dblTestX = dblTestX + dblXRange
4177:     Loop
    
4179:     Do Until dblTestY <= dblYMax
4180:       dblTestY = dblTestY - dblYRange
4181:     Loop
4182:     Do Until dblTestY >= dblYMin
4183:       dblTestY = dblTestY + dblYRange
4184:     Loop
    
4186:     Set pNewPoint = New Point
4187:     pNewPoint.PutCoords dblTestX, dblTestY
4188:     Set pNewPoint.SpatialReference = pSpRef
    
4190:     Set WrapToBoundary = pNewPoint
4191:   Else
    
    ' START BY MAKING A SET OF CLIPPING ENVELOPES COVERING THE EXTENT OF THE SHAPE
    Dim pEnvelope As IEnvelope
4195:     Set pEnvelope = pGeometry.Envelope
    
    Dim dblEnvMaxX As Double
    Dim dblEnvMinX As Double
    Dim dblEnvMaxY As Double
    Dim dblEnvMinY As Double
    
4202:     dblEnvMaxX = pEnvelope.XMax
4203:     dblEnvMinX = pEnvelope.XMin
4204:     dblEnvMaxY = pEnvelope.YMax
4205:     dblEnvMinY = pEnvelope.YMin
    
    Dim dblRunningX As Double
    Dim dblRunningY As Double
    
4210:     dblRunningX = dblXMin
4211:     dblRunningY = dblYMin
    
    Dim dblShiftX As Double
    Dim dblShiftY As Double
    Dim dblShiftBaseX As Double
    Dim dblShiftBaseY As Double
    
4218:     dblShiftBaseX = 0
4219:     dblShiftBaseY = 0
    
    Dim dblRunningBaseY As Double
    
    ' IDENTIFY BOTTOM LEFT CORNER
4224:     Do Until dblRunningX <= dblEnvMinX
4225:       dblRunningX = dblRunningX - dblXRange
4226:       dblShiftBaseX = dblShiftBaseX - dblXRange
4227:     Loop
4228:     Do Until dblRunningY <= dblEnvMinY
4229:       dblRunningY = dblRunningY - dblYRange
4230:       dblShiftBaseY = dblShiftBaseY - dblYRange
4231:     Loop
4232:     dblRunningBaseY = dblRunningY
    
    ' MAKE SET OF RECTANGULAR POLYGONS, WHERE EACH RECTANGLE IS EQUAL TO SIZE OF BOUNDARY, AND SET ENTIRELY COVERS SHAPE
    Dim pRectArray As esriSystem.IArray
4236:     Set pRectArray = New esriSystem.Array
    
    Dim pShiftArray As esriSystem.IArray
4239:     Set pShiftArray = New esriSystem.Array
    Dim pShiftSubArray As esriSystem.IDoubleArray
    
    Dim pRect As IPointCollection
    
    Dim pRelOp As IRelationalOperator
4245:     Set pRelOp = pGeometry
    
4247:     dblShiftX = dblShiftBaseX
4248:     dblShiftY = dblShiftBaseY
    
4250:     Do Until dblRunningX >= dblEnvMaxX
4251:       Do Until dblRunningY >= dblEnvMaxY
4252:         Set pRect = New Polygon
        
4254:         Set pNewPoint = New Point
4255:         pNewPoint.PutCoords dblRunningX, dblRunningY
4256:         pRect.AddPoint pNewPoint
        
4258:         Set pNewPoint = New Point
4259:         pNewPoint.PutCoords dblRunningX, dblRunningY + dblYRange
4260:         pRect.AddPoint pNewPoint
        
4262:         Set pNewPoint = New Point
4263:         pNewPoint.PutCoords dblRunningX + dblXRange, dblRunningY + dblYRange
4264:         pRect.AddPoint pNewPoint
        
4266:         Set pNewPoint = New Point
4267:         pNewPoint.PutCoords dblRunningX + dblXRange, dblRunningY
4268:         pRect.AddPoint pNewPoint
        
4270:         Set pNewPoint = New Point
4271:         pNewPoint.PutCoords dblRunningX, dblRunningY
4272:         pRect.AddPoint pNewPoint
        
4274:         Set pTopoOp = pRect
4275:         pTopoOp.Simplify
        
        ' ONLY CONSIDER RECTANGLES THAT ACTUALLY INTERSECT THE ORIGINAL SHAPE
4278:         If Not pRelOp.Disjoint(pRect) Then
4279:           pRectArray.Add pRect
4280:           Set pShiftSubArray = New esriSystem.DoubleArray
4281:           pShiftSubArray.Add dblShiftX
4282:           pShiftSubArray.Add dblShiftY
4283:           pShiftArray.Add pShiftSubArray
          
4285:         End If
4286:         dblRunningY = dblRunningY + dblYRange
4287:         dblShiftY = dblShiftY + dblYRange
4288:       Loop
4289:       dblRunningY = dblRunningBaseY
4290:       dblShiftY = dblShiftBaseY
4291:       dblRunningX = dblRunningX + dblXRange
4292:       dblShiftX = dblShiftX + dblXRange
4293:     Loop
    
    Dim pOutputColl As IGeometryCollection
4296:     Set pOutputColl = New GeometryBag
    Dim pUnionTopoOp As ITopologicalOperator
    
    Dim pRectPoly As IPolygon
    Dim pClipPointColl As IPointCollection
    
    Dim pClone As IClone
    
4304:     If TypeOf pGeometry Is IPolyline Then
      Dim pPolyline As IPolyline
4306:       Set pPolyline = pGeometry
4307:       Set pUnionTopoOp = New Polyline
      
      Dim pClipPolyline As IPolyline
4310:       For lngIndex = 0 To pRectArray.Count - 1
4311:         Set pRectPoly = pRectArray.Element(lngIndex)
4312:         Set pClone = pPolyline
4313:         Set pClipPolyline = pClone.Clone
4314:         Set pTopoOp = pClipPolyline
4315:         pTopoOp.Simplify
        
4317:         pTopoOp.Clip pRectPoly.Envelope
4318:         Set pClipPointColl = pClipPolyline
        
4320:         ShiftPointsToWrapBoundary pClipPointColl, pShiftArray.Element(lngIndex)
        
'        If Not pMxDoc Is Nothing Then
'          InvoiceCreator.MyGeneralOperations.Graphic_MakeFromGeometry pMxDoc, pClipPointColl
'        End If
4325:         Set pTopoOp = pClipPolyline
4326:         pTopoOp.Simplify
        
4328:         pOutputColl.AddGeometry pClipPolyline
4329:       Next lngIndex
      
4331:     ElseIf TypeOf pGeometry Is IPolygon Then
      Dim pPolygon As IPolygon
4333:       Set pPolygon = pGeometry
4334:       Set pUnionTopoOp = New Polygon
      
      Dim pClipPolygon As IPolygon
4337:       For lngIndex = 0 To pRectArray.Count - 1
4338:         Set pRectPoly = pRectArray.Element(lngIndex)
4339:         Set pClone = pPolygon
4340:         Set pClipPolygon = pClone.Clone
4341:         Set pTopoOp = pClipPolygon
4342:         pTopoOp.Simplify
4343:         pTopoOp.ClipDense pRectPoly.Envelope, 0.1
4344:         Set pClipPointColl = pClipPolygon
        
4346:         ShiftPointsToWrapBoundary pClipPointColl, pShiftArray.Element(lngIndex)
4347:         Set pTopoOp = pClipPolygon
4348:         pTopoOp.Simplify
        
4350:         pOutputColl.AddGeometry pClipPolygon
4351:       Next lngIndex
    
4353:     ElseIf TypeOf pGeometry Is IMultipoint Then
      Dim pMultipoint As IMultipoint
4355:       Set pMultipoint = pGeometry
4356:       Set pUnionTopoOp = New Multipoint
      
      Dim pClipMultipoint As IMultipoint
4359:       For lngIndex = 0 To pRectArray.Count - 1
4360:         Set pRectPoly = pRectArray.Element(lngIndex)
4361:         Set pClone = pMultipoint
4362:         Set pClipMultipoint = pClone.Clone
4363:         Set pTopoOp = pClipMultipoint
4364:         pTopoOp.Clip pRectPoly.Envelope
4365:         Set pClipPointColl = pClipMultipoint
        
4367:         ShiftPointsToWrapBoundary pClipPointColl, pShiftArray.Element(lngIndex)
4368:         Set pTopoOp = pClipMultipoint
4369:         pTopoOp.Simplify
        
4371:         pOutputColl.AddGeometry pClipMultipoint
4372:       Next lngIndex
      
4374:     End If
4375:     pUnionTopoOp.ConstructUnion pOutputColl
4376:     pUnionTopoOp.Simplify
4377:     Set WrapToBoundary = pUnionTopoOp
4378:   End If


  Exit Function
ErrorHandler:
  HandleError True, "WrapToBoundary " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Private Sub ShiftPointsToWrapBoundary(pPointColl As IPointCollection, pShiftSubArray As esriSystem.IDoubleArray)
  On Error GoTo ErrorHandler

  
  ' THIS FUNCTION ASSUMES THAT SHAPE HAS ALREADY BEEN CLIPPED TO RECTANGLE REPRESENTING WRAP BOUNDARY INCREMENTS
  
  Dim pTransform As ITransform2D
4393:   Set pTransform = pPointColl
4394:   pTransform.Move -pShiftSubArray.Element(0), -pShiftSubArray.Element(1)


  Exit Sub
ErrorHandler:
  HandleError False, "ShiftPointsToWrapBoundary " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Sub

Public Function HSin(dblRadians As Double) As Double
  On Error GoTo ErrorHandler


4406:   HSin = (Exp(dblRadians) - Exp(-dblRadians)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "HSin " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HCos(dblRadians As Double) As Double
  On Error GoTo ErrorHandler


4418:   HCos = (Exp(dblRadians) + Exp(-dblRadians)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "HCos " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HTan(dblRadians As Double) As Double
  On Error GoTo ErrorHandler


4430:   HTan = (Exp(dblRadians) - Exp(-dblRadians)) / (Exp(dblRadians) + Exp(-dblRadians))


  Exit Function
ErrorHandler:
  HandleError True, "HTan " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HArcSin(dblValue As Double) As Double
  On Error GoTo ErrorHandler


4442:   HArcSin = Log(dblValue + Sqr(dblValue * dblValue + 1))


  Exit Function
ErrorHandler:
  HandleError True, "HArcSin " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HArcCos(dblValue As Double) As Double
  On Error GoTo ErrorHandler


4454:   HArcCos = Log(dblValue + Sqr(dblValue * dblValue - 1))


  Exit Function
ErrorHandler:
  HandleError True, "HArcCos " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function HArcTan(dblValue As Double) As Double
  On Error GoTo ErrorHandler


4466:   HArcTan = Log((1 + dblValue) / (1 - dblValue)) / 2


  Exit Function
ErrorHandler:
  HandleError True, "HArcTan " & c_sModuleFileName & " " & GetErrorLineNumberString(Erl), err.Number, err.Source, err.Description, 4
End Function

Public Function CalcInternalAngle(theP As IPoint, theQ As IPoint, theR As IPoint, Optional dblAngleDev As Double) As Double
 
On Error GoTo err
' CalcCheckClockwise
' Jenness Enterprises <www.jennessent.com)>
' Given 3 consecutive points, this scripts calculates the internal angle

' INTERNAL ANGLE WITH LAW OF COSINES;
'       c^2 = a^2 + b^2 - (2ab * Cos C), OR
'       Cos C = (a^2 +b^2 - c^2)/(2ab)

  Dim dblLenPQ As Double
  Dim dblLenQR As Double
  Dim dblLenRP As Double
  
4489:   dblLenPQ = ((theP.X - theQ.X) ^ 2 + (theP.Y - theQ.Y) ^ 2) ^ (0.5)
4490:   dblLenQR = ((theQ.X - theR.X) ^ 2 + (theQ.Y - theR.Y) ^ 2) ^ (0.5)
4491:   dblLenRP = ((theR.X - theP.X) ^ 2 + (theR.Y - theP.Y) ^ 2) ^ (0.5)
  
4493:   CalcInternalAngle = (((dblLenPQ ^ 2) + (dblLenQR ^ 2) - (dblLenRP ^ 2)) / (2 * dblLenPQ * dblLenQR))
4494:   CalcInternalAngle = ArcCosJen(CalcInternalAngle)
4495:   CalcInternalAngle = Round(RadToDeg(CalcInternalAngle), 10)
4496:   dblAngleDev = 180 - CalcInternalAngle

    Exit Function
err:
  Dim dblbearing1 As Double
  Dim dblBearing2 As Double
4502:   dblbearing1 = Round(CalcBearing(theP, theQ))
4503:   dblBearing2 = Round(CalcBearing(theQ, theR))
4504:   If (dblbearing1 = dblBearing2) Then
4505:     CalcInternalAngle = 180
4506:     dblAngleDev = 0
4507:   Else
4508:     CalcInternalAngle = 0
4509:     dblAngleDev = 180
4510:   End If

End Function
